/******************************************************************/
/* accent.x -- Help with accents      Toby Thurston - 28 Jul 1997 */
/******************************************************************/
/* This macro is for those with keyboards lacking accent chars,   */
/* yet who occasionally need to type words with accents in them.  */
/*                                                                */
/* Assign this macro to a key (I use alt-e).  Recompile XPROFILE. */
/*                                                                */
/* Press the key with the cursor on blank: e-acute is entered,    */
/* and the cursor is moved one place to the right.                */
/*                                                                */
/* Press the key with the cursor on 'e': e-acute is entered,      */
/* and the cursor is not moved.  Repeated presses of the key will */
/* cycle through e-grave, e-hat, e-umlaut, e, e-acute again etc.  */
/* Press the key with the cursor on 'a', 'i', 'o' or 'u' and the  */
/* same cycle is repeated for those letters.                      */
/*                                                                */
/* Other characters (for Spanish):  c goes to c-cedilla & back,   */
/* n to n-tilde, ? to ?-inverted and ! to !-inverted.             */
/*                                                                */
/* If you press the key with the cursor on anything else, then    */
/* nothing happens.  If you want accented upper case letters,     */
/* start with a lower case letter and then press Ctrl-Up when you */
/* finish.                                                        */
/*                                                                */
/* As shipped this macro works with codepages 437 and 850.        */
/******************************************************************/
/* You can extend the functions for yourself by adding new chains */
/* or adding more chars to a chain, for example you might add     */
/* '86'x to the 'a' chain, in order to get a-ring.  You may also  */
/* care to alter the order of the chains, for example you might   */
/* put '94'x directly after 'o' to have o-umlaut first.  You      */
/* could also remove characters you never use.                    */
/******************************************************************/
next. = '00'x                          /* set up the data         */
default = '82'x                        /* set a default character */
/* vowels...   acute grave hat  umlaut    ring                    */
call chain 'a' 'A0'x '85'x '83'x '84'x /* '86'x                   */
call chain 'e' '82'x '8A'x '88'x '89'x
call chain 'i' 'A1'x '8D'x '8C'x '8B'x
call chain 'o' 'A2'x '95'x '93'x '94'x
call chain 'u' 'A3'x '97'x '96'x '81'x
/* others....                                                     */
call chain 'c' '87'x                   /* c -> c-cedilla          */
call chain 'n' 'A4'x                   /* n -> n-tilda            */
call chain '?' 'A8'x                   /* invert ?                */
call chain '!' 'AD'x                   /* invert !                */
'extract /cursor/'                     /* get the current char    */
'extract /line/'
char = substr(line.1,cursor.2,1)
if char = ''
  then 'keyin' default                 /* enter default if blank  */
else if next.char <> '00'x
  then do
    'keyin' next.char
    'cursor +0 -1'
  end
exit
chain: procedure expose next.          /* proc to make a chain    */
  parse arg inchars
  do i = 1 to words(inchars) - 1
    char = word(inchars,i)
    next.char = word(inchars,i+1)
  end
  char = word(inchars,i)  /* note i is incremented at end of loop */
  next.char = word(inchars,1)  /* set last char to point at first */
return
