/***************************************************************************/
/*                                                                         */
/* Allow box drawing in the editor.  On initial invocation, saves the      */
/* environment and sets up PF keys to call this macro with certain         */
/* parameters.  After that, draws boxes or restores the environment        */
/* depending on the parameter.  Boxes can only be drawn in the file which  */
/* was current at the time of invocation.                                  */
/*                                                                         */
/* Requires version 1.43 or later of the editor.  The box characters are   */
/* defined at the top of this macro for codepage 437.  They may need to be */
/* tailored for a different codepage.                                      */
/*                                                                         */
/* Written by B. Thompson, June 24, 1994                                   */
/*                                                                         */
/* Modified by Patrick Leabo, September 27, 1994                           */
/*                                                                         */
/*  Added vertical and horizontal line routines to intersect               */
/*    a little nicer at the end points                                     */
/*  Allowed line corners - If the origin is set to the last draw point.    */
/*    a corner is drawn towards the next draw point                        */
/*  Added line styles arg to boxes  invoke with "boxes n"                  */
/*     noarg=1=  2=  3=  4=  5=+  6=                                  */
/*  It still surprises sometimes, but you can repair the 'surprises' ;-)   */
/*  Known Problems:                                                        */
/*   On first invocation of boxes, the help MSG doesn't show - OK after 1st*/
/*   Occasionally gets stuck where F3 won't exit boxes mode                */
/*                                                                         */
/* Use extract /key n/ for version 1.90, August 13, 1997                   */
/* Check operating system, October 14, 1998                                */
/* Use hidden file to store vars instead of env vars, February 22, 1999    */
/*                                                                         */
/***************************************************************************/

  Parse Upper Arg parm .
  Parse Upper Source os . myname '.X' .

  Call get_globals
  nocharparm=TRUE                                /* Help flag - No char arg*/
  If 0<pos(parm,'123456')                   /* calling other line char set?*/
    Then Do
      g._chset = parm
      parm = ''                               /* let it initialize normally*/
      nocharparm=FALSE                          /* Help flag - was char arg*/
    End
    Else Do
      If 0=wordpos(parm,'$ORIGIN $DRAW $END')                  /* not F_Key*/
        Then g._chset = 1                               /* Set symbol set 1*/
    End

  set=(g._chset-1)*11

  all6= 'B3'x||'C4'x||'DA'x||'C2'x||'BF'x||'C3'x||,          /* ¿Ŵ*/
        'C5'x||'B4'x||'C0'x||'C1'x||'D9'x||,
        'BA'x||'CD'x||'C9'x||'CB'x||'BB'x||'CC'x||,          /* ˻ιʼ*/
        'CE'x||'B9'x||'C8'x||'CA'x||'BC'x||,
        'B3'x||'CD'x||'D5'x||'D1'x||'B8'x||'C6'x||,          /* ѸصϾ*/
        'D8'x||'B5'x||'D4'x||'CF'x||'BE'x||,
        'BA'x||'C4'x||'D6'x||'D2'x||'B7'x||'C7'x||,          /* ҷ׶н*/
        'D7'x||'B6'x||'D3'x||'D0'x||'BD'x||,
        '7C'x||'2D'x||'2B'x||'2B'x||'2B'x||'2B'x||,          /* |-+++++++++*/
        '2B'x||'2B'x||'2B'x||'2B'x||'2B'x||,
        'DB'x||'DF'x||'DB'x||'DB'x||'DB'x||'DB'x||,          /* */
        'DB'x||'DB'x||'DF'x||'DF'x||'DF'x

  box._v  = Substr(all6,set+1,1)                            /* Vertical bar*/
  box._h  = Substr(all6,set+2,1)                          /* Horizontal bar*/
  box._ul = Substr(all6,set+3,1)                       /* Upper left corner*/
  box._td = Substr(all6,set+4,1)         /* Top Tee junction(pointing down)*/
  box._ur = Substr(all6,set+5,1)                      /* Upper right corner*/
  box._tr = Substr(all6,set+6,1)      /* Right Tee junction(pointing right)*/
  box._tx = Substr(all6,set+7,1)                  /* Centre cross character*/
  box._tl = Substr(all6,set+8,1)        /* Left Tee junction(pointing left)*/
  box._ll = Substr(all6,set+9,1)                       /* Lower left corner*/
  box._tu = Substr(all6,set+10,1)      /* Bottom Tee junction (pointing up)*/
  box._lr = Substr(all6,set+11,1)                     /* Lower right corner*/

  'EXTRACT /CURSOR/'                                /* Get current position*/
  'EXTRACT /NAME/'                                  /* Get current filename*/
  samefile = (name.1 = g._origname)

  Select
    When parm = '$ORIGIN'
      Then Do                                         /* Set the new origin*/
        If samefile
          Then Do
            Call clear_origin             /* Clear possible previous origin*/
            g._origin = cursor.1 cursor.2
            'EXTRACT /CURLINE/'
            If Substr(curline.1,cursor.2,1) = ' '
              Then Do                          /* Mark the current position*/
                outline = Overlay('*',curline.1,cursor.2)
                'REPLACE' outline
              End
            If g._origin = g._ending    /* See if origin at last draw point*/
              Then                                   /* Set the Eureka flag*/
                g._bingo = TRUE
              Else
                g._bingo = FALSE
            'MSG Origin is now at' g._origin
          End
      End
    When parm = '$DRAW'
      Then Do                                         /* Draw a box or line*/
        If samefile
          Then Call draw
      End
    When parm = '$END'
      Then Do                               /* Restore original environment*/
        Call clear_origin                 /* Clear possible previous origin*/
        'KEY F1 =' g._f1
        'KEY F2 =' g._f2
        'KEY F3 =' g._f3
        'PFLINE' g._pfline
      End
    Otherwise Do                               /* Assume initial invocation*/
      'EXTRACT /KEY F1/'
      g._f1 = key.1
      'EXTRACT /KEY F2/'
      g._f2 = key.1
      'EXTRACT /KEY F3/'
      g._f3 = key.1
      'EXTRACT /PFLINE/'
      g._pfline = pfline.1
      'EXTRACT /FILENAME/'
      g._origname = filename.1

      'KEY F1 = MACRO' myname '$ORIGIN'
      'KEY F2 = MACRO' myname '$DRAW'
      'KEY F3 = MACRO' myname '$END'
      'PFLINE Boxes:  F1=Set Origin  F2=Draw Box  F3=End Boxes'
      call DispHelp nocharparm
    End
  End                                                         /* End Select*/

  Call set_globals
  'CURSOR' cursor.1 cursor.2                     /* Restore cursor position*/
  'REFRESH'                                     /* Ensure screen is redrawn*/
Exit


/***************************************************************************/
/*                                                                         */
/* Draw a box between the saved ORIGIN and the current cursor position.    */
/* We set up a translate table for the possible intersections of various   */
/* box characters.  Then all we have to do is look up the target character */
/* and the existing character in the table to find out which character to  */
/* write on the screen.                                                    */
/*                                                                         */
/***************************************************************************/

DRAW:  Procedure Expose box. cursor. corner g.
  Parse Value g._origin With orow ocol .
  If ocol = ''
    Then Do                                  /* Origin has not yet been set*/
      'MACRO' myname '$ORIGIN'
      Return
    End

  boxchars = box._v box._h box._ul box._td box._ur box._tr,
    box._tx box._tl box._ll box._tu box._lr               /* Box characters*/

  otable.1 = box._v box._tx box._tr box._tx box._tl box._tr,
    box._tx box._tl box._tr box._tx box._tl                 /* Vertical bar*/
  otable.2 = box._tx box._h box._td box._td box._td box._tx,
    box._tx box._tx box._tu box._tu box._tu               /* Horizontal bar*/
  otable.3 = box._tr box._td box._ul box._td box._td box._tr,
    box._tx box._tx box._tr box._tx box._tx            /* Upper left corner*/
  otable.4 = box._tx box._td box._td box._td box._td box._td,
    box._tx box._tx box._tx box._tx box._tx       /* Downwards Tee (unused)*/
  otable.5 = box._tl box._td box._td box._td box._ur box._tx,
    box._tx box._tl box._tx box._tx box._tl           /* Upper right corner*/
  otable.6 = box._tr box._tx box._tr box._tx box._tx box._tr,
    box._tx box._tx box._tr box._tx box._tx           /* Right Tee (unused)*/
  otable.7 = box._tx box._tx box._tx box._tx box._tx box._tx,
    box._tx box._tx box._tx box._tx box._tx        /* Centre cross (unused)*/
  otable.8 = box._tl box._tl box._tx box._tx box._tl box._tx,
    box._tx box._tl box._tx box._tx box._tl            /* Left Tee (unused)*/
  otable.9 = box._tr box._tu box._tr box._tx box._tx box._tr,
    box._tx box._tx box._ll box._tu box._tu            /* Lower left corner*/
  otable.10= box._tx box._tu box._tx box._tx box._tx box._tx,
    box._tx box._tx box._tu box._tu box._tu         /* Upwards Tee (unused)*/
  otable.11= box._tl box._tu box._tx box._tx box._tl box._tx,
    box._tx box._tl box._tu box._tu box._lr           /* Lower right corner*/
  /*        Line Ending masks            */
  otable.12= box._ul box._ur                     /* vertical top corner  */
  otable.13= box._ul box._td box._ur        /* vertical top intersect   */
  otable.14= box._ll box._lr                     /* vertical bot corner  */
  otable.15= box._ll box._tu box._lr        /* vertical bot intersect   */

  otable.16= box._ll box._ul                       /* horiz left corner  */
  otable.17= box._ul box._ll box._tr          /* horiz left intersect   */
  otable.18= box._ur box._lr                      /* horiz right corner  */
  otable.19= box._ur box._lr box._tl         /* horiz right intersect   */

  toprow   = Min(orow, cursor.1)
  lastrow  = Max(orow, cursor.1)
  leftcol  = Min(ocol, cursor.2)
  rightcol = Max(ocol, cursor.2)
  g._ending = cursor.1 cursor.2

  Select
    When toprow = lastrow
      Then Do                                /* Just draw a horizontal line*/
        'EXTRACT /CURLINE/'
        If cursor.2>leftcol                          /* Determine direction*/
          Then dir = 'R'
          Else dir = 'L'
        corner = g._lastdir||dir                                    /* Doit*/
        'REPLACE' horizontal(curline.1, leftcol, rightcol, g._bingo, corner)
        If cursor.2>ocol                             /* Save last direction*/
          Then g._lastdir = 'R'
          Else g._lastdir = 'L'
      End
    When leftcol = rightcol
      Then Do                                  /* Just draw a vertical line*/
        If cursor.1>toprow
          Then dir = 'D'                             /* Determine direction*/
          Else dir = 'U'
        corner = g._lastdir||dir
        outline=vertical(leftcol, toprow, lastrow, g._bingo, corner)/* doit*/
        If cursor.1>orow
          Then g._lastdir = 'D'                 /* Save last line direction*/
          Else g._lastdir = 'U'
      End
    Otherwise Do row = toprow To lastrow                      /* Draw a Box*/
      'CURSOR' row
      'EXTRACT /CURLINE/'
      g._lastdir = ' '                                /* Nul last direction*/
      Select
        When leftcol = rightcol
          Then outline = replacechar(curline.1, leftcol, box._v)     /* Top*/
        When row = toprow
          Then Do
            outline = ohorizontal(curline.1, leftcol+1, rightcol-1)
            outline = replacechar(outline, leftcol, box._ul)
            outline = replacechar(outline, rightcol, box._ur)
          End
        When row = lastrow
          Then Do
            outline = ohorizontal(curline.1,leftcol+1, rightcol-1)/* Bottom*/
            outline = replacechar(outline, leftcol, box._ll)
            outline = replacechar(outline, rightcol, box._lr)
          End
        Otherwise Do
          outline = replacechar(curline.1, leftcol, box._v)        /* Sides*/
          outline = replacechar(outline, rightcol, box._v)
        End
      End                                                     /* End select*/
      'REPLACE' outline
    End                                                           /* End do*/
 End                                                          /* End select*/
 'MSG Ending is now at' g._ending
Return

/***************************************************************************/
/* Draw a single horizontal line between the specified co-ordinates. (orig)*/
/***************************************************************************/

OHORIZONTAL:  Procedure Expose box. boxchars otable.
  Parse Arg inline, leftcol, rightcol
  Do col = leftcol To rightcol
    inline = replacechar(inline,col,box._h)
  End                                                             /* End do*/
Return inline

/***************************************************************************/
/*                                                                         */
/* Replace the specified character on the input line with the target       */
/* character.  If the existing character is a box character, we will want  */
/* to use our translate table to modify the target character.              */
/*                                                                         */
/***************************************************************************/

REPLACECHAR:  Procedure Expose box. boxchars otable.
  Parse Arg inline, col, newchar
  char = Substr(inline,col,1)
  charpos = wordpos(char,boxchars)
  newcharpos = wordpos(newchar,boxchars)
  If charpos > 0
    Then newchar = Word(otable.newcharpos, charpos)
Return Overlay(newchar,inline,col)

/***************************************************************************/
/*                                                                         */
/* We may have marked the origin in the file with an asterisk character.   */
/* If this character is still there, we want to restore it to the blank    */
/* that was there previously.                                              */
/*                                                                         */
/***************************************************************************/

CLEAR_ORIGIN:  Procedure Expose cursor. g.
  Parse Value g._origin With orow ocol .
  If ocol <> ''
    Then Do                      /* Possibly need to restore origin pointer*/
      'CURSOR' orow
      'EXTRACT /CURLINE/'
      If Substr(curline.1,ocol,1) = '*'
        Then Do                                     /* Replace with a blank*/
          outline = Overlay(' ',curline.1,ocol)
          'REPLACE' outline
        End
    End
  'CURSOR' cursor.1 cursor.2                     /* Restore cursor position*/
Return

/***************************************************************************/
/* Draw a single vertical line between the specified co-ordinates. PL      */
/***************************************************************************/

VERTICAL:  Procedure Expose box. boxchars otable.
  Parse Arg mycol, toprow, lastrow, bingo, corner
  'CURSOR' toprow
  'EXTRACT /CURLINE/'
  inline=curline.1
  char = Substr(inline,mycol,1)
  If bingo=TRUE & 0<wordpos(corner,'LD RD')         /* Top of vertical line*/
    Then do                                           /* Fix corner turning*/
      cornerchar=word(otable.12,wordpos(corner,'LD RD'))
      outline=overlay(cornerchar,inline,mycol)
    End                                                           /* End do*/
    Else do
      Select                                         /* Or Fix line endings*/
        When char=box._h
          Then outline = Overlay(box._td,inline, mycol)     /* intersect it*/
        When (wordpos(char,otable.13)>0)
          Then outline=inline                                 /* do nothing*/
        Otherwise outline = replacechar(inline, mycol, box._v)    /* normal*/
      End                                                     /* End select*/
    End                                                           /* End do*/
  'REPLACE' outline
  Do row = toprow+1 To lastrow-1  /* Draw vertical line (except end points)*/
    'CURSOR' row
    'EXTRACT /CURLINE/'
    outline = replacechar(curline.1, mycol, box._v)
    'REPLACE' outline
  End                                                             /* End do*/
  'CURSOR' lastrow
  'EXTRACT /CURLINE/'
  inline=curline.1
  char = Substr(inline,mycol,1)
  If bingo=TRUE & 0<wordpos(corner,'LU RU')      /* Bottom of vertical line*/
    Then Do                                           /* Fix corner turning*/
      cornerchar=word(otable.14,wordpos(corner,'LU RU'))
      outline=overlay(cornerchar,inline,mycol)
    End                                                           /* End do*/
    Else Do
      Select                                         /* Or Fix line endings*/
        When char=box._h
          Then outline = Overlay(box._tu,inline, mycol)
        When (wordpos(char,otable.15)>0)
          Then outline=inline                                  /* No Change*/
        Otherwise outline = replacechar(inline, mycol, box._v)
      End                                                     /* End select*/
    End                                                           /* End do*/
  'REPLACE' outline
Return outline

/***************************************************************************/
/* Draw a single horizontal line between the specified co-ordinates. PL    */
/***************************************************************************/

HORIZONTAL:  Procedure Expose box. boxchars otable.
  Parse Arg inline, leftcol, rightcol, bingo, corner
  char = Substr(inline,leftcol,1)
  If bingo=TRUE & 0<wordpos(corner,'DR UR')       /* Left end of horiz line*/
    Then Do                                           /* Fix corner turning*/
      cornerchar=word(otable.16,wordpos(corner,'DR UR'))
      inline=overlay(cornerchar,inline,leftcol)
    End                                                           /* End do*/
    Else Do
      Select                                         /* Or Fix line endings*/
        When char=box._v
          Then inline = Overlay(box._tr,inline, leftcol)    /* intersect it*/
        When (wordpos(char,otable.17)>0)
          Then a=a                                            /* do nothing*/
        Otherwise
          inline = replacechar(inline, leftcol, box._h)           /* normal*/
      End                                                     /* End Select*/
    End                                                           /* End do*/
  Do col = leftcol+1 To rightcol-1                       /* Draw horiz line*/
    inline = replacechar(inline,col,box._h)
  End                                                             /* End do*/
  char = Substr(inline,rightcol,1)
  If bingo=TRUE & 0<wordpos(corner,'UL DL')      /* Right end of horiz line*/
    Then Do                                           /* Fix corner turning*/
      cornerchar=word(otable.18,wordpos(corner,'UL DL'))
      inline=overlay(cornerchar,inline, rightcol)
    End                                                           /* End do*/
    Else Do
      Select                                         /* Or Fix line endings*/
        When char=box._v
          Then inline = Overlay(box._tl,inline, rightcol)   /* intersect it*/
        When (wordpos(char,otable.19)>0)                      /* do nothing*/
          Then a=a
        Otherwise inline = replacechar(inline, rightcol,box._h)   /* normal*/
      End                                                     /* End Select*/
    End                                                           /* End do*/
Return inline


/***************************************************************************/
/* Show some help on the command line on initial invocation          PL    */
/***************************************************************************/

DispHelp: Procedure Expose box. all6
  Parse arg nocharparm rest
  If nocharparm = TRUE                                /* Show a little help*/
    Then Do
      msg='Set to Draw '||Substr(all6, 6,1)||'    "boxes n" will set'||,
      ' 1='||Substr(all6, 6,1)||,
      ' 2='||Substr(all6,17,1)||,
      ' 3='||Substr(all6,28,1)||,
      ' 4='||Substr(all6,39,1)||,
      ' 5='||Substr(all6,50,1)||,
      ' 6='||Substr(all6,61,1)
      'MSG' msg
    End
  Else If nocharparm = FALSE
    Then 'MSG Set to Draw' box._tr             /* No help, Just information*/
Return


/***************************************************************************/
/*                                                                         */
/* Store global information in a hidden file in the ring.  More portable   */
/* than the previous method of storing everything in OS/2 environment      */
/* variables.                                                              */
/*                                                                         */
/* Written by B. Thompson, February 22, 1999                               */
/*                                                                         */
/***************************************************************************/

SET_GLOBALS:  Procedure Expose g.
  'EXTRACT /FILENAME/'
  'EDIT $BOXES.$$$'
  'HIDEFILE ON'

  Do i = 1 To words(g._varnames)
    varname = word(g._varnames,i)
    '0'
    'LOCATE /'varname'/ q'
    If rc = 0
      Then 'REPLACE' varname '=' Value(g.varname)
      Else 'INPUT' varname '=' Value(g.varname)
    'RESTORE_FIND'
  End                                                             /* End do*/

  'EDIT' filename.1                                /* Back where we started*/
Return


/***************************************************************************/
/*                                                                         */
/* Get global information from a hidden file in the ring.                  */
/*                                                                         */
/* Written by B. Thompson, February 22, 1999                               */
/*                                                                         */
/***************************************************************************/

GET_GLOBALS:  Procedure Expose g.
  g._varnames = '_f1 _f2 _f3 _pfline _origname _origin',
                '_ending _chset _bingo _lastdir'   /* Global variable names*/
  'EXTRACT /FILENAME/'
  'EDIT $BOXES.$$$'
  'HIDEFILE ON'

  Do i = 1 To words(g._varnames)
    varname = word(g._varnames,i)
    '0'
    'LOCATE /'varname'/ q'
    If rc = 0
      Then Do
        'EXTRACT /CURLINE/'
        Parse Var curline.1 . '=' varvalue
        Interpret 'g.'varname '= varvalue'
      End                                                         /* End do*/
    'RESTORE_FIND'
  End                                                             /* End do*/

  'EDIT' filename.1                                /* Back where we started*/
Return
