/* Backslash = '\' */

/* BTAGS - Interface to Exuberant C tags for X2 */

/* ??START - End of verbose help */
/***************************************************************************/
/*                                                                         */
/*\eBBTAGS.X - Version 1.3 - 4/12/2000                                     */
/*                                                                         */
/* Look for the token at the cursor position, and jump to its definition.  */
/* If more than one match is found, a popup window is displayed with all   */
/* matches and the symbol types.                                           */
/*                                                                         */
/* Uses output from the Exuberant CTAGS program, which is available from   */
/* http://darren.hiebert.com/ctags/.                                       */
/*                                                                         */
/*\eETags file                                                             */
/*                                                                         */
/* To generate the TAGS file (must be in the current directory) issue the  */
/* CTAGS command with a list of files to be parsed, e.g.                   */
/*                                                                         */
/*   ctags <options> *                                                     */
/*                                                                         */
/* You may use the --excmd=number option to generate line numbers, or use  */
/* the default which will generate a search string to use to find the      */
/* definition. The -R option can be used to tag an entire source tree.     */
/* Since I like prototypes tagged as well, I use:                          */
/*                                                                         */
/*   ctags -R --c-types=+px --totals=yes *                                 */
/*                                                                         */
/* Give full path names to ctags or use the '-p' option to generate tags   */
/* files that have fully qualified path names.                             */
/*                                                                         */
/*\eESearching for tags                                                    */
/*                                                                         */
/* To look for the token, position the cursor anywhere on the token and    */
/* press the key assigned to this macro, or call it from the command line. */
/* If called from the command line without parameters, it will look for    */
/* token at the cursor positon. If given a parameter, it will search for   */
/* a token of that name. Run "btags ?" for online help.                    */
/*                                                                         */
/* By default searches are case sensitive and must match the entire token. */
/* This can be changed by editing the macro or via the -C option. In       */
/* addition, if the token is passed on the command line, a '*' can be      */
/* placed at the beginning and/or end of the token to allow partial        */
/* matches to be done. Wildcard searches are not case sensitive by default */
/*                                                                         */
/* Tags file should be placed in the directory where it is created because */
/* the directory names in it are relative. The tagFile and tagFileDir      */
/* variables below can be modified to reflect its location. However, if    */
/* the file isn't found and the searchCurrentTree flag is set, the parent  */
/* directories are searched as well, allowing you to create different tags */
/* files for different source trees. The directory name where the tags     */
/* file is prepended to any filenames in the tags file that start with     */
/* the '.' directory.                                                      */
/*                                                                         */
/* Both the name of the tags file and the directory can be specified by    */
/* command line options. I set the macro in my profile:                    */
/*                                                                         */
/*     key c-Q = macro btags -PX,c=1 -Fd:\tags                             */
/*                                                                         */
/*\eESearch Procedures                                                     */
/*                                                                         */
/* Version 1.3 introduced different search methods. The default method     */
/* (X) uses the X editor to search for matches in the tags files. While    */
/* this is fairly quick on small files since no additional program must    */
/* be run, it isn't so quick as files get bigger. By default, tag files    */
/* are cached in memory to speed searches. The bigger the tag file the     */
/* more improvement, but also the more memory taken up. To disable         */
/* caching, set the cacheTagFile flag to 0 in the source code or pass      */
/* the parameter "-PX,c=0"                                                 */
/*                                                                         */
/* An additional search procedure (Grep) was added that uses the grep      */
/* program and passes results back via rxqueue. To use it, pass btags      */
/* the -pgrep flag (or modify searchProcName in the macro code). Most      */
/* flavors of grep based on the Unix/GNU versions should work. The search  */
/* procedure is fairly easily tweakable otherwise.                         */
/*                                                                         */
/* See the source for information on writing your own search routines.     */
/*                                                                         */
/*\eEOriginal TAGS.X version \eB(Blair Thompson, January 22, 1999)         */
/*                                                                         */
/*\eEInitial BTAGS version\eB(Buzzy Brown, 23 Feb 2000)                    */
/*   - Add support for multiple matches and file selection window          */
/*   - Add support for wildcards and case-insensitive searches             */
/*                                                                         */
/*\eEChanges in 1.1 \eB(Buzzy Brown, 23 Feb 2000)                          */
/*   - Moved percentages for popup window to setup section                 */
/*   - Added sortByType config option                                      */
/*                                                                         */
/*\eEChanges in 1.2 \eB(Buzzy Brown, 27 Feb 2000)                          */
/*   - Add msg when positioning cursor after single match                  */
/*   - Handle tags file in root directory                                  */
/*   - Add includeFullDir option                                           */
/*   - Update column sizing                                                */
/*   - Add some parameters (-F -S -C)                                      */
/*   - Fixed non-exact case non-wildcard matching                          */
/*   - Added online help                                                   */
/*   - Fixed extension location for filenames containing '.'               */
/*                                                                         */
/*\eEChanges in 1.2b \eB(Wesley Miller, 10 Mar 2000)                       */
/*   - Allowed '/' to start options in addition to '-'                     */
/*   - Shorten parameters to one character                                 */
/*   - Add -d option                                                       */
/*   - Allow options to be case insensitive                                */
/*   - Add support for -?? (verbose help)                                  */
/*                                                                         */
/*\eEChanges in 1.3 \eB(Buzzy Brown, 12 Apr 2000)                          */
/*   - Restructure to allow drop in of new search procedures and -P option */
/*   - Added search code that uses grep (SearchForHits_Grep)               */
/*   - Fix bugs where filenames/paths start with n (i.e. path contains \n) */
/*   - Fix bugs with relative paths in tag file and/or explict tag file    */
/*   - Add cacheTagFile (C) option to X search procedure.                  */
/*   - Add match coloring (highlightMatch and highlightColour)             */
/*   - Fix navigation bugs w/ multiple files in the ring.                  */
/*                                                                         */
/***************************************************************************/
/* ??END  - End of verbose help */

  Signal On Novalue

  Parse Arg parms

  alwaysprompt = 0      /* If 0, automatically jumps to the file containing */
                        /* the token if there is only one match. If 1, it   */
                        /* will always prompt, even for a single match      */

  exactCase = 1         /* If true, normal searches must match case */

  wildStart = 0         /* If true, token is assumed to begin with a       */
                        /* wildcard and all tokens in the tags file ENDING */
                        /* with the user's token will be matched           */

  wildEnd = 0           /* If true, token is assumed to end in a wildcard, */
                        /* and all tokens in the tags file STARTING with   */
                        /* the user's token will be matched                */

  sortByType = 0        /* If true, hits are sorted by symbol type. They   */
                        /* are sorted the way ctags does it (by symbol     */
                        /* name) by default.                               */

  wildCardExactCase = 0 /* If true, wildcard searches must match case */

  tagFile = 'tags'     /* Name of tags file */

  tagFileDir = '.'      /* Location to start search for tags file. */

  searchCurrentTree = 1 /* If true, the current directory and its parents   */
                        /* are searched for the tags file if it isn't found */
                        /* in tagFileDir. Set to false if dir is explicit   */

  popupHeightPct = 0.90 /* Max percentage of screen height of popup window  */

  popupWidthPct  = 0.90 /* Max percentage of screen width of popup window   */

  includeFullDir = 1    /* If true, file directory is included in popup box */

  cacheTagFile = 1      /* If true, the 'X' search procedure will cache the */
                        /* tags file in memory by keeping it loaded and     */
                        /* hiding it from the edit ring. Note that this may */
                        /* make searches faster, but will use more memory.  */
                        /* All tag files will be cached and you'll have to  */
                        /* exit the editor to unload the files.             */

  highlightMatch = 0    /* If true, highlights matched token. Unfortunately */
                        /* it doesn't go away like the "locate" higlighting */

  highlightColour = '=highlight'
                        /* Color to highlight with in the form "fg ON bg"   */
                        /* as described by the LINECOLOUR command. Can also */
                        /* be set to "=areaname" where areaname is one of   */
                        /* the defined color areas.                         */

  /* Name of line searching procedure (SearchForHits_ is prepended to name)  */
  /*                                                                         */
  /* This procedure is passed:                                               */
  /*   tagFile   - Fully qualified tag file name                             */
  /*   token     - Identifier to search for                                  */
  /*   wildStart - As described above                                        */
  /*   wildEnd   - As described above                                        */
  /*   exactCase - As described above                                        */
  /*   options   - Any option string given after the procedure name.         */
  /*                                                                         */
  /* Other global variables can be exposed if needed.                        */
  /*                                                                         */
  /* If it is non-successful, it should return a non-blank error message. If */
  /* successful, it should return an empty string and fill in the stem       */
  /* matchedLine. with each the contents of line of the file that matched.   */

  searchProcName = "X"
  searchProcOpts = ""

  searchProcList = "X Grep"

  /* Valid characters that can make up an identifier */

  idchars  = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_$~"

  /****************************************************************************/

  /* Extensions for various file types in ctags */

  EXTS_c       = "c C c++ cc cpp cxx h H hh hpp hxx h++ inl hdr"
  EXTS_eiffel  = "e"
  EXTS_fortran = "f for ftn f77 f90 f95 F FOR FTN F77 F90 F95"
  EXTS_java    = "java"

  /* Descriptions for symbol 'kind' fields for each file type */

  TYPES.1   = "java     c   class"
  TYPES.2   = "java     f   field"
  TYPES.3   = "java     i   interface"
  TYPES.4   = "java     m   method"
  TYPES.5   = "java     p   package"
  TYPES.6   = "c        c   class"
  TYPES.7   = "c        d   macro"
  TYPES.8   = "c        e   enum value"
  TYPES.9   = "c        f   function"
  TYPES.10  = "c        g   enum name"
  TYPES.11  = "c        m   member"
  TYPES.12  = "c        n   namespace"
  TYPES.13  = "c        p   prototype"
  TYPES.14  = "c        s   structure"
  TYPES.15  = "c        t   typedef"
  TYPES.16  = "c        u   union"
  TYPES.17  = "c        v   variable"
  TYPES.18  = "c        x   extern"
  TYPES.19  = "fortran  b   block data"
  TYPES.20  = "fortran  c   comm block"
  TYPES.21  = "fortran  e   entry pnt"
  TYPES.22  = "fortran  f   function"
  TYPES.23  = "fortran  i   interface"
  TYPES.24  = "fortran  l   labels"
  TYPES.25  = "fortran  m   modules"
  TYPES.26  = "fortran  n   namelist"
  TYPES.27  = "fortran  p   program"
  TYPES.28  = "fortran  s   subroutine"
  TYPES.29  = "fortran  t   derivedtype"
  TYPES.30  = "eiffel   c   class"
  TYPES.31  = "eiffel   f   feature"
  TYPES.32  = "eiffel   l   local entity"
  TYPES.0   = 32

  /* Names of the extended flag names we process */
  /* Space prepended to names so we don't get partial hits */

  ExtFlag.1 = " class:"
  ExtFlag.2 = " struct:"
  ExtFlag.3 = " enum:"
  ExtFlag.4 = " union:"
  ExtFlag.5 = " interface:"
  ExtFlag.6 = " namespace:"
  ExtFlag.0 = 6

  /****************************************************************************/

  /* Do some screen specific setup */

  popupHeight = 15                      /* Pick reasonable defaults */
  popupWidth  = 72

  'EXTRACT /FLSCREEN/'

  If flscreen.0 >= 2 Then
    popupHeight = trunc((flscreen.2 - flscreen.1 + 1) * popupHeightPct)

  'EXTRACT /SCREEN/'

  If screen.0 >= 2 Then
    popupWidth = trunc(screen.2 * popupWidthPct)

  typeColWidth = 10                             /* Fixed at this size   */
  fileColWidth = 20                             /* Minimum to allow for */

  /****************************************************************************/

  If Substr(DIRECTORY(),2,1) = ':' Then         /* Try to make unix compliant */
    slash = '\'
  Else
    slash = '/'

  /****************************************************************************/
  /* Display help */

  If Wordpos(translate(word(parms,1)), "? -? /? -HELP /HELP ?? -?? /??") > 0 Then
    Do
      'EXTRACT /ESCAPE/'
      e = escape.1

      If Pos("??", word(parms,1)) > 0 Then
        Do
          help.title = 'Verbose Help (from macro source)'

          maxline = sourceline()

          Do i = 1 To maxline
            If Pos("??START", sourceline(i)) > 0 Then
              Do
                i = i + 1
                Leave
              End
          End

          h = 0

          Do i = i To maxline
            If Pos("??END", sourceline(i)) > 0 Then
              Leave

            /* Strip off comment characters and lines of asterisks */

            l = Strip(sourceline(i))

            If Substr(l,1,2) = "/*" Then
              l = Substr(l,3)

            If Right(l,2) = "*/" Then
              l = Substr(l,1,Length(l)-2)

            If copies("*",Length(l)) <> l Then
              Do
                h = h + 1
                help.h = Strip(l,'T')
              End
          End

          help.0 = h
        End
      Else
        Do
          help.title = 'Syntax Help'

          help.1  = ""
          help.2  = "\eBSyntax:\eE btags -F file -D dir -S (n|t) -C (e|c) ..."
          help.3  = "\eE              -P proc,opts tag"
          help.4  = ""
          help.5  = "All options may start with '-' or '/'. Enclose values in double quotes"
          help.6  = "if they contain embedded blanks. The space between option flags and"
          help.7  = "values is optional."
          help.8  = ""
          help.9  = "   \eE-F\eT  - Specify alternate tags filename. Default is '\eE"tagFile"\eT'"
          help.10 = "   \eE-D\eT  - Specify search directory for tags file. Default is '\eE"tagfiledir"\eT'"
          help.11 = "   \eE-S\eT  - Sort mode - \eEn\eT=Sort symbols by name. \eEt\eT=Sort symbols by type"
          help.12 = "   \eE-C\eT  - Case mode - \eEe\eT=Force exact case match, \eEc\eT=match any case. Affects"
          help.13 = "         both search types. By default, exact matches are exact and"
          help.14 = "         wildcard searches can match any case."
          help.15 = "   \eE-P\eT  - Set search procedure. Default=\eE"searchProcName"\eT, choices=\eE"searchProcList"\eT. Comma"
          help.16 = "         delimited, procedure specific options can follow the proc name."
          help.17 = "         (Note: Grep is faster unless cacheTagFile=1, see verbose help)"
          help.18 = "   \eE-??\eT - Verbose usage/programming help (macro comments)"
          help.19 = "   \eEtag\eT - If blank, uses tag at cursor, otherwise uses the given value"
          help.20 = ""
          help.0  = 20
        End

      hmax = 30

      Do i = 1 To help.0
        epos = 1
        ecnt = 0
        Do While Pos('\e', help.i) > 0                  /* Handle escape chars */
          epos = Pos('\e', help.i)
          help.i = insert(e, delstr(help.i,epos,2), epos-1)
          ecnt = ecnt + 1
        End
        hmax = max(hmax, Length(help.i)+2-(ecnt*2))
      End

      'WINDOW 'min(help.0,popupHeight) max(hmax,popupWidth) help.0 help.title
      Do i = 1 To help.0
        'WINLINE  'help.i'\nSETRESULT 'i
      End
      'WINWAIT'
      Exit
    End

  /* Process any 'optional' parameters */

  parms = Strip(parms)

  Do While parms <> "" & Pos(substr(parms,1,1), "-/") > 0
    option = Translate(substr(parms,2,1))     /* Pull off letter */
    parms  = Strip(substr(parms,3))           /* Leave rest alone */

    Select
      When option = 'F' Then
        Do
          parms = getParmVal(parms)
          tagFile = parmVal

          If tagFile = "" Then
            Do
              'MSG No tags file given'
              Exit
            End
        End
      When option = 'D' Then
        Do
          parms = getParmVal(parms)
          tagFileDir = parmVal

          If tagFileDir = "" Then
            Do
              'MSG No tags file directory given'
              Exit
            End

          If Right(tagFileDir,1) = slash Then
            tagFileDir = Substr(tagFileDir,1,Length(tagFileDir)-1)

          searchCurrentTree = 0
        End
      When option = 'S' Then
        Do
          parms = getParmVal(parms)

          If Translate(parmVal) = 'N' Then
            sortByType = 0
          Else If Translate(parmVal) = 'T' Then
            sortByType = 1
          Else
            Do
              'MSG Sort parameter must be n or t'
              Exit
            End
        End
      When option = 'C' Then
        Do
          parms = getParmVal(parms)

          If Translate(parmVal) = 'E' Then
            Do
              exactCase = 1
              wildCardExactCase = 1
            End
          Else If Translate(parmVal) = 'C' Then
            Do
              exactCase = 0
              wildCardExactCase = 0
            End
          Else
            Do
              'MSG Case parameter must be e or c'
              Exit
            End
        End
      When option = 'P' Then
        Do
          parms = getParmVal(parms)

          Parse Var parmVal searchProcName ',' searchProcOpts

          If Wordpos(translate(searchProcName), Translate(searchProcList)) = 0 Then
            Do
              'MSG Search procedure 'searchProcName' not found'
              Exit
            End
        End
      Otherwise
        Do
          "MSG Invalid option '"option"'. Run 'btags -?' for help"
          Exit
        End
    End
  End

  If words(parms) > 1 Then
    Do
      'MSG Invalid parameters: Only one tag allowed'
      Exit
    End

  /****************************************************************************/

  /* Split any directory out of the tag filename */

  p = Lastpos(slash, tagFile)

  If p = 0 & slash = '\' Then           /* Allow unix slash on PC systems */
    p = Lastpos('/', tagFile)

  If p > 0 Then
    Do
      searchCurrentTree = 0
      tagFileDir = Substr(tagFile, 1, p-1)
      tagFile    = Substr(tagFile, p+1)
    End

  /* Locate the tags file by searching back in the source tree */

  If stream(tagFileDir||slash||tagFile, 'c', 'query exists') = "" & searchCurrentTree Then
    Do
      found = 0

      tagFileDir = DIRECTORY()

      Do While ((pos(slash,tagFileDir) > 0) | Right('_'tagFileDir,1) = ':') & (\found)

        newfile = tagFileDir||slash||tagFile

        If stream(newfile, 'c', 'query exists') <> "" Then
          Do
            tagFile = newfile
            found = 1
          End
        Else If Lastpos(slash,tagFileDir) > 0 Then
          tagFileDir = Substr(tagFileDir, 1, Lastpos(slash,tagFileDir)-1)
        Else
          tagFileDir = ""                   /* End of search */
      End

      If \found Then
        Do
          'msg Tags file "'tagFile'" not found in source tree: 'DIRECTORY()
          Exit
        End
    End
  Else
    Do
      tagFile = tagFileDir||slash||tagFile              /* Fully qualify */
    End

  /* Find the token or pull from command line */

  If parms <> "" Then
    Do
      /* Use first word of parameter, pulling off wildcard characters */

      token = word(parms,1)

      If Right(token,1) = "*" Then
        Do
          wildEnd = 1
          exactCase = wildCardExactCase
          token = Substr(token,1,Length(token)-1)
        End

      If Left(token,1) = "*" Then
        Do
          wildStart = 1
          exactCase = wildCardExactCase
          token = Substr(token,2)
        End

      If token = "" Then
        Do
          'MSG Invalid token specified'
          Return
        End
    End
  Else
    Do
      'EXTRACT /CURLINE/'
      'EXTRACT /CURSOR/'

      token    = curline.1
      tokstart = cursor.2
      tokend   = cursor.2

      If Pos(substr(token,tokstart,1), idchars) = 0 | tokstart > Length(token) Then
        Do
         'MSG The cursor is not on a valid identifier'
         Return
        End

      /* Find token start */

      Do While Pos(substr(token,tokstart,1), idchars) > 0
        tokstart = tokstart-1
        If tokstart = 0 Then
          Leave
      End

      tokstart = tokstart+1

      /* Find token end */

      Do While Pos(substr(token,tokend,1), idchars) > 0
        tokend = tokend+1
        If tokend > Length(token) Then
          Leave
      End

      tokend = tokend-1

      token = Strip(substr(token,tokstart,tokend-tokstart+1))
      If token = ''
        Then Do
          'MSG No token found for Tags' tokstart tokend
          Return
        End
    End

  /* Search for matches. If succesful, returns an empty string and fills */
  /* in the matchedLine stem with each matching line from the tags file. */
  /* Returns a non-blank error message if failed                         */

  /* ErrMsg = SearchForHits_X(tagFile, token, wildStart, wildEnd, exactCase) */
  Interpret("ErrMsg = SearchForHits_"searchProcName"(tagFile, token, wildStart, wildEnd, exactCase, searchProcOpts)")

  /* If we were successful, display the list of hits */

  If ErrMsg <> "" Then
    Do
      'MSGMODE ON'
      'MSG 'ErrMsg
      Exit
    End
  Else
    Do
      /* Report a message if tag not found */

      If matchedLine.0 <= 0 Then
        Do
          'MSGMODE ON'
          'MSG Identifier 'Strip(token) 'not found in' tagFile
          Exit
        End

      /* Process the lines, building information for display */

      maxTokLen = 0
      maxFileLen = 0

      Do hiti = 1 To matchedLine.0
        Parse Var matchedLine.hiti . file searchCmd ';"' symId extFlags

        /* Determine file long/short names and search command */

        hit_lfile.hiti = file
        hit_searchCmd.hiti = searchCmd

        If includeFullDir Then
          hit_sfile.hiti = file
        Else
          Do
            slp = Lastpos('\', file)

            If slp = 0 Then
              slp = Lastpos('/', file)

            If slp > 0 Then
              hit_sfile.hiti = Substr(file,slp+1)
            Else
              hit_sfile.hiti = file
          End

        /* Select descriptions based on extension */

        exp = Lastpos('.',hit_sfile.hiti)

        If exp > 0 Then
          fileExt = Substr(hit_sfile.hiti,exp+1)
        Else
          fileExt = ""

        If Wordpos(fileExt, EXTS_c) > 0 Then
          fileType = "c"
        Else If Wordpos(fileExt, EXTS_java) > 0 Then
          fileType = "java"
        Else If Wordpos(fileExt, EXTS_fortran) > 0 Then
          fileType = "fortran"
        Else If Wordpos(fileExt, EXTS_eiffel) > 0 Then
          fileType = "eiffel"
        Else
          fileType = "ext:"fileExt

        symType = fileType"("symId")"           /* Pick a default */

        Do typei = 1 To types.0
          If word(types.typei,1) = fileType & word(types.typei,2) = symId Then
            Do
              symType = subword(types.typei,3)
              Leave
            End
        End

        /* Pull off any extension flags. I only pick one extension to use */
        /* (I don't know if there can be more than one). So the order     */
        /* that they're in the stem is important                          */
        /* This is done so class members are prefixed by the class name   */

        extFlags = " "extFlags
        symPrefix = ""

        Do exti = 1 To ExtFlag.0
          p = Pos(ExtFlag.exti, extFlags)

          If p > 0 Then
            Do
              symPrefix = word(substr(extFlags, p+length(ExtFlag.exti)), 1)

              /* Add as separator appropriate to language */
              If Length(symPrefix) > 0 Then
                Do
                  If fileType = "java" Then
                    symPrefixSep = '.'
                  Else
                    symPrefixSep = '::'

                  symPrefix = symPrefix || symPrefixSep
                End
              Leave
            End
        End

        hit_token.hiti = word(matchedLine.hiti,1)

        symbol = symPrefix || hit_token.hiti

        hit_SymType.hiti = Left(symtype,typeColWidth)    /* Limit type description length */
        hit_symbol.hiti  = symbol

        maxFileLen = max(maxFileLen, Length(hit_sfile.hiti))
        maxTokLen = max(maxTokLen, Length(symbol))

      End /* Process hits */

      /* Display the matches if needed */

      If matchedLine.0 > 1 | alwaysprompt Then
        Do
          /* Adjust the window size down if needed */

          popupHeight  = min(matchedLine.0,popupHeight)

          fileColWidth = max(fileColWidth, maxFileLen)  /* Update above minimum if needed */
          tokColWidth  = maxTokLen

          popupWidth   = min(popupWidth, typeColWidth+tokColWidth+fileColWidth+3)

          /* Display the window */

          title = "Matches for: "
          ttWidth = popupWidth - Length("line xx of xxx") - Length(title) - 5

          If Length(token) > ttWidth & ttWidth > 0 Then
            title = title || Left(token,ttWidth) || "..."
          Else
            title = title || token

          'EXTRACT /ESCAPE/'
          e = escape.1

          'window 'popupHeight' 'popupWidth' 'matchedLine.0 title

          Do i = 1 To matchedLine.0

            symbol   = hit_symbol.i
            symType  = hit_symType.i
            filename = hit_sfile.i

            pad = " "
            If Length(symbol) < tokColWidth Then
              pad = copies(' ',tokColWidth-length(symbol))||pad

            p = Pos('\n', filename)           /* Convert \n to \\n */
            Do While p > 0
              filename = insert('\',filename,p)
              p = Pos('\n', filename, p+3)
            End

            'WINLINE 'e'T'symType''e'T 'e'E'symbol||e'T'pad||filename'\nSETRESULT 'i
          End

          If sortByType Then
            'winsort'

          'winwait'

          selection = Value('RESULT')

          If selection = 'RESULT' Then          /* User pressed escape */
            Exit
        End
      Else
        Do
          selection = 1
        End

      /* Add a path name to the file if it is relative and drive name */
      /* for PCs if there isn't one                                   */

      fileToEdit = hit_lfile.selection

      If Wordpos(left(fileToEdit,2), "./ .\") > 0 Then
        fileToEdit = tagFileDir || Substr(fileToEdit,2)
      Else If Pos(left(fileToEdit,1), "\/") = 0 & Substr(fileToEdit,2,1) <> ':' Then
        fileToEdit = tagFileDir || slash || fileToEdit
      Else If Pos(left(fileToEdit,1), "\/") > 0 & Substr(tagFileDir,2,1) = ':' Then
        fileToEdit = Left(tagFileDir,2) || fileToEdit

      /* Open the file and locate the match */

      'EDIT -top 'fileToEdit

      If rc < 0 | rc = 1 Then
        Do
          If rc = 1 Then
            'quit'
          'MSGMODE ON'
          /* For some reason, a MSG here doesn't work...the WINDOW selection */
          /* text remains on the screen                                      */
          'MESSAGEBOX Not found: 'fileToEdit
          Exit
        End
      Else
        Do
          searchcmd = Strip(hit_searchCmd.selection)

          If Datatype(searchcmd,'W') Then
            Do
              'CURSOR' searchcmd
            End
          Else
            Do
              findParms = 'e'

              searchcmd = Strip(searchcmd,'L')

              If Substr(searchcmd,1,2) = '/' Then
                Do
                  searchcmd = Substr(searchcmd,3)
                  findParms = findParms'f'
                End

              If Right(searchcmd,2) = '$/' Then
                searchcmd = Left(searchcmd,Length(searchcmd)-2)

              p = Pos('\/', searchcmd)          /* Remove escape sequences */
              Do While p > 0
                searchcmd = delstr(searchcmd,p,1)
                p = Pos('\/', searchcmd, p)
              End

              'TOP'             /* Do in case symbol is in same file */
              'LOCATE 'd2c(254)||searchcmd||d2c(254)||findParms
              'RESTORE_FIND'

            End

            'EXTRACT /LINE/'
            p = Pos(hit_token.selection, line.1)
            If p > 0 Then
              Do
                'CURSOR +0 'p

                If highlightMatch Then
                  Do
                    /* Lookup area name if needed */

                    If Left(highlightColour,1) = '=' Then
                      Do
                        'EXTRACT /COLOURS/'

                        hc = Translate(substr(highlightColour,2))
                        highlightColour = 'yellow on red'

                        Do i = 1 To colours.0
                          If Translate(word(colours.i,1)) = hc Then
                            Do
                              highlightColour = subword(colours.i, 2)
                              Leave
                            End
                        End
                      End

                    'linecolour 'p' 'p+length(hit_token.selection)-1' 'highlightColour
                  End
              End

            'MSGMODE ON'

            If matchedLine.0 = 1 & \alwaysPrompt Then
              'MSG Only one match found...cursor positioned there'
        End

    End

Exit

/*****************************************************************************/
/* Uses grep to search for lines with a matching tag.  Note that this uses   */
/* some features that may not be part of all grep programs, such as the -h   */
/* flag to avoid printing out the filename, the -i flag to force case        */
/* insensitivity, and the ability to embed escape sequences (like the tab)   */
/* into the regular expression.                                              */

SearchForHits_Grep: Procedure Expose matchedLine. idchars
  Parse Arg tagFile , token , wildStart , wildEnd , exactCase , searchProcOpts , .

  ErrMsg = ""

  If searchProcOpts <> "" Then
    Return "Unknown grep options: '"searchProcOpts"'"

  newQueue = rxqueue("CREATE")          /* Create a new queue since X controls real one */
  oldQueue = rxqueue('SET', newQueue)

  Do While queued() > 0
    Parse Pull .
  End

  flags = "-h"                          /* Don't display filename */

  If \exactCase Then
    flags = flags" -i"                  /* Ignore case */

  /* Add characters to the token/regular expression to control wildcard */
  /* or exact matching.                                                 */

  If wildStart Then
    token = ""idchars"*"token
  Else
    token = ""token

  If wildEnd Then
    token = token""idchars"*"||d2c(9) /* Identifier in tags file ends in tab */
  Else
    token = token||d2c(9)

  Cmd = "grep "flags' "'token'" 'tagFile' | rxqueue 'newQueue
  Cmd

  If RC <> 0 Then
    ErrMsg = "Error "R" calling "Cmd
  Else
    Do
      hits = 0

      /* Put all matches into the stem */

      Do While queued() > 0
        Parse Pull line
        hits = hits + 1
        matchedLine.hits = Translate(line,' ','09'x)
      End

      matchedLine.0 = hits
    End

  R = rxqueue('Set', oldQueue)                  /* Clean up queue */
  R = rxqueue('Delete', newQueue)

Return ErrMsg

/*****************************************************************************/
/* Search for hits using the X editor. This is faster for small files and    */
/* slower machines. On large */

SearchForHits_X: Procedure Expose matchedLine. cacheTagFile

  Parse Arg tagFile , token , wildStart , wildEnd , exactCase , searchProcOpts , .

  /* Process X search options */

  Do While searchProcOpts <> ""
    Parse Var searchProcOpts current ',' searchProcOpts

    Select
      /* Parameter: C=0|1 overrides caching for this file */

      When Left(translate(current),2) = 'C=' Then
        Do
          cacheTagFile = Substr(current,3)
          If \Datatype(cacheTagFile, 'Binary') Then
            Return "Error, the 'C' option must be set to 0 or 1"
        End

      Otherwise
        Do
          Return "Unknown X search option '"current"'"
        End
    End
  End

  If \wildEnd Then
    token = token" "                    /* Search for exact value */

  /* Save the current file being edited so we can return to it if caching */

  If cacheTagFile Then
    Do
      'extract /RING/'
      currentFile = ring.1
    End

  /* Open and search the tags file */

  'EDIT -notabs -b -q' tagFile

  If RC < 0 | RC = 1 Then
    Do
      R = RC
      'QUIT'
      'MSGMODE ON'
      Return 'Error 'R' opening tags file: "'tagFile'" - no longer exists'
    End

  searchFlags = ""

  If \wildStart Then
    searchFlags = searchFlags||'f'

  If exactCase Then
    searchFlags = searchFlags||'e'

  '0'
  'LOCATE /'token'/'searchFlags

  /* Exit if search failed, otherwise process hits */

  If rc <> 0 Then
    Do
      'RESTORE_FIND'
      If cacheTagFile Then
        Do
          'HIDEFILE ON'
          'EDIT 'currentFile
        End
      Else
        'QUIT'
      matchedLine.0 = 0
      Return ""
    End
  Else
    Do
      /* Use a bigger find loop to do wildcard matches */

      wildLoopDone = 0
      hits = 0

      Do Until wildLoopDone
        /* Read in and parse out the matching lines */

        'EXTRACT /CURSOR/'
        linenum = cursor.1
        count   = 0
        'EXTRACT /LINE 'linenum+count'/'

        /* Keep reading in lines until we don't match */

        Do While isMatch(token, line.1)
          count = count+1
          hits  = hits+1

          matchedLine.hits = line.1

          'EXTRACT /LINE 'linenum+count'/'
        End

        'RESTORE_FIND'

        /* If we're doing a wildcard search, we need to keep looking for */
        /* more hits later in the file                                   */

        If wildStart | wildEnd | \exactCase Then
          Do
            If count = 0 Then
              newline = linenum+1
            Else
              newline = linenum+count

            'CURSOR 'newline' 1'          /* Go to next line after match */
            'LOCATE /'token'/'searchFlags
            If rc <> 0 Then
              wildLoopDone = 1
          End
        Else
          Do
            wildLoopDone = 1
          End

      End  /* wildLoop */

      matchedLine.0 = hits

      If cacheTagFile Then
        Do
          'HIDEFILE ON'
          'EDIT 'currentFile
        End
      Else
        'QUIT'

    End /* If first search succeeded */

Return ""

/*****************************************************************************/

isMatch: Procedure Expose exactCase wildStart wildEnd

  Parse Arg token , line

  matched = 0

  token = Strip(token)
  ident = Strip(word(line,1))

  If \exactCase Then
    Do
      ident = Translate(ident)
      token = Translate(token)
    End

  If wildStart | wildEnd Then
    Do
      If Length(ident) < Length(token) Then
        Do
          matched = 0
        End
      Else If Length(ident) = Length(token) Then
        Do
          If ident = token Then
            matched = 1
        End
      Else If wildStart & wildEnd Then
        Do
          If Pos(token, ident) > 0 Then
            matched = 1
        End
      Else If wildStart Then
        Do
          If Right(ident,Length(token)) = token Then
            matched = 1
        End
      Else /* wildEnd */
        Do
          If Left(ident,Length(token)) = token Then
            matched = 1
        End

    End
  Else
    Do
      If ident = token Then
        matched = 1
    End

Return matched

/*****************************************************************************/

getParmVal: Procedure Expose parmVal
  Parse Arg parms

  p1 = Pos('"', parms)          /* Find first quote...must be first in string */

  If p1 = 1 Then
    p2 = Pos('" ', parms" ")    /* Search for ending double quote followed by space */
  Else
    p2 = 0

  If p1 = 1 & p2 > 1 Then
    Do
      parmVal = Substr(parms, 2, p2-p1-1)
      parms = Strip(substr(parms, Length(parmVal)+3))
    End
  Else
    Parse Var parms parmVal parms

Return parms

/*****************************************************************************/

Novalue:

  'MSGMODE ON'
  "MSG Variable '"CONDITION('D')"' not defined at line "sigl
  Exit

/**************** END OF MACRO ****************/
