/*+------------------------------------------------------------------------+
  | CBOX.X - X2 macro to create a box comment.  The box's starting column  |
  |          is determined by the first non-blank column of the previous   |
  |          line(s).  If insert is on, it is turned off.                  |
  |                                                                        |
  |             Modify DEF file as follow:                                 |
  |                                                                        |
  |             key x = macro CBOX  (where x is the key of your choice)    |
  |                                                                        |
  | RELATED MACROS:                                                        |
  |                                                                        |
  | NEWLINE.X - X2 macro to:                                               |
  | 1) Add a line to a comment box when enter is pressed while in a comment|
  |    box.  Turn INSERT mode off if on.                                   |
  | 2) If there is a quick comment without any code preceeding it, change  |
  |    the quick comment to a regular comment and align it to the right    |
  |    margin.                                                             |
  | 3) If INSERT mode is on, add a new line.                               |
  | 4) If INSERT mode is off, go to the next line unless we're             |
  |    on the last line, in which case add a new line.                     |
  |                                                                        |
  | Modify DEF file as follow:                                             |
  |                                                                        |
  |             key enter = macro NEWLINE                                  |
  |------------------------------------------------------------------------|
  | Written by Tom Brock                                TBROCK at BTVMANVM |
  |                                                            20 Jul 1995 |
  +------------------------------------------------------------------------+*/

/*+------------------------------------------------------------------------+
  | Change the following to suit your box style.  For a box comment that   |
  | looks like this one, set the variables to:                             |
  |                                                                        |
  | box_start      = '/*+'  /* 3 chars */                                  |
  | filler_char    = '-'    /* 1 chars */                                  |
  | end_char       = '+  '  /* 3 chars */                                  |
  | box_left_char  = '  |'  /* 3 chars */                                  |
  | box_right_char = '|  '  /* 3 chars */                                  |
  | begin_char     = '  +'  /* 3 chars */                                  |
  | box_end        = '+*/'  /* 3 chars */                                  |
  +------------------------------------------------------------------------+*/

/*+------------------------------------------------------------------------+
  | For a box comment that looks like                                      |
  |                                                                        |
  |     /************************************/                             |
  |     /*                                  */                             |
  |     /************************************/                             |
  |                                                                        |
  | set the variables to:                                                  |
  |                                                                        |
  | box_start      = '/**'    /* 3 chars */                                |
  | filler_char    = '*'      /* 1 chars */                                |
  | end_char       = '**/'    /* 3 chars */                                |
  | box_left_char  = '/* '    /* 3 chars */                                |
  | box_right_char = ' */'    /* 3 chars */                                |
  | begin_char     = '/**'    /* 3 chars */                                |
  | box_end        = '**/'    /* 3 chars */                                |
  +------------------------------------------------------------------------+*/

box_start      = '/*+'
filler_char    = '-'
end_char       = '+  '
box_left_char  = '  |'
box_right_char = '|  '
begin_char     = '  +'
box_end        = '+*/'

/*+------------------------------------------------------------------------+
  | Save the current cursor position                                       |
  +------------------------------------------------------------------------+*/

COMMAND 'EXTRACT /CURSOR/'
save_line = CURSOR.1
save_col  = CURSOR.2

/*+------------------------------------------------------------------------+
  | Find the real indentation (starting column).  Start at the current     |
  | line, if it is blank, go up a line.  Repeat until a non-blank line is  |
  | found or you're at line 1.                                             |
  +------------------------------------------------------------------------+*/

line      = ''
line_num  = save_line
start_col = 1
moved     = 0

do while line = '' & line_num > 0
  COMMAND 'EXTRACT /CURLINE/'
  line = CURLINE.1

  if line <> '' then do
    start_col = wordindex( line, 1 )
    line_num = save_line
    COMMAND 'CURSOR' line_num start_col
  end
  else do
    moved = moved + 1
    line_num = line_num - 1
    COMMAND 'CURSOR' line_num save_col
  end
end

/*+------------------------------------------------------------------------+
  | Determine the last column.                                             |
  +------------------------------------------------------------------------+*/

COMMAND 'EXTRACT /MARGINS/'
end_col = MARGINS.4                              /* last column for comment */

/*+------------------------------------------------------------------------+
  | Calculate total length.                                                |
  +------------------------------------------------------------------------+*/

len = end_col - start_col - 5

filler = ''
blanks = ''

filler = copies( filler_char, len )
blanks = copies( ' ', len )

if start_col > 1 then do
  indent = copies( ' ', start_col - 1 )
end
else do
  indent = ''
end

/*+------------------------------------------------------------------------+
  | Do we need to input a blank line?  I like a blank line before and after|
  | my comment box.                                                        |
  +------------------------------------------------------------------------+*/

if moved = 0 & line_num <> 1 then do
  COMMAND 'INPUT'
  line_num = line_num + 1
end

ins_line = indent || box_start || filler || end_char
COMMAND 'INPUT' ins_line
line_num = line_num + 1

ins_line = indent || box_left_char || blanks || box_right_char
COMMAND 'INPUT' ins_line
line_num = line_num + 1

ins_line = indent || begin_char || filler || box_end
COMMAND 'INPUT' ins_line
COMMAND 'INPUT'

/*+------------------------------------------------------------------------+
  | Place the cursor for immediate editing.                                |
  +------------------------------------------------------------------------+*/

col_num = start_col + 4

COMMAND 'CURSOR' line_num col_num

COMMAND 'EXTRACT /INSERT/'

if INSERT.1 = 1 then do                   /* If in insert mode, turn it off */
  COMMAND 'INSMODE OFF'
end

exit 0
