/***************************************************************************/
/*                                                                         */
/* Document the end of a C block of code.  We look for the close brace,    */
/* then find the matching open brace.  We examine the word before the      */
/* previous left parenthesis to see if we recognise it, and write an       */
/* inline comment accordingly.                                             */
/*                                                                         */
/* We check for several keywords, and if none are found we assume we're at */
/* the end of a C function.                                                */
/*                                                                         */
/* Requires version 1.69 of the editor, or later.                          */
/*                                                                         */
/* Written by B. Thompson, October 30, 1995                                */
/* Support Rexx files, January 12, 1999                                    */
/* Support CGI scripts, March 2, 1999                                      */
/*                                                                         */
/***************************************************************************/

  'EXTRACT /CURLINE/'                       /* Get current file information*/
  'EXTRACT /COMMENTS/'
  'EXTRACT /CURSOR/'
  'EXTRACT /FT/'

  If Pos(comments.1, curline.1) > 0
    Then Exit                                   /* A comment already exists*/

  Select
    When Wordpos(ft.1, 'CMD X REXX') > 0
      Then cmt = rexx_file(curline.1)
    When ft.1 = 'CGI'
      Then cmt = cgi_file(curline.1)
    Otherwise cmt = c_file(curline.1)                /* Default is always C*/
  End                                                         /* End select*/

  'CURSOR' cursor.1 cursor.2
  If cmt <> ''
    Then 'REPLACE' curline.1 Strip(comments.1) cmt Strip(comments.2)
Exit


/***************************************************************************/
/*                                                                         */
/* Look for a brace on the current line; if found, match backwards to find */
/* out which construct it belongs to, so we can provide a meaningful "end" */
/* comment.                                                                */
/*                                                                         */
/***************************************************************************/

C_FILE:  Procedure Expose comments.
  Parse Arg csrline
  brace = Pos('}', csrline)
  If brace = 0
    Then Exit                                      /* No close block marker*/

  'CURSOR +0' brace
  'MATCH'
  'PREVIOUS_WORD'                               /* Check for else statement*/
  'EXTRACT /CURLINE/'
  'EXTRACT /CURSOR/'
  token = substr(curline.1,cursor.2)
  Parse var token token . '{' .
  If Translate(token) = 'ELSE'
    Then Do
      'l /}/-'
      If rc = 0
	Then 'MATCH'
    End
    Else 'NEXT_WORD'

  cmt = ''					  /* Default comment string*/
  'L /)/ -'
  If rc = 0
    Then Do				      /* Looks good, let's parse it*/
      'MATCH'
      'PREVIOUS_WORD'
      'EXTRACT /CURLINE/'
      'EXTRACT /CURSOR/'
      token = substr(curline.1,cursor.2)
      Parse var token token '(' .
      token = strip(token)
      up_token = Translate(token)
      If Wordpos(up_token,'IF DO FOR WHILE SWITCH') > 0
	Then cmt = 'End' Translate(token,,
				   'abcdefghijklmnopqrstuvwxyz',,
				   'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
	Else cmt = 'End of' Word(token, Words(token)) 'function'
    End
  'RESTORE_FIND'
Return cmt


/***************************************************************************/
/*									   */
/* Look for an End or Return statement on the current line; if found,      */
/* match backwards to find out which construct it belongs to, so we can    */
/* provide a meaningful "end" comment.					   */
/*									   */
/***************************************************************************/

REXX_FILE:  Procedure Expose comments.
  Parse Upper Arg csrline

  Select
    When Substr(csrline,1,6) = 'RETURN'
      Then Do					       /* Mark the function*/
	'PREVIOUS_FUNC'
	Do While rc = 0
	  'EXTRACT /CURLINE/'
	  If curline.1 <> ''
	    Then Do			 /* There's something worth parsing*/
	      token = word(curline.1,1)
	      If Substr(token,Length(token)) = ':'
		Then Do						  /* Got it*/
		  Parse Var token char1 2 token ':' .
		  token = char1 || Translate(token,,
				   'abcdefghijklmnopqrstuvwxyz',,
				   'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
		  Leave
		End						  /* End do*/
	    End							  /* End do*/
	  'DOWN 1'
	End							  /* End do*/
	If rc <> 0
	  Then Return ''
      End							  /* End do*/
    When word(csrline,1) = 'END'
      Then Do						/* End do/while etc*/
	'CURSOR +0' Pos('END', csrline)
	'MATCH'
	If rc <> 0
	  Then Return ''
	'EXTRACT /CURLINE/'
	'EXTRACT /CURSOR/'
	token = word(substr(curline.1,cursor.2),1)
      End							  /* End do*/
    Otherwise Exit					/* Tag unrecognised*/
  End							      /* End select*/

  If Wordpos(translate(token),'DO SELECT WHILE') > 0
    Then cmt = 'End' Translate(token,,
			       'abcdefghijklmnopqrstuvwxyz',,
			       'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
    Else cmt = 'End of' token 'function'
Return cmt				       /* End of REXX_FILE function*/


/***************************************************************************/
/*									   */
/* Look for a brace on the current line; if found, match backwards to find */
/* out which construct it belongs to, so we can provide a meaningful "end" */
/* comment.								   */
/*									   */
/***************************************************************************/

CGI_FILE:  Procedure Expose comments.
  Parse Arg csrline
  brace = Pos('}', csrline)
  If brace = 0
    Then Exit					   /* No close block marker*/

  'CURSOR +0' brace
  'MATCH'
  'PREVIOUS_WORD'				/* Check for else statement*/
  'EXTRACT /CURLINE/'
  'EXTRACT /CURSOR/'
  token = substr(curline.1,cursor.2)
  Parse var token token . '{' .
  Select
    When Translate(token) = 'ELSE'
      Then Do
        'l /}/-'
        If rc = 0
          Then 'MATCH'
        'RESTORE_FIND'
      End
    When Wordpos('sub',curline.1) = 1
      Then Return 'End of' word(curline.1,2) 'function'
    Otherwise 'NEXT_WORD'
  End                                                         /* End select*/

  cmt = ''                                        /* Default comment string*/
  'L /)/ -'
  If rc = 0
    Then Do                                   /* Looks good, let's parse it*/
      'MATCH'
      'PREVIOUS_WORD'
      'EXTRACT /CURLINE/'
      'EXTRACT /CURSOR/'
      token = substr(curline.1,cursor.2)
      Parse var token token '(' .
      token = strip(token)
      up_token = Translate(token)
      cmt = 'End' Translate(token,,
                            'abcdefghijklmnopqrstuvwxyz',,
                            'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
    End
  'RESTORE_FIND'
Return cmt
