/******************************************************************/
/* Chart.x -- Toby Thurston - 23 Oct 1997                         */
/*                                                                */
/* Makes an "ASCII ART" bar chart from a table of data.           */
/* Good for notes, :cgraphics., quick data visualization, etc     */
/******************************************************************/
/* Mark a block of text that has a column of numbers on the right */
/* and labels on the left, then do "chart" from the command line  */
/*                                                                */
/*                    |- h -|  |- b -|                            */
/* Syntax: >>- chart ------------------------------------------>< */
/*                    |- v -| |- fill -| |- axis ------------|    */
/*                                                |- size -|      */
/*                                                                */
/* Where "h" means produce a horizontal bar chart (default),      */
/* and "v" means produce a vertical one,                          */
/*                                                                */
/* "fill" is the string to fill the bars with.  Any abbreviation  */
/* of the word "blank" will produce blank bars, which is the      */
/* default.  Don't use numbers or H or V.                         */
/*                                                                */
/* "axis" is the maximum value for the data axis; if you omit     */
/* this is will be set to the power of ten next above the max     */
/* value in the data.                                             */
/*                                                                */
/* and "size" is the number of columns or rows that the chart     */
/* should fit into.  For a vertical chart this will default to 10 */
/* rows (plus rows for the labels), for a horizontal chart it     */
/* defaults to the higher of 40 or your right margin.             */
/*                                                                */
/*********************  Example   *********************************/
/*                                                                */
/* First     35    <-- You mark this block                        */
/* Second    64        CHART.X produces this chart...             */
/*                                                                */
/*        |------------------------                               */
/* First  |                       | 35                            */
/*        |------------------------                               */
/*        |-------------------------------------------            */
/* Second |                                          | 64         */
/*        |-------------------------------------------            */
/*        |---------------------------------------------------    */
/*         0     10     20    30     40     50    60     70       */
/*                                                                */
/******************************************************************/
/* set the box characters here, alter them if not CP 437 or 850   */
/******************************************************************/
tl='da'x; tc='c2'x; tr='bf'x;          /* ¿                     */
ml='c3'x; mc='c5'x; mr='b4'x;          /* Ŵ                     */
bl='c0'x; bc='c1'x; br='d9'x;          /*                      */
mm='b3'x; cc='c4'x;                    /*                       */
warn = 0
arg type filler max display .          /* read the args           */
select
when datatype(type,"N") then do
  display = filler
  max = type
  filler = " "
  type = "H"
end
when datatype(filler,"N") then do
  display = max
  max = filler
  filler = " "
end
otherwise
end
if pos(type,"HV") = 0 then do
  if filler = "" then filler = type
                 else do
                   warn = 1
                   'msg Invalid chart type' type' treated as H.'
                 end
  type = "H"
end
if filler = "" | abbrev("BLANK",filler,1) then filler = " "
if \datatype(max,"N") then max = ""
if \datatype(display,"W") then
  if type = "H" then do
                  'extract /margins/'
                  if rc = 0 then display = margins.4
                            else display = 72
                end
                else display = 10

'extract /mark/'                       /* look for the mark       */
'extract /cursor/'                     /* save the cursor pos     */
if mark.0 = 0 | mark.4 = 0 then do     /* no block mark set       */
  warn = 1
 'msg Chart needs a block mark with labels and data'
  signal bleep                         /* complain and leave      */
end
else do
  row = 0
  longest_label = 0
  highest_value = 0
  mid. = 0
  do i = mark.2 to mark.3
    i                                  /* go to next line of mark */
    'extract /curline/'                /* read the marked cols    */
    line = substr(curline.1,mark.4,mark.5-mark.4+1)
    row = row + 1
    label.row = subword(line,1,words(line)-1)
                         /* 19971017 changed to allow a mid value */
    if datatype(lastword(label.row),"N")
      then do
        mid.row = lastword(label.row)
        label.row = subword(label.row,1,words(label.row)-1)
      end
    longest_label = max(longest_label,length(label.row))
    data_label.row = lastword(line)
    if pos(",",data_label.row) = 0     /* remove commas           */
      then ? = data_label.row
      else ? = space(translate(data_label.row,,","),0)
    if datatype(?,"N")
      then value.row = ?
      else value.row = 0               /* treat non-numbers as 0  */
    highest_value = max(highest_value,value.row)
  end
end
if max = ""                            /* if user did not set max */
  then max = nextpower(highest_value)  /* set automatic max value */
  else do                         /* check user max is reasonable */
    select
      when highest_value / max > 10 then do    /* unreasonable    */
        warn = 1
        'msg Specify a bigger axis scale:  high data value is' highest_value
        signal bleep                   /* complain & leave        */
      end
      when highest_value / max > 1 then do     /* unusual         */
        warn = 1
        'msg Longest bar extends beyond display area!' /* warn    */
      end
    otherwise
    end
  end
select
  when max > 10000000 then do
    axisbase = 1000000
    axislabel = "(Millions)"
  end
  when max > 10000 then do
    axisbase = 1000
    axislabel = "(Thousands)"
  end
otherwise
  axisbase = 1
  axislabel = ""
end
if cursor.1 >= mark.2 & cursor.1 <= mark.3
  then mark.3 + 1
  else cursor.1
'input'                                /* input a blank line      */
select
  when type = "H" then do
    labelspace = copies(" ",longest_label)
    display = max(display - longest_label,40)
    do i = 1 to row                   /* input a bar for each row */
      bar = copies(cc,trunc(value.i/max*display))
      fil = left(copies(filler,trunc(value.i/max*display)),length(bar))
      bbar = bar
      if mid.i > 0
        then do
          bar = overlay(tc,bar,trunc(mid.i/max*display))
          fil = overlay(mm,fil,trunc(mid.i/max*display))
          bbar = overlay(bc,bar,trunc(mid.i/max*display))
        end
      'input' "  "labelspace ml||bar||tr
      'input' "  "overlay(label.i,labelspace) mm||fil||mm data_label.i
      'input' "  "labelspace ml||bbar||br
    end
    /* create the axis & scale, and put them in too                   */
    axis = copies(cc,display)
    scale = copies(" ",display)
    do tick = 0 to 9
      ? = trunc(tick/10*display)
      axis = overlay(tc,axis,?+1)
      ticklabel = format((tick*max)/(10*axisbase),,1)+0
      if ticklabel // 1 = 0 then ticklabel = trunc(ticklabel)
      scale = overlay(ticklabel,scale,?+(length(ticklabel)<3))
    end
    'input' "  "labelspace bl||axis
    'input' "  "labelspace " "scale
    if axislabel <> "" then 'input' "  "labelspace " "right(axislabel,display)
  end
  when type = "V" then do
    if axislabel <> "" then 'input' axislabel
    longest_label = max(10,longest_label)
    barwidth = copies(" ",longest_label)
    ylabel = 6
    filwidth = 2
    filler = left(copies(filler,filwidth),filwidth)
    longbar = copies(cc,filwidth)
    do i = 1 to row                   /* input a bar for each row */
      topofbar.i = format(value.i/max*display,,0)
    end
    do i = display to 1 by -1
      ticklabel = format((i*max)/(display*axisbase),,(max<10))+0
      out = right(ticklabel,ylabel) ml
      do j = 1 to row
        select
          when i = topofbar.j +1 then out = out || centre(value.j/axisbase,longest_label)
          when i > topofbar.j then out = out || barwidth
          when i < topofbar.j then out = out || centre(mm||filler||mm,longest_label)
        otherwise
          out = out || centre(tl||longbar||tr,longest_label)
        end
      end
      'input' out
    end
    out = copies(" ",ylabel) bl
    labels = copies(" ",ylabel+2)
    do i = 1 to row
       if topofbar.i = 0
         then out = out || centre(cc||longbar||cc,longest_label,cc)
         else out = out || centre(bc||longbar||bc,longest_label,cc)
       labels = labels || centre(label.i,longest_label)
    end
    'input' out
    'input' labels
  end
otherwise
  warn = 1
  'msg Invalid type' type
end
'input'                                /* input a blank line      */
'cursor' cursor.1 1
bleep:
if warn then do
  call beep 300,300                    /* Failure or warning      */
end
else do
  call beep 1200,100                   /* Success                 */
  call beep 1000,100
end
exit
lastword: procedure
  return word(arg(1),words(arg(1)))
nextpower: procedure         /* returns next highest power of ten */
  arg x
  select
    when x > 1 then x = 10 ** length(trunc(x-1))
    when x < 1 then do
      parse var x . "." x
      x = 10 ** -(verify(x,"0") - 1)
    end
  otherwise
  end
return x
