/**********************************************************************/
/*                      X2 Editor Macro CONDCMD                       */
/*               Execute editor commands conditionally                */
/*                                                                    */
/*                  /Ŀ          /Ŀ      /Ŀ          */
/*                           /&Ŀ                             */
/*     CONDCMD/cmd1/|cmd2/:cmd3     */
/*                            /~          Ĵ        */
/*                                   */
/*                                                                    */
/* / can be any character, except &|~:                                */
/*                                                                    */
/* The execution of cmd2 and cmd3 depend on the character in front    */
/* of cmd2 and the return code from from cmd1:                        */
/*                                                                    */
/*   & - execute cmd2 if all cmd1 return 0 else execute cmd3          */
/*   | - execute cmd2 if some cmd1 return 0 else execute cmd3         */
/*   ~ - execute cmd2 if none cmd1 returned 0 else execute cmd3       */
/*                                                                    */
/* If cmd2 is executed then the final return code and msg from this   */
/* macro will be that of the last cmd1. However, if cmd3 is executed  */
/* then the final return code and msg will be that of the last cmd1   */
/* that caused cmd3 to be executed:                                   */
/*                                                                    */
/*   & - return code and msg from the last cmd1 that did not return 0 */
/*   | - return code and msg from the last cmd1 that did return 0     */
/*   ~ - return code and msg from the last cmd1 that did return 0     */
/*                                                                    */
/* Examples:                                                          */
/*                                                                    */
/* Edit all macros written or modified by me                          */
/*   x -q -top *.x "-ccondcmd ;/Morten;~quit"                         */
/*                                                                    */
/* Change my name, show changes                                       */
/*   x -q -top *.x "-ccondcmd ;c'Morten'XUser'*;&changes;:quit"       */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/* Written  by Morten Kjrulff 20000320                               */
/**********************************************************************/

  parse arg arg

  /*
  * Help
  */
  if arg = '' then do
    do i = 1 to sourceline()
      if sourceline(i) = '' then leave
    end
    'WINDOW' i length(sourceline(1)) i 'Help'
    do i = 1 to sourceline()
      if sourceline(i) = '' then leave
      'WINLINE' sourceline(i) || '\nREFRESH'
    end
    exit
  end

  /*
  * Get command delimiter
  */
  dlm = substr(arg,1,1)
  arg = substr(arg,2)

  /*
  * Find commands
  */
  cmd.1.0 = 0
  cmd.2.0 = 0
  cmd.3.0 = 0
  group = 1
  do while arg <> ''
    parse var arg cmd (dlm) arg
    if group = 1 & pos(substr(cmd,1,1),'&|~') <> 0 then do
      group = 2
      cmd.group.prefix = substr(cmd,1,1)
      cmd = substr(cmd,2)
    end
    else if group = 2 & pos(substr(cmd,1,1),':') <> 0 then do
      group = 3
      cmd.group.prefix = substr(cmd,1,1)
      cmd = substr(cmd,2)
    end
    i = cmd.group.0 + 1
    cmd.group.0 = i
    cmd.group.i = cmd
  end

  /*
  * Execute cmd1
  */
  all = 1
  some = 0
  none = 1
  finalset = 0
  'MSG .'
  do i=1 to cmd.1.0
    'MSG .'
    ''cmd.1.i
    if rc = 0 then do
      some = 1
      none = 0
      if cmd.2.prefix = '|' | cmd.2.prefix = '~' then do
        finalrc = rc
        'EXTRACT /LASTMSG/'
        finalmsg = lastmsg.1
        finalset = 1
      end
    end
    else do
      all = 0
      if cmd.2.prefix = '&' then do
        finalrc = rc
        'EXTRACT /LASTMSG/'
        finalmsg = lastmsg.1
        finalset = 1
      end
    end
  end
  if \finalset then do
    finalrc = rc
    'EXTRACT /LASTMSG/'
    finalmsg = lastmsg.1
    finalset = 1
  end

  /*
  * Execute cmd2 or cmd3
  */
  execmd2 = 0
  if all  & cmd.2.prefix = '&' then execmd2 = 1
  if some & cmd.2.prefix = '|' then execmd2 = 1
  if none & cmd.2.prefix = '~' then execmd2 = 1
  if execmd2 then do
    do i=1 to cmd.2.0
      ''cmd.2.i
    end
  end
  else do
    do i=1 to cmd.3.0
      ''cmd.3.i
    end
  end

  /*
  * Set final msg and exit
  */
  'MSG' finalmsg

  exit finalrc
