/***************************************************************************/
/*                                                                         */
/* DATACMD - Execute command on current line or the LINE marked lines.     */
/*                                                                         */
/*           Arguments:                                                    */
/*             L - Execute current line, even if a mark exists             */
/*             M - Execute marked lines, only if a mark exists             */
/*                                                                         */
/* Works on Windows/NT with Regina                                         */
/*                                                                          */
/* Written  by Morten Kjrulff 20000309                                    */
/*                                                                         */
/***************************************************************************/

  parse upper arg what

  what = substr(what,1,1)
  if what = '' then do
    what = '?'
  end
  else do
    if pos(what,'LM') = 0 then do
      'MSG DATACMD: Valid arguments are: Line Mark'
      return(1)
    end
  end

  question = ''

  if what = '?' | what = 'M' then do
    'EXTRACT /MARK/'
  end

  if what = '?' then do /* don't know what to execute */
    if mark.0 = 0 then do /* have no mark */
      what = 'L'
    end
    else do /* have a mark */
      if mark.4 = 0 then do /* a line mark */
        question = 'A LINE mark exists'
        choice = 'Line Mark'
        vans = 'LM'
      end
      else do /* a block mark */
        question = 'A BLOCK mark exists'
        choice = 'Line'
        vans = 'L'
      end
    end
  end
  else do /* know what to execute */
    if what = 'M' then do /* want mark */
      if mark.0 = 0 then do /* have no mark */
        question = 'No mark exists'
        choice = 'Line'
        vans = 'L'
      end
      else do /* have a mark */
        if mark.4 <> 0 then do /* a block mark */
          question = 'A BLOCK mark exists'
          choice = 'Line'
          vans = 'L'
        end
      end
    end
  end

  if question <> '' then do
    'EXTRACT /ESCAPE/'
    nl = escape.1'N'
    mbox = question''nl''nl'Execute? 'choice
    'MESSAGEBOX' mbox
    parse upper var result what
    if pos(what,vans) = 0 then do
      'MSG DATACMD: No commands executed'
      return(0)
    end
  end

  /* Collect commands */
  if what = 'L' then do
    'EXTRACT /LINE/'
    cmd.1 = line.1
    cmd.0 = 1
  end
  else do
    if mark.6 = 0 then do /* mark is in another file */
      'EXTRACT /NAME/'
      'X' mark.1
    end
    j=0
    do i=mark.2 to mark.3
      'EXTRACT /LINE 'i'/'
      j=j+1
      cmd.j=line.1
    end
    cmd.0=j
    if mark.6 = 0 then do /* mark is in another file */
      'X' name.1
    end
  end

  /* Execute commands */
  do i=1 to cmd.0
    if cmd.i <> '' then do
      ''cmd.i
      'REFRESH'
    end
  end

  'MSG DATACMD: Done'

  return(0)
