/*

  handle input (enter key) defined syntax expansion for C files
  also called by the standard space expansion macro

  Author: Sam Detweiler
          June 6, 1999
*/

/* default syntax indent assumed from standard xprofile.def */
syntax_indent  = 2
true = 'on'

  /* get the first word and the rest */
  parse value arg(1) with indent keyword rest
  /* get JUST the text and uppercase it */
  parse value translate(keyword,'    ABCDEFGHIJKLMNOPQRSTUVWXYZ','();:abcdefghijklmnopqrstuvwxyz') with ukeyword .
  /* space fill length of keyword on line */
  fill=copies(' ',indent)
  /* space fill length of indented keywords */
  pad=copies(' ',indent+syntax_indent)
Select
  When ukeyword = 'IF' Then do
    /* delete the blank line just inserted */
    'delete'
    Select
      /* if there is just one token on the line */
      When  rest='' Then do
        /* back to the original line */
        'up'
        /* remember that keyword is SPACE delimited word */
        /* if a complete prototype */
        If right(keyword,2)='()' Then do
         /* replace the orignal line */
         'replace' fill || 'if()'
         call cursor_in_paren -1 , true
          end
        else do
          /* see if we have an expression, or just spaces */
          parse var keyword . '(' expression ')' .
          If expression<>'' Then do
            /* down a line */
            /* woo hoo!! we have something in the parens */
            /* the programmer can type!! */
            /* give him another line
               note that the enter key already moved down 1 line
            */
            'down'
            /* insert a new line */
            'input'
            /* move the cursor */
            'cursor +0 -' || syntax_indent
            /* set insert mode off */
            'insmode off'
          end
          Else do
            /* watch out for the space expansion
               on the first line of the file */
            'extract /line/'
            if line.1<>'if' then
              'down'
            /* replace the orignal line */
            /* and insert the full prototype */
            'replace' fill ||keyword || '()'
            "input" pad || "{"
            "input" pad || "} /* end if */"
            "input" fill || "else"
            "input" pad || "{"
            "input" pad || "} /* end else */"
            'up 5'
            call cursor_in_paren -1, true
          end
        end
      end
      /* if this is the empty prototype line */
      When rest='()'| right(keyword,2)='()' then do
        /* move back up to the original line */
        'up'
        /* replace the orignal line */
        'replace' fill || keyword || '()'
        call cursor_in_paren -1 , true
      end
      /* we have a line already */
      Otherwise  do
        /* get what is in the parens, if anything */
        parse arg . '(' expression ')' .
        /* see if we have an expression, or just spaces */
        If expression<>'' Then do
          /* woo hoo!! we have something in the parens */
          /* the programmer can type!! */
          /* give him another line
             note that the enter key already moved down 1 line
          */
          'input'
          /* and move there */
          'cursor +0 -' || syntax_indent
          /* set insert mode off */
          'insmode off'
        end
        Else do
          /* empty expression, but with spaces */
          /* back to the original line */
          'up'
          /* replace it, be consistant */
          'replace' fill || 'if()'
          call cursor_in_paren -1, true
        end
      end
    End                                                       /* End select*/
  end
  When ukeyword = 'FOR' then do
    /* delete the new line */
    'delete'
    /* back up one line to original */
    'up'
    /* if just the word 'for' */
    If rest='' & right(keyword,1)<>')' Then do
      /* build the initial prototype */
      'replace' fill ||keyword || '(;;)'
      "input" pad || "{"
      "input" pad || "} /* end for */"
      'up 2'
      call cursor_in_paren -3, true
    end
    Else do
      Select
        /* user hasn't filled in the for clause, wait for first part */
        When right(keyword,4)='(;;)'Then do
           call cursor_in_paren -3 , true
          end
        /* user hasn't finished the for clause, wait for next part */
        When right(keyword,3)=';;)'Then do
           call cursor_in_paren -2 , true
          end
        /* user hasn't filled in the for clause, wait for last part */
        When right(keyword,2)=';)'Then do
           call cursor_in_paren -1 , true
          end
        Otherwise do
          /* is a body for this for clause, go to line after brace */
          If right(arg(1),1)<>';' then
            'down'
          /* add new line */
          'input'
          'cursor +0 -' || syntax_indent
          'insmode off'
        end
      End                                                     /* End select*/
    end
    end
  When ukeyword = 'DO' Then do
    /* get rid of the new line */
    'delete'
    /* get next line contants */
    'down'
    'extract /line/'
    'up'
    /* if this is just a brace, */
    If line.1='{' Then do
      /* insert a line in the braces */
      'input'
      /* go there */
      'cursor +0 -' || syntax_indent
    end
    Else do
      /* first time for Do */
      /* insert the prototype */
      'up'
      'input' pad ||'{'
      'input'
      'input' pad ||'} while() /* end do */'
      /* position the cursor */
      'up'
    end
  end
  When ukeyword = 'SWITCH' then do
    /* delete the new line */
    'delete'
    /* back up to original line */
    'up'
    /* if just the keyword and no fiddling with the parens */
    If rest='' & right(keyword,1)<>')' Then do
      /* first time, insert the prototype */
      'replace' fill ||keyword || '()'
      'input' pad || "{"
      'input' pad || "case :"
      'input' pad || "  break;"
      'input' pad || "default:"
      'input' pad || "  break;"
      'input' pad || "} /* end switch */"
      'up 6'
      call cursor_in_paren -1, true
    end
    else do
      /* user hasn't finished the switch clause, wait */
      If right(keyword,2)='()' then do
        call cursor_in_paren -1, true
      end
      else do
        /* switch is finished, go to case statement */
        'down 2'
        call cursor_in_paren -1, true
      end
    end
  end
  When ukeyword = 'CASE' then do
    /* get the next line */
    'down'
    'extract /line/'
    'up'
    /* delete the inserted line */
    'delete'
    /* if the next line is break */
    If line.1='break;' Then do
      'up'
      /* if just 'case on this line */
      If keyword='case' & rest='' then DO
        /* fill in the whole keyword */
        'replace' fill || keyword ':'
        call cursor_in_paren -1, true
      end
      else do
        /* check for case constant being supplied */
        parse arg . keyword constant ':'
        /* yes, programmer does work!! */
        if constant<>'' then do
          /* insert a new line and position the cursor */
          'input' pad
          'cursor +0 +' || syntax_indent
          'insmode off'
        end
        else do
          /* wait for the programmer to DO something */
          call cursor_in_paren -1, true
        end
      end
    end
    Else do
      /* get the case constant value if any */
      parse value arg(1) with 'case' value ':' .
      if value = '' then do
        'up'
        call cursor_in_paren -1, true
      end
      else do
        /* get the left to chars of the first word on the next line */
        kw=left(strip(line.1),2)
        If kw='if' | kw='do' | kw='wh' | kw='fo'| kw='{' Then do
          of = '-' || syntax_indent
          /* if no keyword, just open brace,
             back up one line to fake it for next statement */
          If kw='{' then do
            of='-2'
            'up'
          end
          /* go past the brace of the next statement */
          'down'
          /* insert a new line */
          'input' pad
          /* position the cursor */
          'cursor +0' of
          'insmode off'
        end
        else do
          /* was a case constant, but next line is blank */
          If line.1='' then do
            /* position on the blank line
               this requies special treatment else
               the cursor will end up WAY out at the
               start of the word 'case'
            */
            'cursor +0 +' || length(keyword)
          end
          Else do
           /* case constant but next line is NOT blank,
              so insert before the next line */
           'up'
           'input' pad
           'cursor +0 +' || length(keyword)
          end
          'insmode off'
        end
      end
    end
  end
  When ukeyword = 'BREAK' then do
    /* get rid of the new line */
    'delete'
    /* get the line before where the cursor was */
    'up 2'
    'extract /line/'
    'down'
    /* was this the last section of the switch */
    If strip(line.1)<>'default:' Then do
      /* yes, add a new case statement */
      'input' left(fill,length(fill)-2)|| 'case :'
      'input' fill|| 'break;'
      'up'
      /* position at the case constant */
      call cursor_in_paren -1, true
    end
    else do
      /* was the break after default:, so wait.. */
      'insmode off'
    end
  end
  When ukeyword = 'DEFAULT' then do
    /* was the default */
    /* new line already inserted.. wait */
    'cursor +0 +' || syntax_indent
    end
  When ukeyword='{' Then do
    /* fix the default behaviour to keep the
       cursor in the right place */
    'up 2'
    'extract /line/'
    parse value translate(line.1) with keyword .
    If left(keyword,2)='IF' | left(keyword,2)='WH' then do
      'down 2'
      'cursor +0 -' || syntax_indent
      'insmode off'
    end
    else do
      'down'
      'cursor +0 -' || syntax_indent
      'insmode off'
    end
  end
  When ukeyword = 'WHILE' then do
    /* delete the inserted line */
    'delete'
    /* nothing else on the line */
    If rest='' then do
      /* not empty clause */
      If right(keyword,2)<>'()' then do
        /* still have the closing paren */
        If right(keyword,1)=')' then do
          /* get the expression value */
          parse var keyword . '(' expression ')' .
          /* oops, expression WAS empty after all */
          If expression <>'' then do
            'input'
            'cursor +0 -' || syntax_indent
            'insmode off'
          end
          else do
            'up'
            'replace' fill || keyword || '()'
            call cursor_in_paren -1, true
          end
        end
        Else do
          'up'
          'replace' fill || keyword || '()'
          'input' pad || '{'
          'input' pad || '} /* end while*/'
          'up 2'
          call cursor_in_paren -1, true
        end
      end
      else do
        /* empty while clause, wait */
        'up'
        call cursor_in_paren -1, true
      end
    end
    else do
      parse arg . '(' expression ')' .
      /* oops, expression WAS empty after all */
      If expression ='' then do
        'up'
        'replace' fill || 'while' || '()'
        call cursor_in_paren -1, true
      end
      else do
        'input'
        'cursor +0 -' || syntax_indent
        'insmode off'
      end
    end
  end
  When ukeyword='MAIN' then do
    /* delete the inserted line */
    'delete'
    /* if not last line of file */
    if rc=0 then
      /* then move up */
      'up'
    /* insert the full prototype */
    'replace' 'int ' || keyword || '(int argc ,char *argv[], char *envp[])'
    'input'  '{'
    'input'  'int rc=0;'
    'input'  'return rc;'
    'input'  '}'
    'up 2'
    'input'
    'cursor +0 +2'
  end
  Otherwise do
    /* not a special case line, delete empty line
       without delete/input, cursor won't be in right place */
    'delete'
    /* input new line */
    'input'
    /* position cursor */
    'cursor +0 -' || syntax_indent
  end
End                                                           /* End select*/
exit 0
/* common routine to put cursor inside paren or at case constant */
cursor_in_paren:
     /* end of line, stay on this line */
     'cursor eol stay'
     /* back up specifed number of chars */
     'cursor +0' arg(1)
     /* set insert mode as requested */
     'insmode' arg(2)
return

