/***************************************************************************/
/*                                                                         */
/* Write a skeleton function from the prototype.  Supported languages are  */
/* C and C++.                                                              */
/*                                                                         */
/* Written by B. Thompson, November 10, 1993                               */
/* Fix C++ code where class name contains embedded colon, June 30, 1994    */
/* Mark newly added code for easy manipulation, September 13, 1994         */
/* Identify current function in TADS diagnostics, August 27, 1996          */
/* Supply default return value, August 27, 1996                            */
/* Add optional STAY parameter, February 26, 1997                          */
/* Strip off "virtual" identifier in C++ functions, June 24, 1999          */
/* Save/restore LINEND setting, January 24, 2000                           */
/* Conditionally add __IBMC__ checking, November 26, 2001                  */
/*                                                                         */
/***************************************************************************/

  Parse Upper Arg opt .

  'EXTRACT /COMMENTS/'                      /* Get current file information*/
  'EXTRACT /MARGINS/'
  'EXTRACT /CURSOR/'

  'locate /#if defined(__IBMC__)/ f'
  ibmc = (rc = 0)
  'cursor' cursor.1 cursor.2
  comments.1 = strip(comments.1)
  If substr(comments.1,length(comments.1)) <> '*'
    Then comments.1 = comments.1 || '*'
  comments.1 = comments.1 || ' '
  If comments.2 = ''
    Then comments.2 = '*'                            /* Close a box comment*/
  fillspace = margins.4 - length(strip(comments.1)) - length(strip(comments.2))
  sep = strip(comments.1) || left('', fillspace, '*') || strip(comments.2)
  fillspace = margins.4 - length(comments.1) - length(comments.2)
  blank = comments.1 || left('', fillspace) || comments.2
  cmt_col = length(comments.1) + 1

  'extract /linend/'
  'LINEND OFF'
  'EXTRACT /FT/'
  cpp = 0                                          /* Assume this is C code*/
  Select
    When ft.1 = 'C'
      Then Nop
    When ft.1 = 'H'
      Then Nop
    When WordPos(ft.1, 'CPP HPP CPV HPV') > 0
      Then cpp = 1                                              /* C++ code*/
    Otherwise
      'linend' linend.1
      Exit                                              /* Unsupported file*/
  End                                                         /* End select*/

  Do numlines = 1                            /* Get all the prototype lines*/
    'EXTRACT /CURLINE/'
    outline.numlines = Strip(curline.1)
    semi = Pos(';',outline.numlines)
    If semi > 0
      Then Do
        outline.numlines = Substr(outline.numlines,1,semi-1)
        Leave numlines
      End
    'DOWN'
    If rc <> 0
      Then Leave                                             /* We hit EOF!*/
  End                                                             /* End do*/

  return_type = Translate(outline.1)
  If word(return_type,1) = 'INLINE'
    Then return_type = Subword(return_type,2)
  If word(return_type,1) = 'VIRTUAL'
    Then return_type = Subword(return_type,2)
  return_type = Word(return_type,1)
  Select
    When Wordpos(return_type,,
                 'INT UINT UNSIGNED LONG ULONG USHORT APIRET') > 0
      Then return_value = '0'                             /* Assume numeric*/
    When Wordpos(return_type, 'PSZ CHAR PVOID') > 0
      Then return_value = 'NULL'                    /* Assume ptr to string*/
    When return_type = 'UCHAR'
      Then return_value = '\0'                 /* Assume unsigned character*/
    When return_type = 'BOOL'
      Then return_value = 'TRUE'                          /* Assume boolean*/
    When return_type = 'BOOLEAN'
      Then return_value = 'true'                              /* Assume C++*/
    Otherwise
      return_value = ''                              /* Default return type*/
  End                                                         /* End select*/

  If cpp
    Then Do
      inline = 'inline '
      'L /class / -eq'
      Do While rc = 0                              /* Add in the class name*/
        'EXTRACT /CURLINE/'
        Parse Var curline.1 . 'class ' classname .
        If Pos(';', classname) > 0
          Then Do
            'REPFIND'
            Iterate
          End
        If Pos(':', classname) > 0
          Then Parse Var classname classname ':' .
        Parse Var outline.1 funcname '(' args
        funcname = Strip(funcname)
        If word(funcname,1) = 'virtual'
          Then funcname = Strip(subword(funcname,2))
        func_offset = Lastpos(' ',funcname)
        prefix = Substr(funcname, 1, func_offset)
        funcname = Substr(funcname, func_offset+1)
        outline.1 = prefix classname'::'funcname '('args
        Leave
      End                                                         /* End do*/
      'RESTORE_FIND'                        /* Restore previous find target*/
    End
    Else
      inline = ''

  outline.1 = Space(outline.1)
  Parse Var outline.1 funcname '(' .
  funcname = Strip(funcname)
  func_offset = Lastpos(' ',funcname)
  funcname = Substr(funcname, func_offset+1)
  lead_spaces = Pos('(',outline.1) + Length(inline)

  'TOP'                                               /* In case WRAP is ON*/
  'L /DIAG.H/'                                /* Look for DIAG include line*/
  got_diag = (rc = 0)
  'RESTORE_FIND'

  'BOTTOM'
  'MARK CLEAR'
  'INPUT'                                         /* Start with blank lines*/
  'MARK'                                             /* This is top of mark*/
  'INPUT'
  'INPUT'
  If ibmc
    Then 'INPUT #if defined(__IBMC__)'
  'INPUT #pragma page()'
  If ibmc
    Then 'INPUT #endif'
  'INPUT' sep                                            /* A comment block*/
  'INPUT' blank
  'INPUT' Overlay(funcname, blank, cmt_col)
  'INPUT' blank
  'INPUT' blank
  'INPUT' blank
  'INPUT' sep
  'INPUT'
  'INPUT' inline || outline.1                        /* Insert the 1st line*/
  Do ctr = 2 To numlines                  /* Insert remaining function text*/
    'INPUT' Left('',lead_spaces) || outline.ctr
  End                                                             /* End do*/

  'INPUT {'
  If cpp
    Then 'INPUT   IFUNCTRACE_DEVELOP();'
    Else If got_diag
      Then 'INPUT   DIAG3((s, "Starting %s function", __FUNCTION__));'
  'INPUT'
  If return_value <> ''
    Then 'INPUT   return' return_value';'
  'INPUT } /* End of' funcname 'function*/'
  'MARK'                                             /* This is end of mark*/
  If opt = 'STAY'
    Then 'CURSOR' cursor.1 cursor.2
    Else 'CURSOR -1 3'
  'linend' linend.1
Exit
