/***************************************************************************/
/*                                                                         */
/* Create standard header for C, H, SCR, and MAK source files.  This is a  */
/* sample macro which creates skeleton source files according to the       */
/* author's preferences.  It should be modified to suit your own tastes.   */
/*                                                                         */
/* This macro may be called from the editor command line, or as a syntax   */
/* expansion macro with the following lines in the profile definition      */
/* file:                                                                   */
/*                                                                         */
/*   extension          = C,H                                              */
/*   expand_keyword     = header                                           */
/*   expand_macro       = "header"                                         */
/*                                                                         */
/*   extension          = SCR                                              */
/*   expand_keyword     = header                                           */
/*   expand_macro       = "header"                                         */
/*                                                                         */
/*   extension          = MAK                                              */
/*   expand_keyword     = header                                           */
/*   expand_macro       = "header"                                         */
/*                                                                         */
/*   extension          = CMD,X                                            */
/*   expand_keyword     = header                                           */
/*   expand_macro       = "header"                                         */
/*                                                                         */
/*   extension          = RC                                               */
/*   expand_keyword     = header                                           */
/*   expand_macro       = "header"                                         */
/*                                                                         */
/* Written by B. Thompson, September 19, 1995                              */
/* Updated to input Bookmaster skeleton, December 1, 1995                  */
/* Added skeleton make file, December 21, 1995                             */
/* Add REXX skeleton, January 19, 1996                                     */
/* Updated to handle C++ compiles, March 6, 1996                           */
/* Add resource file RC skeleton, October 14, 1996                         */
/* Add header for public class headers in .hpp files, January 30, 1997     */
/* Add JAVA extension, April 21, 1997                                      */
/* Add BAT skeleton, October 24, 1997                                      */
/* Add HTML skeleton, November 29, 1997                                    */
/*                                                                         */
/***************************************************************************/

  id = 'Blair Thompson'                                          /* My name*/

  'EXTRACT /COMMENTS/'                      /* Get current file information*/
  'EXTRACT /MARGINS/'
  'EXTRACT /FILENAME/'

  'CURSOR 0'
  comments.1 = Strip(comments.1)
  cmtlen = Length(comments.1)
  If cmtlen > 0
    Then If Substr(comments.1,cmtlen) <> '*'
      Then comments.1 = comments.1 || '*'
  comments.1 = comments.1 || ' '
  If comments.2 = ''
    Then comments.2 = '*'                            /* Close a box comment*/
  fillspace = margins.4 - Length(Strip(comments.1)) - Length(Strip(comments.2))
  sep = Strip(comments.1) || Left('', fillspace, '*') || Strip(comments.2)
  fillspace = margins.4 - Length(comments.1) - Length(comments.2)
  blank = comments.1 || Left('', fillspace) || comments.2
  cmt_col = Length(comments.1) + 1
  Parse Value date('L') With dd mm yy .
  longdate = mm dd',' yy

  'EXTRACT /FT/'
  If ft.1 <> 'JAVA'
    Then filename.1 = Translate(filename.1)
  pname = lastpos('\', filename.1)
  If pname > 0
    Then filename.1 = Substr(filename.1, pname + 1)

  Select
    When Wordpos(ft.1,'C H RC CPV HPV CPP HPP JAVA') > 0
      Then Call c_header
    When ft.1 = 'SCR'
      Then Call script_header
    When ft.1 = 'MAK'
      Then Call make_header
    When ft.1 = 'BAT'
      Then Call bat_header
    When Wordpos(ft.1,'CMD X') > 0
      Then Call rexx_header
    When Wordpos(ft.1,'HTM HTML') > 0
      Then Call html_header
    Otherwise
      'MSG Unrecognised file extension'
  End                                                         /* End select*/
Exit


/***************************************************************************/
/*                                                                         */
/* Utility function to simplify adding lines within a block comment.       */
/*                                                                         */
/***************************************************************************/

INPUT_CMT:  Procedure Expose blank cmt_col
  Parse Arg linetext
  'INPUT' Overlay(linetext, blank, cmt_col + 1)
Return                                         /* End of Input_cmt function*/


/***************************************************************************/
/*                                                                         */
/* Input the header for C programming source files.                        */
/*                                                                         */
/***************************************************************************/

C_HEADER:  Procedure Expose sep blank filename. cmt_col ,
                            comments. ft. id longdate

  'INPUT' sep
  'INPUT' blank
  Call input_cmt Left(' MODULE:',17) filename.1
  'INPUT' blank
  Call input_cmt Left(' APPLICATION:',17) 'App Name'
  Call input_cmt Left(' AUTHOR:',17) id
  Call input_cmt Left(' DATE:',17) longdate
  'INPUT' blank
  Call input_cmt Left(' DESCRIPTION:',17)
  'INPUT' blank
  'INPUT' blank
  If ft.1 = 'RC'
    Then Do
      Call input_cmt 'A naming convention is used for constant names.  Each control has a'
      Call input_cmt 'unique prefix:'
      'INPUT' blank
      Call input_cmt '  Prefix    Type'
      Call input_cmt '  IDB_      Button'
      Call input_cmt '  IDLB_     ListBox'
      Call input_cmt '  IDMI_     Menu Item'
      Call input_cmt '  IDM_      Menu'
      Call input_cmt '  IDRB_     Radio Button'
      'INPUT' blank
      'INPUT' blank
    End                                                           /* End do*/
  Call input_cmt 'Change History:'
  'INPUT' blank
  Call input_cmt 'Rel   Pgmr  Date      Description'
  Call input_cmt '----  ----  -------- ' Left('', 40, '-')
  Call input_cmt '1.0   BWT  ' Date('O') ' Initial version'
  'INPUT' blank
  'INPUT' sep

  If ft.1 = 'JAVA'
    Then Return

  If ft.1 = 'HPP'
    Then Call input_block('Classes defined in this header file')
  If ft.1 <> 'RC'
    Then Do
      'INPUT'
      'INPUT'
    End

  If ft.1 = 'H' | ft.1 = 'HPP'
    Then Do                                                /* C header file*/
      def_name = Translate(filename.1,'_','.')
      'INPUT #if (!defined('def_name'))'
      'INPUT #define' def_name
      If ft.1 = 'HPP'
        Then Do
          Call input_block('Required public classes')
          Call input_block('Forward declarations')
          Call input_block('Class Definition')
        End
        Else
          Call input_block('Function Prototypes')
      'INPUT'
      'INPUT #endif'
      Return
    End

  If ft.1 <> 'RC'
    Then 'INPUT #pragma strings(readonly)'

  If ft.1 <> 'CPP'
    Then Do
      Call input_block('Operating System specific includes and definitions')

      If ft.1 <> 'RC'
        Then Do
          'INPUT'
          'INPUT #define  INCL_NOCOMMON  ' comments.1 "Don't give me anything by default" comments.2
        End
      'INPUT'
      'INPUT #include <os2.h>'

      If ft.1 <> 'RC'
        Then Do
          Call input_block('C runtime header files')

          'INPUT'
          'INPUT #include <stdio.h> ' comments.1 'Defines NULL' comments.2
          'INPUT #include <stdlib.h>' comments.1 'Memory management, sorting' comments.2
          'INPUT #include <string.h>' comments.1 'String manipulation functions' comments.2
        End
    End

  If ft.1 = 'CPP'
    Then Call input_block('Required public classes')
  Call input_block('Application function headers')
  Parse Var filename.1 fn '.' .                          /* Strip off extension*/
  fn = Translate(fn, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
  If ft.1 = 'CPP'
    Then ext = 'hpp'
    Else ext = 'h'                                              /* Assume C*/
  'INPUT'
  'INPUT #include "'fn'.'ext'"'

  If ft.1 = 'RC'
    Then Return                                         /* RC resource file*/

  call input_block('Defined constants')
  call input_block('Global variables')
  call input_block('Function Prototypes')
Return


/***************************************************************************/
/*                                                                         */
/* Input the header for REXX command files and X2 macros.                  */
/*                                                                         */
/***************************************************************************/

REXX_HEADER:  Procedure Expose sep blank filename. cmt_col id longdate

  'INPUT' sep
  'INPUT' blank
  'INPUT' Overlay(filename.1, blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('Written by' id',' longdate, blank, cmt_col)
  'INPUT' blank
  'INPUT' sep
  'INPUT'
  'INPUT'
  'INPUT'
  'INPUT Exit'
Return


/***************************************************************************/
/*                                                                         */
/* Input a commented block surrounding a single line of text.              */
/*                                                                         */
/***************************************************************************/

INPUT_BLOCK:  Procedure Expose blank cmt_col sep
  Parse Arg instring

  'INPUT'
  'INPUT'
  'INPUT' sep
  'INPUT' blank
  'INPUT' Overlay(instring, blank, cmt_col)
  'INPUT' blank
  'INPUT' sep
Return


/***************************************************************************/
/*                                                                         */
/* Input the header for SCRIPT (Bookmaster) source files.                  */
/*                                                                         */
/***************************************************************************/

SCRIPT_HEADER:  Procedure

  'INPUT :userdoc.'
  'INPUT :prolog.'
  'INPUT :docprof style=IBMXAGD layout=1.'
  'INPUT :eprolog.'
  'INPUT :body.'
  'INPUT :h1.'
  'INPUT :euserdoc.'
Return


/***************************************************************************/
/*                                                                         */
/* Input the header for a make file.                                       */
/*                                                                         */
/* Add prompt for executable type, August 25, 1999                         */
/*                                                                         */
/***************************************************************************/

MAKE_HEADER:  Procedure Expose filename. comments. sep blank cmt_col ,
                               id longdate

  Parse Var filename.1 fn '.' .
  'WINDOW 2 50 2 Select Executable Type'
  'WINLINE Text (VIO) \nSETRESULT VIO'
  'WINLINE GUI (PM) \nSETRESULT PM'
  'WINWAIT'
  If rc <> 0
    Then app_type = 'PM'
    Else app_type = result

  'INPUT' sep
  'INPUT' blank
  'INPUT' Overlay(Left('MODULE:',16) filename.1, blank, cmt_col + 2)
  'INPUT' blank
  'INPUT' Overlay(Left('APPLICATION:',16) 'App Name', blank, cmt_col + 2)
  'INPUT' Overlay(left('AUTHOR:',16) id, blank, cmt_col + 2)
  'INPUT' Overlay(Left('DATE:',16) longdate, blank, cmt_col + 2)
  'INPUT' blank
  'INPUT' Overlay('We generate the executable, with the following steps:', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('  1. Set up our compile and link options depending on whether we want to', blank, cmt_col)
  'INPUT' Overlay('     compile with optimisation turned on, etc.', blank, cmt_col)
  'INPUT' Overlay('  2. Set up our inference rules.  If no build command follows a', blank, cmt_col)
  'INPUT' Overlay('     dependency list, the default inference rules will be used to build', blank, cmt_col)
  'INPUT' Overlay('     a target.', blank, cmt_col)
  'INPUT' Overlay('  3. Generate the EXE from all the required object decks and resource', blank, cmt_col)
  'INPUT' Overlay('     files.  Specify the dependency list for each of our output files.', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('C Set++ compiler option flags:', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('  /Fi+ :  Create a pre-compiled header file if none exists or if the', blank, cmt_col)
  'INPUT' Overlay('          existing one is out of date.', blank, cmt_col)
  'INPUT' Overlay('  /Ft- :  Do not generate template code into a TEMPINC directory', blank, cmt_col)
  'INPUT' Overlay('  /G4  :  Optimize code for use with a 486 processor.  The code will run', blank, cmt_col)
  'INPUT' Overlay('          on a 386 or Pentium microprocessor.  The compiler includes any', blank, cmt_col)
  'INPUT' Overlay('          Pentium microprocessor optimizations that do not detract from', blank, cmt_col)
  'INPUT' Overlay('          the performance on the 486 processor.', blank, cmt_col)
  'INPUT' Overlay('  /Gd+ :  Dynamically link runtime libraries.  The default is Gd-.', blank, cmt_col)
  'INPUT' Overlay('  /Gd- :  Compile code for a statically linked executable.', blank, cmt_col)
  'INPUT' Overlay('  /Ge+ :  The default, generate an EXE file.  Ge- creates a DLL file.', blank, cmt_col)
  'INPUT' Overlay('  /Gh+ :  Enable code to be run by EXTRA and other profiling tools by', blank, cmt_col)
  'INPUT' Overlay('          generated profiler hooks in function prologs.', blank, cmt_col)
  'INPUT' Overlay('  /Gi+ :  Use fast integer execution.', blank, cmt_col)
  'INPUT' Overlay('  /Gm+ :  Link with the multithread library, instead of the', blank, cmt_col)
  'INPUT' Overlay('          single-thread library.', blank, cmt_col)
  'INPUT' Overlay('  /Gs+ :  Remove stack probes.  This option can only safely be used if', blank, cmt_col)
  'INPUT' Overlay('          the application will have only one thread.', blank, cmt_col)
  'INPUT' Overlay('  /Gs- :  Do not remove stack probes.  (This is the default.)', blank, cmt_col)
  'INPUT' Overlay('  /O+  :  Optimize code.', blank, cmt_col)
  'INPUT' Overlay('  /O-  :  Do not optimize code.', blank, cmt_col)
  'INPUT' Overlay('  /Oc+ :  Optimize code to create a small executable', blank, cmt_col)
  'INPUT' Overlay('  /Q+  :  Quiet, suppress copyright information', blank, cmt_col)
  'INPUT' Overlay('  /Rn  :  Generate executable code that can be used as a sub-system', blank, cmt_col)
  'INPUT' Overlay('          without a runtime environment.  Requires the code to be linked', blank, cmt_col)
  'INPUT' Overlay('          with the DDE4NBS library.', blank, cmt_col)
  'INPUT' Overlay('  /Sa  :  Conform to ANSI standards.', blank, cmt_col)
  'INPUT' Overlay('  /Se  :  Allow all VisualAge for C++ language extensions. (This is the', blank, cmt_col)
  'INPUT' Overlay('          default.)', blank, cmt_col)
  'INPUT' Overlay('  /Sh- :  Allow use of ddnames.', blank, cmt_col)
  'INPUT' Overlay('  /Si+ :  Use pre-compiled header files if they exist and are current.', blank, cmt_col)
  'INPUT' Overlay('  /Tdp :  Compile source file as C++ code', blank, cmt_col)
  'INPUT' Overlay('  /Ti+ :  Generate debugger information.', blank, cmt_col)
  'INPUT' Overlay('  /Ti- :  Do not generate debugger information.', blank, cmt_col)
  'INPUT' Overlay('  /Tm  :  Use the debug memory management functions.  This option also', blank, cmt_col)
  'INPUT' Overlay('          defines the "__DEBUG_ALLOC__" macro.', blank, cmt_col)
  'INPUT' Overlay('  /W3  :  Highest possible warning level', blank, cmt_col)
  'INPUT' Overlay('  /Wgrp:  Warning groups:', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('          /Wall :  All diagnostics.', blank, cmt_col)
  'INPUT' Overlay('          /Wcls :  Use of classes.      EDC3110, EDC3253, EDC3266', blank, cmt_col)
  'INPUT' Overlay('          /Wcmp :  Possible             EDC3138, EDC3140', blank, cmt_col)
  'INPUT' Overlay('                   redundancies in', blank, cmt_col)
  'INPUT' Overlay('                   unsigned', blank, cmt_col)
  'INPUT' Overlay('                   comparisons.', blank, cmt_col)
  'INPUT' Overlay('          /Wcnd :  Possible             EDC0424, EDC0425, EDC0426,', blank, cmt_col)
  'INPUT' Overlay('                   redundancies or      EDC0427, EDC0420, EDC0421,', blank, cmt_col)
  'INPUT' Overlay('                   problems in          EDC0422, EDC0423, EDC3107,', blank, cmt_col)
  'INPUT' Overlay('                   conditional          EDC3130, EDC3388, EDC3389,', blank, cmt_col)
  'INPUT' Overlay('                   expressions.         EDC3390, EDC3391, EDC3392,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC3393', blank, cmt_col)
  'INPUT' Overlay('          /Wcns :  Operations involving EDC0475, EDC0476, EDC0477,', blank, cmt_col)
  'INPUT' Overlay('                   constants.           EDC3131, EDC3219, EDC3220', blank, cmt_col)
  'INPUT' Overlay('          /Wcnv :  Conversions.         EDC3313, EDC3528', blank, cmt_col)
  'INPUT' Overlay('          /Wcpy :  Problems generating  EDC3199, EDC3200', blank, cmt_col)
  'INPUT' Overlay('                   copy constructors.', blank, cmt_col)
  'INPUT' Overlay('          /Weff :  Statements with no   EDC0509, EDC0435, EDC0436,', blank, cmt_col)
  'INPUT' Overlay('                   effect.              EDC0437, EDC0473, EDC0474,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC0478, EDC0479, EDC0483,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC3165, EDC3215', blank, cmt_col)
  'INPUT' Overlay('          /Wenu :  Consistency of enum  EDC0439, EDC0440, EDC0471,', blank, cmt_col)
  'INPUT' Overlay('                   variables.           EDC3137, EDC3366', blank, cmt_col)
  'INPUT' Overlay('          /Wext :  Unused external      EDC0415, EDC0493, EDC0494,', blank, cmt_col)
  'INPUT' Overlay('                   definitions.         EDC3127', blank, cmt_col)
  'INPUT' Overlay('          /Wgen :  General diagnostics. EDC0438, EDC0448, EDC0466,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC0480, EDC0489, EDC0492,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC3101', blank, cmt_col)
  'INPUT' Overlay('          /Wgnr :  Generation of        EDC3151', blank, cmt_col)
  'INPUT' Overlay('                   temporary variables.', blank, cmt_col)
  'INPUT' Overlay('          /Wgot :  Usage of goto        EDC0413', blank, cmt_col)
  'INPUT' Overlay('                   statements.', blank, cmt_col)
  'INPUT' Overlay('          /Wini :  Possible problems    EDC0444, EDC0445, EDC0446,', blank, cmt_col)
  'INPUT' Overlay('                   with initialization. EDC0447, EDC0482', blank, cmt_col)
  'INPUT' Overlay('          /Winl :  Functions not        EDC3542', blank, cmt_col)
  'INPUT' Overlay('                   inlined.', blank, cmt_col)
  'INPUT' Overlay('          /Wlan :  Effects of the       EDC3116', blank, cmt_col)
  'INPUT' Overlay('                   language level.', blank, cmt_col)
  'INPUT' Overlay('          /Wobs :  Features that are    EDC0450, EDC0470', blank, cmt_col)
  'INPUT' Overlay('                   obsolete.', blank, cmt_col)
  'INPUT' Overlay('          /Word :  Unspecified order of EDC0428, EDC0429, EDC0430,', blank, cmt_col)
  'INPUT' Overlay('                   evaluation.          EDC0431, EDC0432', blank, cmt_col)
  'INPUT' Overlay('          /Wpar :  Unused parameters.   EDC0414, EDC3126', blank, cmt_col)
  'INPUT' Overlay('          /Wpor :  Nonportable language EDC0433, EDC0434 EDC3108,', blank, cmt_col)
  'INPUT' Overlay('                   constructs.          EDC3133, EDC3135, EDC3136,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC3307', blank, cmt_col)
  'INPUT' Overlay('          /Wppc :  Possible problems    EDC0076, EDC0290, EDC0293,', blank, cmt_col)
  'INPUT' Overlay('                   with using the       EDC0311, EDC0312, EDC0313,', blank, cmt_col)
  'INPUT' Overlay('                   preprocessor.        EDC0389, EDC0441, EDC0442,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC0443, EDC0457, EDC0468,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC3645', blank, cmt_col)
  'INPUT' Overlay('          /Wppt :  Trace of             EDC0467', blank, cmt_col)
  'INPUT' Overlay('                   preprocessor', blank, cmt_col)
  'INPUT' Overlay('                   actions.', blank, cmt_col)
  'INPUT' Overlay('          /Wpro :  Missing function     EDC0304', blank, cmt_col)
  'INPUT' Overlay('                   prototypes.', blank, cmt_col)
  'INPUT' Overlay('          /Wrea :  Code that cannot be  EDC0472, EDC0520, EDC3119', blank, cmt_col)
  'INPUT' Overlay('                   reached.', blank, cmt_col)
  'INPUT' Overlay('          /Wret :  Consistency of       EDC0449, EDC0481', blank, cmt_col)
  'INPUT' Overlay('                   return statements.', blank, cmt_col)
  'INPUT' Overlay('          /Wtrd :  Possible truncation  EDC0374, EDC0416, EDC0418,', blank, cmt_col)
  'INPUT' Overlay('                   or loss of data or   EDC0419, EDC0451, EDC0452,', blank, cmt_col)
  'INPUT' Overlay('                   precision.           EDC0453, EDC0495, EDC3108,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC3135, EDC3136', blank, cmt_col)
  'INPUT' Overlay('          /Wtru :  Variable names       EDC0484', blank, cmt_col)
  'INPUT' Overlay('                   truncated by the', blank, cmt_col)
  'INPUT' Overlay('                   compiler.', blank, cmt_col)
  'INPUT' Overlay('          /Wund :  Casting of pointers  EDC3098, EDC3397, EDC3405,', blank, cmt_col)
  'INPUT' Overlay('                   to or from an        EDC3406', blank, cmt_col)
  'INPUT' Overlay('                   undefined class.', blank, cmt_col)
  'INPUT' Overlay('          /Wuni :  Uninitialized        EDC0412', blank, cmt_col)
  'INPUT' Overlay('                   variables.', blank, cmt_col)
  'INPUT' Overlay('          /Wuse :  Unused auto and      EDC0409, EDC0410, EDC0469,', blank, cmt_col)
  'INPUT' Overlay('                   static variables.    EDC0490, EDC0491, EDC3002,', blank, cmt_col)
  'INPUT' Overlay('                                        EDC3099, EDC3100, EDC3101', blank, cmt_col)
  'INPUT' Overlay('          /Wvft :  Generation of        EDC3280, EDC3281, EDC3282', blank, cmt_col)
  'INPUT' Overlay('                   virtual function', blank, cmt_col)
  'INPUT' Overlay('                   tables.', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('LINK386 linker flags:', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('  /ALIGNMENT:4        :  Set the alignment factor in the executable to', blank, cmt_col)
  'INPUT' Overlay('                         4, the recommended value', blank, cmt_col)
  'INPUT' Overlay('  /BASE:0x10000       :  The base address for the executable.  Should', blank, cmt_col)
  'INPUT' Overlay('                         NOT be used when generating a DLL', blank, cmt_col)
  'INPUT' Overlay('  /DEBUG              :  With this option, LINK386 embeds symbolic data', blank, cmt_col)
  'INPUT' Overlay('                         and line number information in the executable', blank, cmt_col)
  'INPUT' Overlay('                         output file.', blank, cmt_col)
  'INPUT' Overlay('  /EXEPACK            :  EXEPACK causes pages of data in the file to be', blank, cmt_col)
  'INPUT' Overlay('                         compressed.  The OS/2 Applications Loader will', blank, cmt_col)
  'INPUT' Overlay('                         automatically decompress these pages when the', blank, cmt_col)
  'INPUT' Overlay('                         program is run.', blank, cmt_col)
  'INPUT' Overlay('  /FARCALLTRANSLATION :  This option causes LINK386 to optimize far-call', blank, cmt_col)
  'INPUT' Overlay('                         instructions made from one segment to a target', blank, cmt_col)
  'INPUT' Overlay('                         address in the same segment.', blank, cmt_col)
  'INPUT' Overlay('  /LINENUMBERS        :  This option includes source file line numbers', blank, cmt_col)
  'INPUT' Overlay('                         and associated addresses in the map file.', blank, cmt_col)
  'INPUT' Overlay('  /MAP                :  This option lists in the map file all public', blank, cmt_col)
  'INPUT' Overlay('                         (global) symbols defined in the object files.', blank, cmt_col)
  'INPUT' Overlay('  /NOEXTDICTIONARY    :  This option prevents LINK386 from searching the', blank, cmt_col)
  'INPUT' Overlay('                         extended dictionary, an internal list of symbol', blank, cmt_col)
  'INPUT' Overlay('                         locations included with libraries generated', blank, cmt_col)
  'INPUT' Overlay("                         with the old LIB utility's /NOE option.", blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('                         Normally, LINK386 uses the extended dictionary', blank, cmt_col)
  'INPUT' Overlay('                         to speed up library searches; thus, using /NOE', blank, cmt_col)
  'INPUT' Overlay('                         slows LINK386.  This option should be used when', blank, cmt_col)
  'INPUT' Overlay('                         a library symbol is redefined.  You need to use', blank, cmt_col)
  'INPUT' Overlay('                         this option when LINK386 issues error L2044.', blank, cmt_col)
  'INPUT' Overlay('  /NOFREE             :  Undocumented option, allows plus signs in ILINK', blank, cmt_col)
  'INPUT' Overlay('                         response file.  Must be first option specified.', blank, cmt_col)
  'INPUT' Overlay('  /NOIGNORECASE       :  This option turns case sensitivity on; that is,', blank, cmt_col)
  'INPUT' Overlay('                         LINK386 treats ABC, abc, and Abc as unique', blank, cmt_col)
  'INPUT' Overlay('                         names.  By default, case sensitivity is off.', blank, cmt_col)
  'INPUT' Overlay('  /NOLOGO             :  This option disables the sign-on banner', blank, cmt_col)
  'INPUT' Overlay('                         displayed when LINK386 starts.', blank, cmt_col)
  'INPUT' Overlay('  /PACKCODE           :  This option groups neighboring code segments.', blank, cmt_col)
  'INPUT' Overlay('  /PACKDATA           :  This option groups neighboring data segments.', blank, cmt_col)
  'INPUT' Overlay('  /PMTYPE:PM          :  This option specifies the type of application', blank, cmt_col)
  'INPUT' Overlay('                         being generated.  The value can be PM, VIO, or', blank, cmt_col)
  'INPUT' Overlay('                         NOVIO.', blank, cmt_col)
  'INPUT' Overlay('  /STACK:32768        :  This option controls the stack size (in bytes)', blank, cmt_col)
  'INPUT' Overlay('                         of your program.', blank, cmt_col)
  'INPUT' Overlay('  /WARNFIXUP          :  This option directs LINK386 to issue a warning', blank, cmt_col)
  'INPUT' Overlay('                         for each segment-relative fix-up of', blank, cmt_col)
  'INPUT' Overlay('                         location-type offset when the segment is', blank, cmt_col)
  'INPUT' Overlay('                         contained within a group, but not at the', blank, cmt_col)
  'INPUT' Overlay('                         beginning.', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('The following runtime flags are used to speed up the build process, but', blank, cmt_col)
  'INPUT' Overlay('are not required to just generate the executables:', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('  CPP   :  Use conditional logic for compiling C++ source code', blank, cmt_col)
  'INPUT' Overlay('  OPT   :  Turn on optimisation.  The code will be optimised for speed', blank, cmt_col)
  'INPUT' Overlay('           and reduced size', blank, cmt_col)
  'INPUT' Overlay('  RUN   :  Automatically invoke the main executable after a successful', blank, cmt_col)
  'INPUT' Overlay('           compile', blank, cmt_col)
  'INPUT' Overlay('  LST   :  Generate listing files when compiling', blank, cmt_col)
  'INPUT' Overlay('  DEBUG :  Invoke IPMD on the main executable after a successful compile', blank, cmt_col)
  'INPUT' Overlay("  NOHALT:  Don't auto-edit file in case of error.  Best used with the /I", blank, cmt_col)
  'INPUT' Overlay('           NMAKE switch', blank, cmt_col)
  'INPUT' Overlay("  NOLINK:  Compile source code only - don't link", blank, cmt_col)
  'INPUT' Overlay("  WIN   :  Compile on Windows NT/95 platform", blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay("The runtime flags don't have to be hardcoded within this file; they can", blank, cmt_col)
  'INPUT' Overlay('be specified on the OS/2 command line as follows:', blank, cmt_col)
  'INPUT' blank
  'INPUT' Overlay('  NMAKE /F' filename.1 'OPT=1 RUN=1', blank, cmt_col)
  'INPUT' blank
  'INPUT' sep
  'INPUT'
  'INPUT'
  'INPUT CPP      = 0'
  'INPUT OPT      = 0'
  'INPUT RUN      = 0'
  'INPUT LST      = 0'
  'INPUT DEBUG    = 0'
  'INPUT NOHALT   = 0'
  'INPUT NOLINK   = 0'
  'INPUT WIN      = 0'
  call input_block('Abbreviations, set up for either debugging or optimisation')
  'INPUT'
  'INPUT # COPTS    = /C /Kabcegopr /W3 /Q+ /G4 /Fi+ /Si+ /Sh- /Gi+ /Gm+'
  'INPUT CPPOPTS  = /Wall+cls-cmp-cnv-cpy-eff-ext-gen-gnr-ini-inl-ppc-ppt-trd-uni-vft- \'
  'INPUT            /Fi+ /Ft- /G4 /Gi+ /Gm /Q+ /Sh- /Si+ /Tdp /W3'
  'INPUT COPTS    = /Wall+ext-gen-ini-ppt-trd-uni- \'
  'INPUT            /Fi+ /G4 /Gi+ /Gs+ /Q+ /Sh- /Si+ /W3'
  'INPUT DLLOPTS  = /Ge- /Gm /Gs-'
  'INPUT'
  'INPUT !IF $(OPT) == 1'
  'INPUT OBJ      = opt'
  'INPUT EXENAME  =' fn
  'INPUT OPTOPTS  = /O+ /Ti- /Gd-'
  'INPUT !ELSE'
  'INPUT OBJ      = obj'
  'INPUT CV       = /DEBUG /DBGPACK'
  'INPUT EXENAME  = T'fn
  'INPUT OPTOPTS  = /O- /Ti+ /Tm /Gd+ /DIC_TRACE_ALL'
  'INPUT !ENDIF'
  'INPUT'
  'INPUT !IF $(LST) == 1'
  'INPUT CPPOPTS  = $(CPPOPTS) /Lt+ /Ls+ /Lp90'
  'INPUT COPTS    = $(COPTS) /Lt+ /Ls+ /Lp90'
  'INPUT !ENDIF'
  'INPUT'
  'INPUT CC       = ICC.EXE'
  'INPUT LINK     = ILINK.EXE'
  'INPUT LINKOPTS = /LINENUMBERS /MAP /NOLOGO /PMTYPE:'app_type
  'INPUT EXELINK  = /BASE:0x10000'
  'INPUT'
  'INPUT !IF $(WIN) == 1'
  'INPUT OS       = WINDOWS'
  'INPUT CPPOPTS  = $(CPPOPTS) /DWINDOWS /D__BOOLEAN__'
  'INPUT LINKOPTS = $(LINKOPTS) /NOEXTDICTIONARY /NOFIXED'
  'INPUT OUT      = /out:'
  'INPUT RC       = IRC'
  'INPUT DEBUGGER = IDEBUG'
  'INPUT !ELSE                                  # OS/2'
  'INPUT OS       = OS2'
  'INPUT PLUS     = +'
  'INPUT CPPOPTS  = $(CPPOPTS) /DOS2'
  'INPUT LINKOPTS = /NOFREE $(LINKOPTS) /NOIGNORECASE /ALIGNMENT:4 /STACK:32768'
  'INPUT MAP      = $*.map;'
  'INPUT RC       = RC'
  'INPUT DEBUGGER = IPMD'
  'INPUT !IF $(OPT) == 1'
  'INPUT CV       = /EXEPACK /PACKCODE /PACKDATA'
  'INPUT OPTOPTS  = $(OPTOPTS) /Sa'
  'INPUT !ENDIF'
  'INPUT !IF $(NOHALT) == 0'
  'INPUT !IF $(CPP) == 1'
  'INPUT REDIR    = 2>&1 || x $*.cpp -ERR'
  'INPUT !ELSE'
  'INPUT REDIR    = 2>&1 || x $*.c -ERR'
  'INPUT !ENDIF'
  'INPUT !ENDIF'
  'INPUT !ENDIF'
  'INPUT'
  'INPUT !IF $(WIN) == 1'
  'INPUT LIBS =' fn'.res'
  'INPUT !ELSE                                  # OS/2'
  'INPUT LIBS = ;'
  'INPUT !ENDIF'
  call input_block('Inference rules')
  'INPUT'
  'INPUT .SUFFIXES: .opt .rc .lib .def .ipf .ico .wco .bmp .wmp'
  'INPUT'
  'INPUT .cpp.obj:'
  'INPUT   $(CC) $(CPPOPTS) $(OPTOPTS) /C $*.cpp >$*.err $(REDIR)'
  'INPUT   @attrib -a $@'
  'INPUT   @attrib -a $*.err'
  'INPUT'
  'INPUT .cpp.opt:'
  'INPUT   $(CC) $(CPPOPTS) $(OPTOPTS) /C /Fo$*.opt $*.cpp >$*.err $(REDIR)'
  'INPUT   @attrib -a $@'
  'INPUT   @attrib -a $*.err'
  'INPUT'
  'INPUT .c.obj:'
  'INPUT   $(CC) $(COPTS) $(OPTOPTS) /C $*.c >$*.err $(REDIR)'
  'INPUT   @attrib -a $@'
  'INPUT   @attrib -a $*.err'
  'INPUT'
  'INPUT .c.opt:'
  'INPUT   $(CC) $(COPTS) $(OPTOPTS) /C /Fo$*.opt $*.c >$*.err $(REDIR)'
  'INPUT   @attrib -a $@'
  'INPUT   @attrib -a $*.err'
  'INPUT'
  'INPUT .ico.wco:'
  'INPUT    ibmpcnv.exe $*.ico $*.wco'
  'INPUT    @attrib -a $@'
  'INPUT'
  'INPUT .bmp.wmp:'
  'INPUT    ibmpcnv.exe $*.bmp $*.wmp'
  'INPUT    @attrib -a $@'
  'INPUT'
  'INPUT .rc.res:'
  'INPUT !IF $(WIN) == 1'
  'INPUT    irccnv $*.rc $*.rcx'
  'INPUT    $(RC) -dWINDOWS $*.rcx'
  'INPUT    @attrib -a $*.rcx'
  'INPUT !ELSE'
  'INPUT    $(RC) -r /D$(OS) $*.RC'
  'INPUT !ENDIF'
  'INPUT    @attrib -a $@'
  'INPUT'
  'INPUT .def.lib:'
  'INPUT    IMPLIB $@ $*.DEF'
  'INPUT    @attrib -a $@'
  'INPUT'
  'INPUT .ipf.hlp:'
  'INPUT    ICC.EXE /P $*.IPF'
  'INPUT    IPFC $*.I /q'
  'INPUT    @erase $*.I'
  'INPUT    @attrib -a $@'
  call input_block('Define all the executables we will build in this make file.')
  'INPUT'
  'INPUT ALL: $(EXENAME).EXE'
  'INPUT !IF $(RUN)'
  'INPUT   $(EXENAME) >$(EXENAME).out 2>&1'
  'INPUT !ENDIF'
  'INPUT !IF $(DEBUG)'
  'INPUT   $(DEBUGGER) $(EXENAME)'
  'INPUT !ENDIF'
  call input_block('Statements to build the main executable.')
  'INPUT'
  'INPUT $(EXENAME).exe :  'fn'.$(OBJ)     \'
  'INPUT                   'fn'.res        \'
  'INPUT                   'filename.1
  'INPUT !IF $(NOLINK) == 0'
  'INPUT   $(LINK) $(LINKOPTS) $(EXELINK) $(CV) @<<'
  'INPUT     'fn'.$(OBJ)'
  'INPUT     $(OUT)$@'
  'INPUT     $(MAP)'
  'INPUT     $(LIBS)'
  'INPUT <<'
  'INPUT !ENDIF'
  'INPUT   @attrib -a $*.map'
  'INPUT   @attrib -a $@'
  'INPUT'
  'INPUT 'fn'.$(OBJ):   'fn'.c'
  'INPUT'
  'INPUT 'fn'.res:   'fn'.rc'
  'INPUT'
Return


/***************************************************************************/
/*                                                                         */
/* Skeleton for a DOS batch file - it will call a REXX CMD file with the   */
/* same name                                                               */
/*                                                                         */
/***************************************************************************/

BAT_HEADER:  Procedure
  'INPUT @ECHO %ECHO%'
  'INPUT if .%9 == . goto ok'
  'INPUT ECHO %0: Too many parameters (max: 8)'
  'INPUT goto exit'
  'INPUT :ok'
  'INPUT REXX %0.CMD %1 %2 %3 %4 %5 %6 %7 %8'
  'INPUT :exit'
Return


/***************************************************************************/
/*                                                                         */
/* Skeleton for HTML source files.                                         */
/*                                                                         */
/***************************************************************************/

HTML_HEADER:  Procedure Expose sep blank filename. cmt_col id longdate
  'INPUT <!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">'
  'INPUT' sep
  'INPUT' blank
  'INPUT' Overlay(filename.1, blank, cmt_col + 2)
  'INPUT' blank
  'INPUT' Overlay('Written by' id',' longdate, blank, cmt_col + 2)
  'INPUT' blank
  'INPUT' sep
  'INPUT'
  'INPUT <HTML>'
  'INPUT <BODY bgcolor="#FFFFFF">'
  'INPUT <H1 align=center>'
  'INPUT   Main Title'
  'INPUT </H1>'
  'INPUT'
  'INPUT </BODY>'
  'INPUT </HTML>'
Return
