/************************************ HEXT *********************************/
/*                           X2 Editor Macro HEXT                          */
/*                                  HEXType                                */
/*                                                                         */
/* HEXT:  An editor macro to impersonate the ISPF editor's HEX ON VERT,    */
/* HEX ON DATA, and HEX OFF commands and the VM/CMS X editor's HEXTYPE     */
/* command and show the hexadecimal representation of a data line.  In     */
/* this version, these commands only apply to the current line.  The name  */
/* HEXT has been chosen to differentiate this macro from the editor's HEX  */
/* command and in honour of the VM/CMS Xedit command of the same name      */
/* (abbreviation).  The HEXT representations (and for HEXT DATA the        */
/* character representation of the current line) are written as X editor   */
/* error lines.  These lines may be removed with HEXT OFF provided the     */
/* cursor has not been moved to another line.  The author has assigned     */
/* this macro (just HEXT without arguments for reasons explained below,    */
/* i.e. 'cycling') to the Alt-H key which in the default configuration     */
/* invokes the editor's HEX command.                                       */
/*                                                                         */
/* CALL:                                                                   */
/*           HEXT < ? | Vert | Off | Data >                                */
/*                                                                         */
/* where all options are mutually exclusive and have the following meaning:*/
/*                                                                         */
/* HEXT ?    puts out these comments as a kind of help text.               */
/*                                                                         */
/* HEXT VERT puts out 2 lines of hex underneath the relevant characters in */
/*           the current line.                                             */
/*                                                                         */
/* HEXT DATA puts out the current line with a blank inbetween the          */
/*           characters and underneath one row of hex digits               */
/*                                                                         */
/* HEXT OFF  removes the above lines if the cursor has not been moved      */
/*           <otherwise Ctrl-O will remove all error lines).               */
/*                                                                         */
/* Invoking this macro with no parameters cycles through the above         */
/* sequence (VERT, DATA, OFF).  These parameters may be arbitrarily        */
/* abbreviated down to V, D and O.                                         */
/*                                                                         */
/* This macro is known to work under OS/2 and Windows.  It has not been    */
/* tested under the unices.  It has also not been tested with extremely    */
/* long lines likely to exceed the editor's line length limit.             */
/*                                                                         */
/* Author: Roger C. Spencer                                                */
/*                                                                         */
/* Change Activity                                                         */
/* 00-03-14 RCS New                                                        */
/* 00-09-28 RCS check for error line corrected; errortext() for syntax:    */
/***************************************************************************/

Signal Endhelpline                     /* end of help text                 */
ENDHELPLINE: lasthelpline = sigl -1

/*-------------------------------------------------------------------------*/
/* N O T E !  keep the above two lines as they are used to pick up the     */
/* number of the last line of help text to put out to a window.            */
/*-------------------------------------------------------------------------*/

Signal On Novalue; Signal On Syntax

Parse Source os . myname .             /* get this macro's name            */

false = 0; true = 1                    /* Boolean values                   */

"EXTRACT /FILEINFO/"

"EXTRACT /ESCAPE/"

/* Get the name of this macro                                              */
/* ----------------------------------------------------------------------- */

/* Define in the variable 'filespec_os' the names of systems as returned   */
/* by 'EXTRACT /OS/' that support the filespec() function.  Note that the  */
/* use of the filespec() function is a known system dependency in this     */
/* macro, however, it is protected by checking the system we are running   */
/* under.  If filespec is not supported we assume Unix (not tested).       */

filespec_os = 'OS/2 WINDOWS NT WINDOWS 95'

'EXTRACT /OS/'
If (wordpos(translate(os.1), filespec_os) = 0) Then Do
  myname = Substr(myname, Lastpos('/',myname)+1)       /* chars behind '/' */
  Parse Var myname myname '.' .        /* drop extension                   */
End                                    /* end if                           */
Else Do
  myname = Translate(Filespec('Name', myname))
  Parse Var myname myname '.' .        /* drop extension                   */
End                                    /* end else                         */

If (translate(os) = "OS/2") Then Do    /* 'selector'...                    */
  os2e = 'OS2ENVIRONMENT'              /* ...for use in 'value' bif        */
End
Else Do
  os2e = 'ENVIRONMENT'                 /* shorthand for use in 'value' bif */
End

/* check if help wanted                                                    */
/* ----------------------------------------------------------------------- */

Arg parm

'EXTRACT /FLSCREEN/'                   /* get no. of lines on screen       */
If (flscreen.0 <> 2)
  Then window_size = 18                /* set up no. of lines...           */
  Else Do                              /* ...for pop-up windows            */
    window_size = (flscreen.2 - flscreen.1 + 1)- 4
  End

help = '? /? -? HELP'                  /* Help text wanted ?      .        */
If (wordpos(parm, help) <> 0) Then Do
  window_width = Max(length(Sourceline(1)) ,   /* line 1 may be empty      */
    , Length(Sourceline(2)), Length(Sourceline(3)))
  'WINDOW' window_size window_width lasthelpline myname 'Help text'
  Do i = 1 To lasthelpline-1           /* make sure loop ends sometime     */
    'WINLINE' Sourceline(i) || '\nREFRESH'
  End
  Exit 0
End

/* Real work starts here, see to input parameter, if any                   */
/* ----------------------------------------------------------------------- */

'EXTRACT /CURLINE/'                    /* get contents of current line     */

If (parm = '') Then Do
  temp = 'VERT'; parms.temp = 'DATA'   /* cycle through in this order      */
  temp = 'DATA'; parms.temp = 'OFF'
  temp = 'OFF' ; parms.temp = 'VERT'
  parm = Value('X_HEXT', , os2e)
  If (parm = '') Then Do               /* not been set yet                 */
    parm = 'OFF'                       /* will pick up 'ON' from parms.ׯ  */
  End                                  /* end if                          */
  parm = parms.parm                    /* ]  */
  Call Value 'X_HEXT' , parm, os2e     /* write back to environment        */
End                                    /* end if                           */
Else Do
  Call Value 'X_HEXT' , '', os2e       /* clear environment variable       */
End                                    /* end else                         */

/* Now see to performing the required function                             */
/* ----------------------------------------------------------------------- */

/* Check if we are in Browse mode, if yes switch to edit mode for now      */

If (fileinfo.5 = "BROWSE") Then Do
  'BROWSE OFF'
End

Select

  When (abbrev('VERT',parm)) Then Do
    Call Value 'X_HEXT' , 'VERT', os2e /* write back to environment        */
    hex1 = ''                          /* for first hex digit              */
    hex2 = ''                          /* for second hex digit             */
    Do i = 1 To Length(curline.1)
      hexdigits = C2x(substr(curline.1,i,1))

      hex1 = hex1 || Left(hexdigits,1)
      hex2 = hex2 ||right(hexdigits,1)

    End                                /* end do over line                 */
    'INPUT_ERRORLINE' hex1
    'INPUT_ERRORLINE' hex2
    'TOPLINE' flscreen.1
    'CURSOR' fileinfo.11
    'MSG Hex lines inserted, ctrl-o, "save" or "'myname' OFF" will remove'
  End                                  /* end when                         */

  When (abbrev('DATA',parm)) Then Do
    Call Remove_errors                 /* remove any lines from HEXT DATA  */
    char = ''                          /* chars with intermediate blanks   */
    hex  = ''                          /* hex string                       */
    Do i = 1 To Length(curline.1)
      hex  = hex || C2x(substr(curline.1,i,1)) /* hex chars                */
      char = char || Substr(curline.1,i,1) || ' '
    End                                /* end do over line                 */
    'INPUT_ERRORLINE' char
    'INPUT_ERRORLINE' hex
    'TOPLINE' flscreen.1
    'CURSOR' fileinfo.11
    'MSG Hex lines inserted, ctrl-o, "save" or "'myname' OFF" will remove'
  End                                  /* end when                         */

  When (abbrev('OFF',parm)) Then Do
    Call Value 'X_HEXT' , 'OFF', os2e  /* write back to environment        */
    Call Remove_errors                 /* remove any lines from HEXT DATA  */
    'TOPLINE' flscreen.1
    'CURSOR' fileinfo.11
  End                                  /* end when                         */

  Otherwise Do
    'MESSAGEBOX' myname': unrecognised option:' parm ,
      escape.1'Nenter "'myname' ?" for help, aborting.'
    Exit 0
  End                                  /* end otherwise                    */

End                                    /* end select                       */

If (fileinfo.5 = "BROWSE") Then Do     /* revert back to browse mode?      */
  'BROWSE ON'
End

Exit 0

/* Final Exit statement                                                    */
/* ----------------------------------------------------------------------- */

Exit 0

/* utility subprocedure to remove existing hex lines if cycling through    */
/* ----------------------------------------------------------------------- */

REMOVE_ERRORS:                         /* remove any lines from HEXT data  */

If (value('X_HEXT', , os2e) = '') Then Do    /* if cleared, arg was given..*/
  Return                               /* .. on invocation, so we..        */
End                                    /* ..are not cycling through        */

'CURSOR +1'                            /* point to first hex line          */

Do 2
  'EXTRACT /LINE/'
  If (line.5 = 0) Then Do
    'MESSAGEBOX' myname': Not an error line, not deleting'
    Exit 8
  End                                  /* end if                           */
  Else Do
    'DELETE'                           /* delete one hex line              */
  End                                  /* end else                         */
End                                    /* end do                           */

'CURSOR -1'                            /* point to text line again         */

Return

/* Debugging Routines                                                      */
/* ----------------------------------------------------------------------- */

Novalue:
msg = 'Uninitialized variable:' Condition('D')
Call Invoke_ed sigl
Exit

Syntax: saverc = rc
msg = 'Rexx error' saverc ':' Errortext(rc)
Call Invoke_ed sigl
Exit

/* Put out Error Message to a file                                         */
/* ----------------------------------------------------------------------- */

INVOKE_ED: Parse Arg errline errno
Parse Source os . program              /* find the program name            */
errfile = Substr(program,1,Lastpos(".",program))"err"  /* make err file    */
If Stream(errfile,"C","Query Exists") <> ""    /* if errfile exists        */
  Then 'del' errfile                  /* delete it                        */
Call Lineout errfile, program"("errline"):" msg, 1     /* write err        */
Call Stream errfile, "C", "Close"      /* close errfile                    */
Signal Off Error                       /* turn off error trap as           */
'E' program '/err'                     /* X gives RC -1 with /err          */
Exit

/* End of Macro HEXT                                                        */
/* ----------------------------------------------------------------------- */
