/**********************************************************************/
/* Override the default EDIT and SAVE commands so we can intercept    */
/* requests to deal with host files.                                  */
/*                                                                    */
/* Our first parameter will be either EDIT or SAVE.  For EDIT, we     */
/* examine the filespec to see if it looks like a host file.  If so,  */
/* we look for spaces in it; if has spaces then it's VM so we will    */
/* use ALMCOPY if no spaces it's MVS so use SEND/RECEIVE.  Either way */
/* we download the file, load it into the editor, and rename it.      */
/* MVS filenames are examined to determine correct filetype.          */
/* For SAVE we reverse the process - save the file to disk and upload */
/* it to the host.                                                    */
/*                                                                    */
/* For either case, if the supplied file is not a host file, we use   */
/* the internal save and edit commands to process the file.           */
/*                                                                    */
/* Requires version 1.52 or later of the editor (for RING command)    */
/*                                                                    */
/* Installation:                                                      */
/*                                                                    */
/* - Download to a directory on the PATH                              */
/* - Add the following two lines to your XPROFILE:                    */
/*      synonym edit = macro hostedit edit                            */
/*      synonym save = macro hostedit save                            */
/* - Re-compile your XPROFILE, and copy X.PRO to the XPATH directory. */
/*                                                                    */
/* Written by B.  Thompson, January 13, 1994                          */
/*                                                                    */
/* Amended for MVS Toby Thurston, December 1994                       */
/* Improved version, finds files in ring, January 1995                */
/* Patch for almcopy default usage February 6, 1995                   */
/* Add Toby's STDIN mods, November 19, 1997                           */
/*                                                                    */
/**********************************************************************/

/* read the arguments */
Parse Upper Arg op fn

/* check for STDIN                                                */
if op = "EDIT" & fn = "" & lines("STDIN") > 0
  then do
    'command edit .STDIN'                                 /* edit temp file*/
    'top'                                                      /* go to top*/
    'delete *'                                       /* remove any contents*/
    Do While Lines('stdin') > 0                     /* while there is STDIN*/
     'input' Linein('stdin')                       /* input it line by line*/
    end
    'alt 0 0'                                         /* allow user to quit*/
    'top'                                                    /* back to top*/
    Exit                                                      /* exit macro*/
  end

/* check to see if it's a PC file */
Parse Var fn drive ':' file
If Substr(drive,1,1) <> 'H' | file = '' | Pos('\',fn) > 0
  Then Do
    'COMMAND' op fn
    Exit                                            /* ---> EXIT <----*/
  End
/* patch almcopy default session */
if drive = "H" then fn = "HA:"file

/* define our temporary file name */
tempfile = '$$HOST$$.$X$'

/* decide whether its VM or MVS, based on number of words in name  */
if words(fn) > 1 then do                                  /* its VM...*/
   MVS = 0
   sr_get_command = "almcopy" fn tempfile "/ascii"
   sr_put_command = "almcopy" tempfile fn "/ascii"
end
else do                                                  /* its MVS...*/
   MVS = 1
   sr_get_command = "receive" tempfile substr(fn,2) "ASCII CRLF"
   sr_put_command = "send"    tempfile substr(fn,2) "ASCII CRLF"
end
/* if we are EDITing .... */
If op = 'EDIT' Then Do
  /* look for it in the ring */
  'extract /ring/'
  if rc = 0 then do                     /* if there is a ring open....*/
    do i = 1 to ring.0
      if ring.i = fn then leave
      'command next'
    end
    if i <= ring.0 then do                   /* we have found the file*/
      if mvs then call fix_ft fn                   /* Fix the filetype*/
      exit                                           /* ---> EXIT <---*/
    end
  end
  /* now the file is not in the ring so actually get it */
  Address CMD sr_get_command
  sr_rc = rc                                   /* save the return code*/
  'COMMAND EDIT' tempfile                             /* Load the file*/
  'COMMAND NAME' fn                      /* rename it to the host name*/
  if mvs then call fix_ft fn                       /* Fix the filetype*/
  'ALT 0'                                        /* Re-set alterations*/
  If sr_rc <> 0 Then 'MSG Download return code was' sr_rc
End

If op = 'SAVE' Then Do                          /* Save a file to disk*/
  'COMMAND SAVE' tempfile
  Address CMD sr_put_command
  sr_rc = rc
  If sr_rc <> 0 Then Do                             /* Error uploading*/
    'MSG Upload return code was' sr_rc
    'ALT 1'
  End
  Else 'ALT 0'                                  /* Save was successful*/
End
Address CMD 'ERASE' tempfile                               /* Clean up*/
Exit                                                 /* ---> EXIT <---*/

fix_ft: procedure
  arg . ":" dsname                     /* strip the hx: off the dsname*/
  x = lastpos(".",dsname) + 1          /* find where the llq starts...*/
  y = pos("(",dsname,x)                  /* look for a ( after the LLQ*/
  if y = 0 then llq = substr(dsname,x)      /* if no ( then sequential*/
           else llq = substr(dsname,x,y-x)/* if ( then remove (member)*/
  select
    when llq = "SCRIPT" then 'COMMAND FT SCR' /* extend this as needed*/
    when llq = "REXX"   then 'COMMAND FT CMD'
    when llq = "EXEC"   then 'COMMAND FT CMD'
    when llq = "C"      then 'COMMAND FT C'
  otherwise
  end
return
