/***************************************************************************/
/*                                                                         */
/* Create a popup window to select and enter a C runtime include file into */
/* the current file.  The selected text is inserted after the current      */
/* line.                                                                   */
/*                                                                         */
/* Useful when assigned to a syntax expansion macro (in version 1.67 and   */
/* above):                                                                 */
/*                                                                         */
/*   expand_keyword     = inclc                                            */
/*   expand_macro       = "inclc"                                          */
/*                                                                         */
/* Written by B. Thompson, September 22, 1995                              */
/* Updated January 20, 1996 for version 1.72, to input a new line          */
/* Add C++ headers, January 28, 1997                                       */
/* Sort the entries, February 12, 1997                                     */
/* Add process control entry, October 8, 1998                              */
/*                                                                         */
/***************************************************************************/

  'EXTRACT /FT/'
  'EXTRACT /ESCAPE/'
  totlines = 0
  If Wordpos(ft.1, 'CPP HPP') > 0
    Then Do
      title = 'C++ Runtime Header Files'
      Call setup_line 'Colour class (icolor)',,
                      '#include <icolor.hpp> /* IColor*/'
      Call setup_line 'Combo Box class (icombobx)',,
                      '#include <icombobx.hpp> /* IComboBox*/'
      Call setup_line 'Command Handler class (icmdhdr)',,
                      '#include <icmdhdr.hpp> /* ICommandEvent & ICommandHandler*/'
      Call setup_line 'Container Control class (icnrctl)',,
                      '#include <icnrctl.hpp> /* IContainerControl*/'
      Call setup_line 'Entry field class (ientryfd)',,
                      '#include <ientryfd.hpp> /* IEntryField*/'
      Call setup_line 'File dialog class (ifiledlg)',,
                      '#include <ifiledlg.hpp> /* IFileDialog*/'
      Call setup_line 'File I/O library (fstream)',,
                      '#include <fstream.h> /* File I/O*/'
      Call setup_line 'Frame controls class (iframe)',,
                      '#include <iframe.hpp> /* IFrameWindow*/'
      Call setup_line 'I/O library (iostream)',,
                      '#include <iostream.h> /* C++ I/O library */'
      Call setup_line 'Key Sorted Set class (iksset)',,
                      '#include <iksset.h> /* IKeySortedSet*/'
      Call setup_line 'Key Set class (ikeyset)',,
                      '#include <ikeyset.h> /* IKeySet*/'
      Call setup_line 'Listbox control class (ilistbox)',,
                      '#include <ilistbox.hpp> /* IListBox*/'
      Call setup_line 'Menu bar class (imenubar)',,
                      '#include <imenubar.hpp> /* IMenuBar*/'
      Call setup_line 'Menu item class (imnitem)',,
                      '#include <imnitem.hpp> /* IMenuItem*/'
      Call setup_line 'Message Box class (imsgbox)',,
                      '#include <imsgbox.hpp> /* IMessageBox*/'
      Call setup_line 'Multi Cell Canvas class (imcelcv)',,
                      '#include <imcelcv.hpp> /* IMultiCellCanvas*/'
      Call setup_line 'Multi Line Edit class (imle)',,
                      '#include <imle.hpp> /* IMultiLineEdit*/'
      Call setup_line 'Pushbutton class (ipushbut)',,
                      '#include <ipushbut.hpp> /* IPushButton*/'
      Call setup_line 'Select Handler class (iselhdr)',,
                      '#include <iselhdr.hpp> /* ISelectHandler*/'
      Call setup_line 'Static Text class (istattxt)',,
                      '#include <istattxt.hpp> /* IStaticText*/'
      Call setup_line 'String class (istring)',,
                      '#include <istring.hpp> /* IString*/'
      Call setup_line 'Thread class (ithread)',,
                      '#include <ithread.hpp> /* IThread*/'
      Call setup_line 'Title class (ititle)',,
                      '#include <ititle.hpp> /* ITitle*/'
      Call setup_line 'Trace class (itrace)',,
                      '#include <itrace.hpp> /* ITrace*/'
    End
    Else
      title = 'C Runtime Header Files'

  Call setup_line 'Complex math functions (math)',,
                  '#include <math.h> /* Complex math functions */'
  Call setup_line 'Data type definitions (stddef)',,
                  '#include <stddef.h> /* Data type definitions */'
  Call setup_line 'Date & time functions (time)',,
                  '#include <time.h> /* Date & time functions */'
  Call setup_line 'Input/Output libraries (stdio)',,
                  '#include <stdio.h> /* Input/Output libraries */'
  Call setup_line 'Keyboard console interactions (conio)',,
                  '#include <conio.h> /* Keyboard console interactions */'
  Call setup_line 'Low level I/O (stream)',,
                  '#include <stream.h> /* Low level I/O */'
  Call setup_line 'Memory management, sorting (stdlib)',,
                  '#include <stdlib.h> /* Memory management, sorting */'
  Call setup_line 'Process control (process)',,
                  '#include <process.h> /* Process control */'
  Call setup_line 'String manipulation functions (string)',,
                  '#include <string.h> /* String manipulation functions */'
  Call setup_line 'TADS functions (tadsapi)',,
                  '#include <tadsapi.h> /* TADS functions */'

  'WINDOW 10 40' totlines title
  Do i = 1 To totlines
    winline.i
  End                                                             /* End do*/
  'WINSORT'
Exit


/***************************************************************************/
/*                                                                         */
/* Set up the command to create a window line, but don't issue it yet - we */
/* need to count the total number of entries before we create the window.  */
/*                                                                         */
/***************************************************************************/

SETUP_LINE:  Procedure Expose escape. totlines winline.
  Parse Arg showline, cmd
  totlines = totlines + 1
  winline.totlines = 'WINLINE' escape.1'S'showline '\nINPUT' cmd
Return
