/***************************************************************************/
/*                                                                         */
/* IPF2HTML.X                                                              */
/*                                                                         */
/* Convert a file from IPF input format into a format displayable by any   */
/* HTML browser.                                                           */
/*                                                                         */
/* This file was written specifically to convert the X Editor help file to */
/* HTML.  It is NOT a complete solution, but it's a beginning, and it does */
/* everything I need at the moment.                                        */
/*                                                                         */
/* Written by Blair Thompson, May 6, 1997                                  */
/* Use <tr> For first row in a table definition, May 16, 1997              */
/* Add translation of &gt and &lt, December 8, 1997                        */
/* Add &per translation, change &sqrt to /, December 29, 1997              */
/* Add index entries, October 19, 1999                                     */
/* Fix :note, &check., May 10, 2000                                        */
/*                                                                         */
/***************************************************************************/

  'EXT HTM'
  'WRAP OFF'
  'ALL'                                /* Make sure we're seeing everything*/
  'TOP'

  Call includes                                        /* Make one BIG file*/

  "C /</&lt;/ *"                                     /* Must be done first!*/
  "C />/&gt;/ *"
  'TOP'
  'ALL /.*/ f'                                       /* Remove any comments*/
  'DELETE *'
  'ALL'

  Call symbols
  Call title
  Call headings
  Call links
  Call tables
  Call figures
  Call artwork

  Call convert 'dd',      'dd',      0, 0
  Call convert 'dl',      'dl',      1, 0
  Call convert 'dt',      'dt',      0, 0
  Call convert 'figcap',  'caption', 0, 1
  Call convert 'hp1',     'em',      0, 0
  Call convert 'hp2',     'b',       0, 0
  Call convert 'lines',   'pre',     0, 0
  Call convert 'li',      'li',      0, 0
  Call convert 'note',    'note',    0, 0
  Call convert 'ol',      'ol',      1, 0
  Call convert 'p',       'p',       0, 0
  Call convert 'sl',      'ul',      1, 0       /* There's no <sl> in HTML!*/
  Call convert 'ul',      'ul',      1, 0
  Call convert 'userdoc', 'html',    0, 0
  Call convert 'xmp',     'pre',     0, 0

  Call delete ':docprof'
  Call index
  Call delete ':backm'

  'TOP'
  "C /&amp./&amp;/ *"
  "C /&apos./'/ *"
  "C /&asterisk./*/ *"
  "C /&bslash./\/ *"
  "C /&bxas./&#193;/ *"
  "C /&bxbj./&#193;/ *"
  "C /&bxcr./&#197;/ *"
  "C /&bxh./&#196;/ *"
  "C /&bxlj./&#195;/ *"
  "C /&bxll./&#192;/ *"
  "C /&bxlr./&#217;/ *"
  "C /&bxrj./&#180;/ *"
  "C /&bxuj./&#194;/ *"
  "C /&bxul./&#218;/ *"
  "C /&bxur./&#191;/ *"
  "C /&bxv./&#179;/ *"
  "C /&caret./&#94;/ *"
  "C /&cdq./&#34;/ *"
  "C /&check./&acute;/ *"                                     /* Check mark*/
  "C /&colon./:/ *"
  "C /&copy./&copy;/ *"                                 /* Copyright symbol*/
  "C /&darrow./&#25;/ *"
  "C /&eq./=/ *"
  "C /&gt./&gt;/ *"
  "C /&larrow./&#27;/ *"
  "C /&lbrc./&#123;/ *"
  "C /&lbrk./&#91;/ *"
  "C /&lt./&lt;/ *"
  "C /&numsign./&#35;/ *"
  "C /&osq./&#96;/ *"
  "C /&per././ *"
  "C /&rarrow./&#26;/ *"
  "C /&rbrc./&#125;/ *"
  "C /&rbrk./&#93;/ *"
  "C ?&slash.?/? *"
  "C ?&sqrt.?/? *"
  "C /&tilde./~/ *"
  "C /&uarrow./&#24;/ *"
Exit


/***************************************************************************/
/*                                                                         */
/* Utility function to split a line at the specified IPF tag.  Note that   */
/* the character immediately following the tag is deleted - it is assumed  */
/* to be either a period (.) or a space.                                   */
/*                                                                         */
/***************************************************************************/

SPLIT_LINE:  Procedure Expose prefix suffix
  Parse Arg tag, linetext
  tag_pos = Pos(Translate(tag),Translate(linetext))
  If tag_pos > 0
    Then Do
      prefix = Substr(linetext,1,tag_pos-1)
      suffix = Substr(linetext,tag_pos+Length(tag)+1)
    End                                                           /* End do*/
    Else Do                                                /* Tag not found*/
      prefix = ''
      suffix = linetext
    End
Return tag_pos


/***************************************************************************/
/*                                                                         */
/* Process all include lines by copying them into the current document.    */
/* This causes the base document to become too large, but makes things     */
/* like references and symbols easier to resolve.                          */
/*                                                                         */
/***************************************************************************/

INCLUDES:  Procedure
  'TOP'
  'L /.im/ f'
  Do While rc = 0
    'EXTRACT /CURLINE/'
    Parse Var curline.1 . fn .
    'DELETE 1'
    'UP 1'
    'GET' fn
    'TOP'
    'REPFIND'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Look for a :title tag and convert it.  We'll also add a large heading   */
/* to the screen, because we're assuming that this tag identifies the      */
/* title page!                                                             */
/*                                                                         */
/***************************************************************************/

TITLE:  Procedure
  'TOP'
  'L /:title./'
  If rc = 0
    Then Do                                   /* Convert the document title*/
      'EXTRACT /CURLINE/'
      Call split_line ':TITLE', curline.1
      'REPLACE <head><title>'suffix'</title></head>'
      'INPUT <h1 align=center>'suffix'</h1>'
    End                                                           /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Resolve any IPF symbols.  We look for the symbol identifier, then do a  */
/* global change on that symbol in the file.                               */
/*                                                                         */
/***************************************************************************/

SYMBOLS:  Procedure
  '0'
  'l /.nameit/'                                       /* Look for variables*/
  Do While rc = 0
    'EXTRACT /CURLINE/'
    'DELETE 1'
    Parse Var curline.1 'symbol=' symname 'text=' text
    Parse Var text . "'" text "'"
    'C /&'Strip(symname)'./'Strip(text)'/ *'
    '0'
    'L /.nameit/'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Convert all :h1 through :h4 tags into their HTML equivalent, and make   */
/* sure we end the heading!  We also need to be able to identify these     */
/* headings for hypertext links.                                           */
/*                                                                         */
/***************************************************************************/

HEADINGS:  Procedure
  Do i = 1 To 4
    'TOP'
    'UP 1'
    tag = ':H'i
    'L /'tag'/'
    Do While rc = 0
      'EXTRACT /CURLINE/'
      Call split_line tag, curline.1
      If Pos('.',suffix) > 0
        Then Parse Var suffix ctl_info '.' suffix
        Else ctl_info = ''
      'REPLACE' prefix'<H'i'>'suffix'</H'i'>'
      Parse Value Translate(ctl_info) With . 'ID=' id .
      If id <> ''
        Then Do
          'UP 1'
          'INPUT <a Name="'id'">'
          'DOWN 1'
        End                                                       /* End do*/
      'REPFIND'
    End                                                           /* End do*/
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Resolve all hypertext links.                                            */
/*                                                                         */
/***************************************************************************/

LINKS:  Procedure
  'TOP'
  'UP 1'
  tag = ':LINK'
  'L /'tag'/'
  Do While rc = 0
    'EXTRACT /CURLINE/'
    Call split_line tag, curline.1
    If Pos('.',suffix) > 0
      Then Do
        Parse Var suffix ctl_info '.' suffix
        Parse Value Translate(ctl_info) With . 'REFID=' id .
        'REPLACE' prefix'<a href="#'id'">'
        Do Forever                                    /* Look for ELINK tag*/
          tag_pos = split_line(':ELINK', suffix)
          If tag_pos > 0
            Then Do
              'INPUT' prefix'</a>'suffix
              Leave
            End                                                   /* End do*/
            Else
              'INPUT' suffix                       /* Just restore the line*/
          'DOWN 1'
          If rc <> 0
            Then Leave                                              /* EOF?*/
          'EXTRACT /CURLINE/'
          suffix = curline.1
          'DELETE 1'
          'UP 1'
        End                                                       /* End do*/
      End                                                         /* End do*/
    'REPFIND'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Convert tables from IPF format to HTML.  We handle :table, :etable,     */
/* :row, and :c tags.                                                      */
/*                                                                         */
/***************************************************************************/

TABLES:  Procedure
  Do Forever                                 /* Process one table at a time*/
    'TOP'
    'L /:table/'                                             /* Table start*/
    If rc <> 0
      Then Leave
    'EXTRACT /CURLINE/'
    Parse Value Translate(curline.1) With 'FRAME=' frame .
    If abbrev(frame,'BOX.',3)
      Then border = 1
      Else border = 0
    'REPLACE <BR><BR><TABLE BORDER='border'>'
    'MARK CLEAR'
    'MARK LINE'

    'L /:etable/'                                              /* Table end*/
    If rc <> 0
      Then Leave                                                    /* Huh?*/
    'REPLACE </TR>'
    'INPUT </TABLE>'
    'MARK LINE'

    'TOP'
    'L /:row./ m'
    Do i = 1 While rc = 0                                   /* Process rows*/
      'EXTRACT /CURLINE/'
      Call split_line ':ROW', curline.1
      If i > 1
        Then prefix = prefix'</TR>'                    /* Not the first row*/
      'REPLACE' prefix'<TR>'suffix
      'REPFIND'
    End                                                           /* End do*/

    'TOP'
    'L /:c./ m'
    col_incomplete = 0
    col_tag = 'TH>'                                    /* Table heading tag*/
    Do While rc = 0                                  /* Process row columns*/
      'EXTRACT /CURLINE/'
      outline = curline.1
      Do Until col_pos = 0
        col_pos = split_line(':C', outline)
        If col_pos = 0
          Then col_incomplete = 1
          Else Do
            If Pos('<T',prefix) > 0
              Then outline = prefix'</'col_tag'<'col_tag || suffix
              Else outline = prefix'<'col_tag || suffix
          End                                                     /* End do*/
      End                                                         /* End do*/

      'REPLACE' outline
      If col_incomplete
        Then Do Forever                           /* Look for end of column*/
          'DOWN 1'
          If rc <> 0
            Then Leave                                              /* EOF?*/
          'EXTRACT /CURLINE/'
          upline = Translate(curline.1)
          If Pos(':C',upline) > 0,
           | Pos('<TR',upline) > 0,
           | Pos('</TABLE',upline) > 0
            Then Leave
        End                                                       /* End do*/
      'UP 1'
      'INPUT </'col_tag
      If Pos('<TR',upline) > 0
        Then col_tag = 'TD>'                            /* Table column tag*/
      'REPFIND'
    End                                                           /* End do*/
  End                                                             /* End do*/

Return


/***************************************************************************/
/*                                                                         */
/* Convert figures.  HTML figures aren't really very well defined, so to   */
/* make them stand out a bit we'll add horizontal rules before and after   */
/* each figure.                                                            */
/*                                                                         */
/***************************************************************************/

FIGURES:  Procedure
  Do Forever                                /* Process one figure at a time*/
    'TOP'
    'L /:fig./ | /:efig./'
    If rc <> 0
      Then Leave
    'EXTRACT /CURLINE/'
    If Pos('EFIG',Translate(curline.1)) > 0
      Then 'REPLACE </fig><hr>'                               /* End figure*/
      Else 'REPLACE <br><hr><fig>'                          /* Start figure*/
    'REPFIND'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Pictures!  The IPF file is probably including a bitmap (extension BMP), */
/* but we convert ANY extension to GIF.  The actual graphic conversion     */
/* will have to be done manually!                                          */
/*                                                                         */
/***************************************************************************/

ARTWORK:  Procedure
  'TOP'
  'L /:artwork/'
  Do While rc = 0
    'EXTRACT /CURLINE/'
    line = Translate(curline.1)
    If Pos('NAME=',line) > 0
      Then Parse Var line . 'NAME=' fname .
    fname = Translate(fname,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')
    Parse Var fname . "'" fname "'" .
    lastdot = Lastpos('.',fname)
    If lastdot > 0
      Then fname = Substr(fname,1,lastdot-1)

    If Pos('ALIGN=',line) > 0
      Then Parse Var line . 'ALIGN=' align .
    If Pos('.',align) > 0
      Then Parse Var align align '.' .

    'REPLACE <IMG SRC="'fname'.gif">'
    If align = 'CENTER'
      Then Do
        'INPUT </CENTER>'
        'UP 2'
        'INPUT <CENTER>'
      End                                                         /* End do*/
    'REPFIND'
  End                                                             /* End do*/

Return


/***************************************************************************/
/*                                                                         */
/* General conversion routine to change IPF tags into HTML.  A parameter   */
/* lets us know whether the tag has both a beginning and ending format,    */
/* e.g. a list, or if it only has a beginning, e.g. a list item.  The      */
/* standalone parameter indicates that the HTML tag will be all on its own */
/* on the output line, so we can discard anything else on the input line.  */
/*                                                                         */
/***************************************************************************/

CONVERT:  Procedure
  Parse Arg ipf_tag, html_tag, standalone, noend .
  'TOP'
  'UP 1'
  tag = ':'Translate(ipf_tag)
  'L /'tag'/'                                                  /* Start tag*/
  Do While rc = 0
    'EXTRACT /CURLINE/'
    Call split_line tag, curline.1
    If standalone
      Then suffix = ''                                  /* Remove modifiers*/
    If noend
      Then suffix = suffix'</'html_tag'>'
    'REPLACE' prefix'<'html_tag'>'suffix
    'REPFIND'
  End                                                             /* End do*/

  If noend
    Then Return

  'TOP'
  'UP 1'
  tag = ':E'Translate(ipf_tag)
  'L /'tag'/'                                                  /* Start tag*/
  Do While rc = 0
    'EXTRACT /CURLINE/'
    Call split_line tag, curline.1
    'REPLACE' prefix'</'html_tag'>'suffix
    'REPFIND'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* This one's easy - delete all lines containing the specified string!     */
/*                                                                         */
/***************************************************************************/

DELETE:  Procedure
  Parse Arg search
  'ALL /'search'/'
  'TOP'
  'DELETE *'
  'ALL'
Return


/***************************************************************************/
/*                                                                         */
/* Convert all index entries into markers                                  */
/*                                                                         */
/***************************************************************************/

INDEX: Procedure
  '0'
  entry. = ''

  'locate /:i1'
  numFound = 0
  Do While rc = 0
    numFound = numFound + 1
    'extract /curline/'
    Parse Var curline.1 . '.' entry
    'extract /cursor/'
    entry.numFound = cursor.1 entry
    'replace <a name="IDX'cursor.1'">'
    'REPFIND'
  End                                                             /* End do*/

  Call delete ':i2'
  Call delete ':index'

  'BOTTOM'
  If numFound = 0
    Then Return                                         /* No index entries*/

  'INPUT <a Name="X2IDX">'
  'INPUT <h1>Index</h1>'
  'INPUT <hr>'
  'extract /cursor/'
  top_index = cursor.1
  Do num = 1 To numFound
    Parse Var entry.num id entry
    'INPUT <LI>'entry '<A HREF="#IDX'id'">('id')</A>'
  End                                                             /* End do*/

  Call sort_index
Return


/***************************************************************************/
/*                                                                         */
/* Sort the index entries                                                  */
/*                                                                         */
/***************************************************************************/

sort_index:  Procedure Expose top_index
  'mark clear'
  'mark line'
  'cursor' top_index + 1
  'mark line'
  'sort c'

  'cursor' top_index
  'locate /<A HREF=/ m'
  curr_letter = ' '
  letters = ''
  Do While rc = 0                                     /* Split up the lines*/
    'extract /curline/'
    Parse Upper Var curline.1 . '<LI>' letter1 .
    letter1 = Substr(letter1,1,1)
    If letter1 <> curr_letter
      Then Do                                      /* A new starting letter*/
        curr_letter = letter1
        letters = letters letter1
        'extract /cursor/'
        'up 1'
        'INPUT </MENU>'
        'INPUT <STRONG><A name="IDX_'curr_letter'"',
               'HREF="#IDX0_'curr_letter'">'curr_letter'</A></STRONG>'
        'INPUT <MENU>'
        'cursor' cursor.1 cursor.2
      End                                                         /* End do*/
    'split'
    'down 1'
    'repfind'
  End                                                             /* End do*/
  'INPUT </MENU>'                                /* Finish off the last one*/
  'INPUT <hr>'

  'cursor' top_index
  'input <MENU>'
  Do While letters <> ''
    Parse Var letters letter1 letters
    'INPUT <A NAME="IDX0_'letter1'" HREF="#IDX_'letter1'">'letter1'</A>'
  End                                                             /* End do*/
  'down 1'                                               /* Skip over /MENU*/
  'INPUT <hr>'
Return
