/***************************************************************************/
/*                                                                         */
/* IPF2RTF.X                                                               */
/*                                                                         */
/* Convert a file from IPF input format into Rich Text Format (RTF).       */
/* This code was written to convert the XWING help file into RTF format,   */
/* as an intermediate step to creating a HLP file with the hcrtf tool.  As */
/* such, it makes certain assumptions regarding font size and style which  */
/* may not be appropriate for all applications.  Caveat Emptor.            */
/*                                                                         */
/* Written by Blair Thompson, December 29, 2000                            */
/*                                                                         */
/***************************************************************************/

  'EXT RTF'
  'WRAP OFF'

  Call header
  Call paragraphs                             /* Must be right after header*/
  Call lists                                                /* Must be next*/
  Call headings
  Call links
  Call index
  Call bold
  'BOTTOM'
  'INPUT }'
Exit


/***************************************************************************/
/*                                                                         */
/* Utility function to split a line at the specified IPF tag.  Note that   */
/* the character immediately following the tag is deleted - it is assumed  */
/* to be either a period (.) or a space.                                   */
/*                                                                         */
/***************************************************************************/

SPLIT_LINE:  Procedure Expose prefix suffix
  Parse Arg tag, linetext
  tag_pos = Pos(Translate(tag),Translate(linetext))
  If tag_pos > 0
    Then Do
      prefix = Substr(linetext,1,tag_pos-1)
      suffix = Substr(linetext,tag_pos+Length(tag)+1)
    End                                                           /* End do*/
    Else Do                                                /* Tag not found*/
      prefix = ''
      suffix = linetext
    End
Return tag_pos


/***************************************************************************/
/*                                                                         */
/* Any non-tagged text should be indented by one space                     */
/*                                                                         */
/***************************************************************************/

CHECK_INDENT: Procedure
  Do Forever
    'down 1'
    If rc <> 0
      Then Leave                                                     /* EOF*/
    'extract /curline/'
    c1 = Substr(curline.1,1,1)
    If c1 = ':' & Translate(substr(curline.1,2,2)) = 'HP'
      Then c1 = 'x'                       /* Bold text should be indented*/
    Select
      When c1 = ':' | c1 = '\'
        Then Leave
      When c1 = ' '
        Then Nop                                     /* Space already there*/
      Otherwise
        'replace ' curline.1                            /* Insert the space*/
    End                                                       /* End select*/
  End                                                             /* End do*/
  'up 1'
Return


/***************************************************************************/
/*                                                                         */
/* Insert standard header information.  We define standard fonts and basic */
/* colours.                                                                */
/*                                                                         */
/***************************************************************************/

HEADER: Procedure
  'ALL /.*/ f'                                       /* Remove any comments*/
  'TOP'
  'DELETE *'
  'ALL'

  '0'
  'INPUT {\rtf1\ansi \deff0\deflang1024'
  'INPUT'
  'INPUT {\fonttbl'
  'INPUT {\f0\froman Times New Roman;}'
  'INPUT {\f1\froman Symbol;}'
  'INPUT {\f2\fswiss Arial;}'
  'INPUT {\f3\froman MS Serif;}'
  'INPUT {\f4\fswiss MS Sans Serif;}}'
  'INPUT'
  'INPUT {\colortbl;'
  'INPUT \red0\green0\blue0;'
  'INPUT \red0\green0\blue255;'
  'INPUT \red0\green255\blue0;'
  'INPUT \red255\green0\blue0;'
  'INPUT \red255\green255\blue255;}'
  'INPUT'
Return


/***************************************************************************/
/*                                                                         */
/* Convert any heading tags into an RTF topic.                             */
/*                                                                         */
/***************************************************************************/

HEADINGS: Procedure
  Do i = 1 To 4
    'TOP'
    'UP 1'
    tag = ':H'i
    'LOCATE /'tag'/ fq'
    Do While rc = 0
      'extract /curline/'
      Call split_line tag, curline.1
      If Pos('.',suffix) > 0
        Then Parse Var suffix ctl_info '.' suffix
        Else ctl_info = ''
      Parse Value Translate(ctl_info) With . 'ID=' id .
      'REPLACE'
      'INPUT \page'
      'INPUT #{\footnote' id'}'
      'INPUT ${\footnote' suffix'}'
      'INPUT'
      'INPUT \pard\plain \li120\sb340\sa120\sl-320 \f3\fs28' suffix
      'INPUT'
      'REPFIND'
    End                                                           /* End do*/
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Convert all :p. tags, and make sure the following text is indented by a */
/* single space.                                                           */
/*                                                                         */
/***************************************************************************/

PARAGRAPHS: Procedure
  'top'
  'locate /:p./'
  Do While rc = 0
    'extract /curline/'
    Parse Var curline.1 . '.' text
    'replace \par \pard\plain \li120 \f4\fs20' text
    Call check_indent
    'repfind'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Convert lists.  We only support simple lists at the moment.             */
/*                                                                         */
/***************************************************************************/

LISTS: Procedure
  'top'
  'locate /:sl/ f'
  Do While rc = 0
    'replace \par'
    'mark clear'
    'mark line'
    'locate /:esl/'
    'replace \par'
    'mark line'
    'cursor BEGMARK'
    'locate /:li./ m'
    Do While rc = 0
      'extract /curline/'
      Parse Var curline.1 . '.' text
      'replace \f4\fs20 \pi720' text
      'up 1'
      'INPUT \pard\plain \s83 \fs20 \fi-360 \li360 \par'
      "INPUT {\*\pn \pnlvlbody\pnindent360\pnhang\pnf1\pnfs24\pnql{\pntxtb \'b7}}"
      'down 1'
      Call check_indent
      'repfind'
    End                                                           /* End do*/
    'locate /:sl/ f'                               /* Look for another list*/
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Convert :hdref references                                               */
/*                                                                         */
/***************************************************************************/

LINKS: Procedure
  'top'
  'locate /:link/'
  Do While rc = 0
    'extract /curline/'
    Call split_line ':LINK', curline.1
    pretext = prefix
    If Pos('.',suffix) > 0
      Then Do
        Parse Var suffix ctl_info '.' suffix
        Parse Value Translate(ctl_info) With . 'REFID=' id .
        Call split_line ':ELINK', suffix
      End                                                         /* End do*/
    'REPLACE' pretext'{\uldb' prefix '\v' id'}' suffix
    'REPFIND'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Convert :i1 index entry tags                                            */
/*                                                                         */
/***************************************************************************/

INDEX: Procedure
  'top'
  'locate /:i1/ f'
  Do While rc = 0
    'extract /curline/'
    Parse Var curline.1 . '.' text
    'replace \par \pard\plain \li120 \f4\fs20' text
    'delete'
    'locate /{\footnote/ -'
    'input K{\footnote' text'}'
    'restore_find'
    'repfind'
  End                                                             /* End do*/
Return


/***************************************************************************/
/*                                                                         */
/* Convert any text that may need bold face                                */
/*                                                                         */
/***************************************************************************/

BOLD: Procedure
  'top'
  'locate /:hp2./'
  Do While rc = 0
    'extract /curline/'
    Call split_line ':HP2', curline.1
    pretext = prefix
    Call split_line ':EHP2', suffix
    'replace' pretext'{\b\fs20' prefix'}'suffix
    'repfind'
  End                                                             /* End do*/
Return
