/********************************** LINES **********************************/
/*                    X2 Editor Macro LINES, Version 3                     */
/*           draw horizontal and vertical LINES by moving cursor           */
/*                                                                         */
/* Allow drawing horizontal and vertical lines in the editor by moving the */
/* cursor up and down, left and right.  Cursor movement is effected in     */
/* this version of Lines by PF keys and/or cursor movement keys.  A        */
/* variety of line styles is available, e.g. single/double line etc.       */
/*                                                                         */
/* This macro, as delivered, uses the PF1-4 and the cursor up,down,left    */
/* and right keys for drawing the lines.  Drawing with the cursor keys may */
/* be toggled (activated, deactivated) with the PF5 key.  With PF11        */
/* another line style may be selected, PF12 exits this macro.  As from     */
/* Version 3, any of the box characters associated with the current line   */
/* style may be placed specifically by using the Alt-1, Alt-2 etc.  keys:  */
/* Alt-1 for the lower left corner character, Alt-2 for the bottum (upside */
/* down) T etc.  Look at the arrangement of the numeric keys on the far    */
/* right keypad - 7 for upper left, 3 for bottom right - and use           */
/* Alt+number.  Alt-0 corresponds to the vertical bar, Alt-- (Alt plus     */
/* minus) corresponds to the horizontal bar.  Any other keys are passed to */
/* the editor.                                                             */
/*                                                                         */
/* Call:                                                                   */
/*                                                                         */
/*      LINES ?                                                            */
/*                                                                         */
/*      puts out these comments as a help text (/? and -? are also         */
/*      accepted for Unix etc. compatibility).                             */
/*                                                                         */
/*      LINES                                                              */
/*                                                                         */
/*      i.e. without parameters, is the normal invocation.  This leads to  */
/*      a popup window with the various line styles being offered for      */
/*      selection.  After selecting a line style (which includes lines     */
/*      composed of blanks therefore offering a 'rubout' function) the     */
/*      PFLine changes so that cursor movement (equals drawing the line)   */
/*      is in operation by using the PF1 to PF4 keys or the normal cursor  */
/*      movement keys (cursor up, down, left, right).  Other options are   */
/*      to chose a new line style (PF11) or to exit the macro (PF12).      */
/*      Drawing with the cursor keys may be toggled with the PF5 key, this */
/*      may be useful for interrupting drawing and moving to another       */
/*      position before resuming drawing or when moving to the command     */
/*      stack etc.                                                         */
/*                                                                         */
/*      LINES n                                                            */
/*                                                                         */
/*      where n is a numeric value indicating which line style is wanted.  */
/*      This is for experienced users who know which line style to use     */
/*      and who wish to avoid the popup window showing the lines styles    */
/*      available.                                                         */
/*                                                                         */
/* As supplied by it's author, this macro does not use the PF9 and PF10    */
/* keys which are the X Editor's "UNDO" and "REDO" default key             */
/* assignments.  These functions are therefore available in this macro.    */
/*                                                                         */
/* Version 3 provides an improved algorithm for drawing lines with the     */
/* cursor (or F1-F4) keys such that more intersections are "guessed"       */
/* correctly. If an intersection character is returned that you regard     */
/* as incorrect, it is often corrected by simply moving the cursor over    */
/* it backwards and forwards. If this is still not the desired character   */
/* you can specifically chose one of the box characters corresponding      */
/* to the selected line style by using one of the Alt-1, Alt2 etc. keys    */
/* as described above.                                                     */
/*                                                                         */
/* Please note that for this macro to work correctly, it is imperative     */
/* that you install xwutils.dll or x2utils.dll as supplied with the        */
/* version of the editor you are using. This version needs editor Release  */
/* 1.99 or higher.                                                         */
/*                                                                         */
/* Together with the macro BOXES.X, this macro provides a means of drawing */
/* diagrams etc. with the X editor.  The first 6 line styles in the popup  */
/* window correspond to the 6 box styles in BOXES.                         */
/*                                                                         */
/* Thanks:  Many ideas etc.  in this macro have been derived from coding   */
/* in the BOXES.X macro (written by B. Thompson, modified by Patrick       */
/* Leabo) and the author of this macro (RCS) wishes to express his thanks  */
/* to Blair and Patrick.  HOWEVER, any errors in this macro are the sole   */
/* property of the author of this macro.                                   */
/*                                                                         */
/* Author:  Roger C. Spencer  (V2SPENCE at SDFVM2)                         */
/*                                                                         */
/* Change activity                                                         */
/* 96-06-27 RCS (=Roger C.Spencer) first version, OS/2 only                */
/* 99-11-19 rewrite for WindowsNT, use KEYPRESS, do not call recursively   */
/* 00-01-05 avoid using odd-bod characters in description in popup window  */
/*          (to avoid corruption when uploading to/downloading from host)  */
/*          Number of calls to put_msg reduced; Code cleanup (beepit etc.  */
/*          removed); System check for filespec(). Needs 1.99 or higher.   */
/* 00-06-26 pick up choice of line style as invocation parm.               */
/* 00-09-26 improved algorithm for intersections.                          */
/* 00-09-29 removed value() invocations; added Alt-numeric support.        */
/* 00-12-20 crash if A-x is first key for drawing; honour INSMODE          */
/***************************************************************************/

 Signal endhelpline                      /* these two lines define the ...,*/
 endhelpline: lasthelpline = sigl -1                 /* ...text for 'help' */

/*-------------------------------------------------------------------------*/
/* N O T E !  keep the above two lines as they are used to pick up         */
/* the number of the last line of help text to put out to a window.        */
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/* Technical notes:  This macro is to be invoked without any parameters    */
/* (apart from ?  for help).  This macro then puts out a selection menue   */
/* in a popup window so that the user may select the style of line.        */
/* Selection leads to this macro remaining active.                         */
/*                                                                         */
/* The characters defining the various line styles are stored in the       */
/* variable 'alls' (=ALL Styles).  These characters work O.K. for:         */
/* - code page 437 (except for one line style with a broken vertical line) */
/* - code page 850 (except for line styles I, H and 'zigzag')              */
/* For other code pages you may have to redefine some of these characters. */
/*                                                                         */
/* Pressing the F11  key results in the line style selection menu being    */
/* popped up once again. The F12 key results in this macro terminating.    */
/*                                                                         */
/*  N O T E S ! ! !                                                        */
/* - The author has had some trouble setting/extracting the curr_pfline    */
/*   and pfline in Release 1.98 Beta C. The previous version of this macro */
/*   therefore used MSG to display the PF keys for Release 1.98 or less,   */
/*   PFLINE for higher release numbers.  This version of this macro relies */
/*   on PFLINE because of performance reasons and therefore assumes        */
/*   Release 1.99 or higher.  This is checked in the macro.                */
/*                                                                         */
/* - This version of the macro (and its predecessor) uses the cursor keys  */
/*   (UP, DOWN, LEFT and RIGHT) to draw the lines.  A new 'line style' has */
/*   been added which does not draw any line but simply moves the cursor.  */
/*   This is to avoid leaving this macro if the user simply wants to stop  */
/*   drawing as line, move to another place in the text and start drawing  */
/*   again.                                                                */
/*                                                                         */
/* - This macro picks up its own name for putting out in error messages    */
/*   and also for invoking itself when PF11 is used.  The function         */
/*   'filespec()' is used to pick up its name.  This is a known system     */
/*   dependency as filespec() is not available on every system.  However,  */
/*   a check is made to see under which system this macro is running and   */
/*   if not WinNT/95 or OS/2, a hardcoded 'LINES' is used.  If users wish  */
/*   to port this macro to other systems, they should look at the coding   */
/*   surrounding filespec() which simply extracts the name of this macro   */
/*   from a full path specification. (Filespec is only invoked once.)      */
/*                                                                         */
/*   However, the really major system dependency is the use of the         */
/*   hexadecimal values for many of the characters used in the box styles. */
/*   This refers to the single and double, horizontal and vertical lines,  */
/*   smiley, bullets etc.  These values assume an ASCII character set with */
/*   Code Page 437 or 850 (this macro has been used with these two code    */
/*   pages, other pages have not been tested but are expected to work      */
/*   subject to some restrictions).  This macro is known to work with      */
/*   satistfactory results under OS/2, Win95 and WinNT.  The author would  */
/*   appreciate hearing if this macro has been used satisfactorily on      */
/*   other systems.                                                        */
/*                                                                         */
/* - This macro uses the MESSAGEBOX and 'EXTRACT /KEYPRESS/' facilities.   */
/*   Use of the 'EXTRACT /KEYPRESS/' facility makes it IMPERATIVE that you */
/*   use the X2UTILS.DLL and/or XWUTILS.DLL SUPPLIED WITH THE VERSION OF   */
/*   THE EDITOR you are using.                                             */
/*-------------------------------------------------------------------------*/

Signal On Novalue;  Signal On Syntax            /* trap programming errors */

true = 1; false = 0                                   /* Boolean variables */

mb_version = 1.96        /* version number that first supported MESSAGEBOX */

min_version = 1.99        /* minimum X editor version supported; 1.99 is.. */
/* ..because of not using MSG but only PFLINE for displaying keys          */

/*-------------------------------------------------------------------------*/
/*    set up the designations of the keys to user for drawing              */
/*-------------------------------------------------------------------------*/

/* If you change any of the following PF key designations, check procedure */
/* put_msg which sets up the PFLINE for displaying.  If you change any of  */
/* the F1-F5, F11 or F12 keys (i.e. map the UP etc.  functionality to      */
/* other keys), you should also check the several uses of the VALUE() bif  */
/* (with X_F1, X_F2 etc.) for saving and retrieving the current settings.  */
key_up.en     = 'F2 UP'                   /* keys when cursor keys enabled */
key_right.en  = 'F4 RIGHT'
key_down.en   = 'F3 DOWN'
key_left.en   = 'F1 LEFT'

key_up.dis    = 'F2'                     /* keys when cursor keys disabled */
key_right.dis = 'F4'
key_down.dis  = 'F3'
key_left.dis  = 'F1'

Call set_enable                          /* enable cursor keys for drawing */

key_endis    = 'F5'              /* enable/disable cursor keys for drawing */
key_newstyle = 'F11'                              /* select new line style */
key_quit     = 'F12'                                    /* quit this macro */

/* The following define the keys used to explicitly insert one of the box  */
/* characters.  These are not shown in the pfline.  They may be redefined  */
/* by the user here.  Note that there is a one-to-one correspondence, i.e. */
/* it is not possible to assign two keys to the one character or           */
/* vice-versa. You can reassign these keys, but they must be kept in the   */
/* same sequence (Vertical bar, horizontal bar, .... bottom right corner). */
box_keys = 'A-0',                                          /* Vertical bar */
           'A--',                                        /* Horizontal bar */
           'A-7',                                     /* Upper left corner */
           'A-8',                                   /* normal, drop down T */
           'A-9',                                  /*  Upper right  corner */
           'A-4',                                   /* T pointing to right */
           'A-5',                                          /* centre cross */
           'A-6',                                    /* T pointing to left */
           'A-1',                                    /* bottom left corner */
           'A-2',                                         /* upside down T */
           'A-3'                                    /* bottom right corner */

/* ------------------------------------------------------------------------*/
/* start work                                                              */
/* ------------------------------------------------------------------------*/

'EXTRACT /PFLINE/'                                   /* get current pfline */
x_pfline = pfline.1                                             /* save it */

/* Get name of this macro for use in error messages etc. and new style     */
/* ------------------------------------------------------------------------*/

/* Define in the variable 'filespec_os' the names of systems as            */
/* returned by 'EXTRACT /OS/' that support the filespec() function.        */

filespec_os = 'OS/2 WINDOWS NT WINDOWS 95'

'EXTRACT /OS/'
If (wordpos(translate(os.1), filespec_os) = 0) Then Do
  myname = 'LINES'          /* filespec not supported, use hardcoded value */
End                                                              /* end if */
Else Do
  Parse Source . . myname .
  myname = Translate(Filespec('Name', myname))
  Parse Var myname myname '.' .                          /* drop extension */
End                                                              /* end if */

/* See to Release dependencies                                             */
/* ----------------------------------------------------------------------- */

bold     = ''                          /* set up escape sequences ...      */
emphasis = ''                          /* ...for text in popup window...   */
normal   = ''                          /* ...and MESSAGEBOX                */
'EXTRACT /ESCAPE/'                     /*                                  */
If (rc = 0) Then Do                    /* highlighting only if o.k.        */
  bold     = escape.1 || 'B'           /* window bold escape sequence      */
  emphasis = escape.1 || 'E'           /* window emphasis escape sequence  */
  normal   = escape.1 || 'T'           /* command stack colour             */
End

'EXTRACT /VERSION/'

If (rc <> 0) Then Do          /* set to minimal version number if not o.k. */
  Version.1 = 0.0
End
Else Do
  Version.1 = Left(version.1,4)                      /* drop "Beta A" etc. */
End

If (version.1 < min_version) Then Do
  If (version.1 < mb_version) Then Do             /* MESSAGEBOX supported? */
    'MSG Version too low:' myname ,
      'needs version' min_version', your editor has version' Version.1
  End
  Else Do
    'MESSAGEBOX Version too low!!' ,
      escape.1'N'myname 'needs version' min_version ,
      escape.1'Nyour editor has version' Version.1  ,
      escape.1'NExiting macro.'
  End                                                          /* end else */
  Exit
End                                                              /* end if */

/* Pick up option, see if help requested                                   */
/* ------------------------------------------------------------------------*/

Arg parm                /* assume opts to be passed to editor at invocation*/

'EXTRACT /CURSOR/'                     /* Get current position             */

'EXTRACT /FLSCREEN/'                   /* get no. of lines on screen       */
If (flscreen.0 <> 2) Then window_size = 18     /* set up no. of lines...   */
Else Do                                /* ...for pop-up windows            */
  window_size = (flscreen.2 - flscreen.1 + 1)- 4
End

help = '? /? -? HELP'            /* if any of these passed at invocation...*/
If (wordpos(parm, help) <>0) Then Do /*...put out comments as help text    */
  window_width = Length(Sourceline(1))
  'WINDOW' window_size window_width lasthelpline myname 'Help text'
  Do i = 1 To lasthelpline-1           /* make sure loop ends sometime     */
    'WINLINE' Sourceline(i) || '\nREFRESH'
  End
  Exit
End
If (datatype(parm) <> 'NUM') ,                                 /* 00-06-26 */
  &  (parm <> '') Then Do                                      /* 00-06-26 */
  "MESSAGEBOX" myname": Illegal option detected: '"parm"'"  ,
    escape.1 || "NOnly one option is allowed and it must be one of:" ,
    escape.1 || "N"help  ,
    escape.1 || "Nor be numeric (line style choice)" ,
    escape.1 || "NExiting macro, please try again"
  Exit
End                                                              /* end if */

/* check if we are in browse mode                                          */
/* ------------------------------------------------------------------------*/

"EXTRACT /FILEINFO/"
If (fileinfo.5 = 'BROWSE') Then Do
  "MESSAGEBOX" myname": In Browse mode, exiting"
  Exit
End                                                              /* end if */

/*-------------------------------------------------------------------------*/
/*  If called with no args, show choice of line style.                     */
/*  However, first set up the variables for the various line styles.       */
/* ------------------------------------------------------------------------*/

/*  Each line style has an 11 byte long entry in the following table (c.f. */
/*  BOXES.X) Of the 11 bytes only the following 6 are used regularly:      */
/*     1    vertical                                                       */
/*     2    horizontal                                                     */
/*     3    upper left corner                                              */
/*     5    upper right  corner                                            */
/*     9    lower left corner                                              */
/*     11   lower right corner                                             */
/*  The other chars are intersections and are used when a line crosses     */
/*  itself (see subprogramm 'replacechar:')                                */
/*  Short descriptions of these lines are placed in 'allsdesc.' later      */

alls ='B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* single line      */
      'BACDC9CBBBCCCEB9C8CABC'x||,                     /* double line      */
      'B3CDD5D1B8C6D8B5D4CFBE'x||,                     /* I line           */
      'BAC4D6D2B7C7D7B6D3D0BD'x||,                     /* H line           */
      '7C2D'x||copies('+',9)||,                        /* |-+++++++++      */
      'DBDFDBDBDBDBDBDBDFDFDF'x||,                     /* Solid 1          */
      copies('DB'x,11)||,                              /* Solid 2          */
      copies('b2'x,11)||,                              /* solid gray 1     */
      copies('b1'x,11)||,                              /* solid gray 2     */
      copies('b0'x,11)||,                              /* solid gray 3     */
      '!'||'2D'x||copies('+',9)||,                     /* !-+++++++++      */
      'DD'x||'2D'x||copies('FA'x,9)||,                 /* |-      */
      '!'||'='||copies('*',9)||,                       /* |=      */
      copies(' ',11)||,                                /* blanks (rubout)  */
      copies('FA'x,11)||,                              /* just dots        */
      copies('FE'x,11)||,                              /* square bullet    */
      copies('07'x,11)||,                              /* just bullet      */
      copies('09'x,11)||,                              /* open bullet      */
      copies('*',11)||,                                /* asterisks        */
      copies('0f'x,11)||,                              /* Sun symbol       */
      copies('02'x,11)||,                              /* smiley           */
      '3d3d3d3d3d3d3d3d3d3d3d'x||,                     /* equals           */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* single+ gaps     */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* 2*single+ gaps   */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* dot dash         */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* dot double dash  */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* bullet ddash     */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* openbullet ddash */
      '121D'x||copies('04'x,9)||,                      /* double arrows    */
      '1E1004C204C3C5B404C104'x||,                     /* largearrowheads  */
      '181aDAC2BFC3C5B4C0C1D9'x||,                     /* smallarrowheads  */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* 2*single+larrows */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* 2*single+sarrows */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* 2*single+quest.  */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* 2*single+exclam. */
      'BACDC9CBBBCCCEB9C8CABC'x||,                     /* double dash      */
      'BACDC9CBBBCCCEB9C8CABC'x||,                     /* ddouble dash     */
      'BACDC9CBBBCCCEB9C8CABC'x||,                     /* 2*double+larrows */
      'BACDC9CBBBCCCEB9C8CABC'x||,                     /* 2*double+bullet  */
      'BACDC9CBBBCCCEB9C8CABC'x||,                     /* 2*double+obullet */
      'DDDC'x||copies('DB'x,9)||,                      /* solid zigzag 437 */
      'B3C4DAC2BFC3C5B4C0C1D9'x||,                     /* suits inline     */
      'B3C403C204C3C5B405C106'x||,                     /* suits at corners */
      'B3C43fC23fC3C5B43fC13f'x||,                     /* ? at corners     */
      'B3C43fC2a8C3C5B4a8C13f'x||,                     /* ?  at corners   */
      'B3C421C221C3C5B421C121'x||,                     /* ! at corners     */
      'B3C421C2adC3C5B4adC121'x||,                     /* !  at corners   */
      'B3C42fC25cC3C5B45cC12f'x||,                     /* / \ at  corners  */
      copies(' ',11)                       /* dummy for just moving cursor */

Left. = ""            /* If a char is defined in 'down.' it is taken to be */
down. = ""            /* the preferred char for a line moving downwards.   */
/* Similarly, an additional char in 'left.' is taken                       */
/* to be the char to be used when proceding towards                        */
/* the left.                                                               */

/* The gaps.  parameter is used to introduce gaps                          */
/* (blanks) at regular intervals.  If the sum of the                       */
/* row and col numbers is exactly divisible by the                         */
/* 'gaps.' number, a blank is inserted.  However,                          */
/* instead of blanks, the 'gaps...!UP' etc chars are                       */
/* used if one has been stipulated for this                                */
gaps.    = ""         /* particular line type.                             */

/* The following variables are used in order to avoid corruption of        */
/* characters in the lines' descriptions due to host upload/download       */

d = 'C4'x                                                     /* dash char */
d2 = d || d                                                 /* two of them */
dd = 'CD'x                                             /* double dash char */
dd2 = dd || dd                                              /* two of them */

/* The following lines define the texts to be put out to the popup         */
/* selection window and the details (gaps. down. etc) for the line styles. */

k = 0                               /* kounter for the various line styles */
k = k+1 ;allsdesc.k  = 'Single line'               ;
k = k+1 ;allsdesc.k  = 'Double line'               ;
k = k+1 ;allsdesc.k  = 'I line (best with cp437)'  ;
k = k+1 ;allsdesc.k  = 'H line (best with cp437)'  ;
k = k+1 ;allsdesc.k  = 'vert.bar, hyphen, cross (cp850)'   ;
k = k+1 ;allsdesc.k  = 'Solid 1'                   ;
k = k+1 ;allsdesc.k  = 'Solid 2'                   ;
k = k+1 ;allsdesc.k  = 'Solid gray 1 (dark)'       ;
k = k+1 ;allsdesc.k  = 'Solid gray 2 (medium)'     ;
k = k+1 ;allsdesc.k  = 'Solid gray 3 (light)'      ;
k = k+1 ;allsdesc.k  = 'exclam.mark, hyphen, cross';
k = k+1 ;allsdesc.k  = 'broken bar, hyphen, dot'   ;
k = k+1 ;allsdesc.k  = 'exclam.mark, equals, asterisk'  ;
k = k+1 ;allsdesc.k  = 'blanks (rubout)'           ;
k = k+1 ;allsdesc.k  = 'dots'                      ;
k = k+1 ;allsdesc.k  = 'square bullet'             ;
k = k+1 ;allsdesc.k  = 'solid bullet'              ;
k = k+1 ;allsdesc.k  = 'open bullet'               ;
k = k+1 ;allsdesc.k  = 'asterisks'                 ;
k = k+1 ;allsdesc.k  = 'Sun symbol'                ;
k = k+1 ;allsdesc.k  = 'smileys'                   ;
k = k+1 ;allsdesc.k  = 'equals sign'               ;
k = k+1 ;allsdesc.k  = 'dashed line' d d d d       ; gaps.k = 2
k = k+1 ;allsdesc.k  = 'double dash line' d2 d2 d2 ; gaps.k = 3

k = k+1 ; gaps.k = 2  ;
gaps.k.!UP   = 'FA'X;      gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;
allsdesc.k  = 'dot dash   'd   || gaps.k.!UP || d || gaps.k.!UP || d

k = k+1 ; gaps.k = 3  ;
gaps.k.!UP   = 'FA'X;      gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;
allsdesc.k  = 'dot double dash   'd2 || gaps.k.!UP || d2

k = k+1 ; gaps.k = 3  ;
gaps.k.!UP   = '07'X;      gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;
allsdesc.k  = 'bullet double dash   'd2 || gaps.k.!UP || d2

k = k+1 ; gaps.k = 3  ;
gaps.k.!UP   = '09'X;      gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;
allsdesc.k  = 'openbullet double dash   'd2 || gaps.k.!UP || d2

k = k+1 ;allsdesc.k  = 'double arrows'

k = k+1 ;allsdesc.k  = 'large arrowheads' ;down.k = '1f'x; Left.k = '11'x

k = k+1 ;allsdesc.k  = 'small arrows'     ;down.k = '19'x; Left.k = '1b'x

k = k+1 ;allsdesc.k  = 'dashes + arrowheads'd2  || '10'x || d2
gaps.k = 3  ;
gaps.k.!UP   = '1E'X; gaps.k.!DOWN = '1F'X;
gaps.k.!left = '11'X; gaps.k.!right= '10'X;

k = k+1 ;allsdesc.k = 'dashes + arrows 'd2 || '1A'x || d2
gaps.k = 3  ;
gaps.k.!UP   = '18'X; gaps.k.!DOWN = '19'X;
gaps.k.!left = '1B'X; gaps.k.!right= '1A'X;

k = k+1 ;allsdesc.k = 'dashes + question mark 'd2'?'d2
gaps.k = 3  ;
gaps.k.!UP   = '?';        gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;

k = k+1 ;allsdesc.k = 'dashes + exclamation mark 'd2'!'d2
gaps.k = 3  ;
gaps.k.!UP   = '!';        gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;

k = k+1 ;allsdesc.k  = 'dashed line 'dd dd dd      ; gaps.k = 2
k = k+1 ;allsdesc.k  = 'double dash line 'dd2 dd2 dd2 ; gaps.k = 3

k = k+1 ;allsdesc.k = 'dashes + arrowheads 'dd2  || '10'x || dd2
gaps.k = 3  ;
gaps.k.!UP   = '1E'X; gaps.k.!DOWN = '1F'X;
gaps.k.!left = '11'X; gaps.k.!right= '10'X;

k = k+1 ;allsdesc.k = 'dashes + bullet     'dd2  || '07'x || dd2
gaps.k = 3  ;
gaps.k.!UP   = '07'X;      gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;

k = k+1 ;allsdesc.k = 'dashes + openbullet 'dd2  || '09'x || dd2
gaps.k = 3  ;
gaps.k.!UP   = '09'X;      gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = gaps.k.!UP; gaps.k.!right= gaps.k.!UP;

k = k+1 ;allsdesc.k = 'solid zigzap (CP437) '||'DCDFDCDFDCDF'x
gaps.k = 2  ;
gaps.k.!UP   = 'DE'X;      gaps.k.!DOWN = gaps.k.!UP;
gaps.k.!left = 'DF'x;      gaps.k.!right= gaps.k.!left;

k = k+1 ;allsdesc.k = 'suits inline 'd2  || '03'x || d2
gaps.k = 4  ;
gaps.k.!UP   = '03'X;      gaps.k.!DOWN = '04'x
gaps.k.!left = '05'x;      gaps.k.!right= '06'x

k = k+1 ;allsdesc.k = 'suits at corners'

k = k+1 ;allsdesc.k = '? at corners'

k = k+1 ;allsdesc.k = '? and' 'A8'x 'at corners'

k = k+1 ;allsdesc.k = '! at corners'

k = k+1 ;allsdesc.k = '! and' 'AD'x 'at corners'

k = k+1 ;allsdesc.k = '/ and  \ at corners   /'d2'\'

k = k+1 ;allsdesc.k  = 'No characters, just move cursor' ; just_cursor = k

/*-------------------------------------------------------------------------*/
/* We have been invoked without parms or with one numeric parm.            */
/* If without parms, Put out selection menu.                               */
/*-------------------------------------------------------------------------*/

/* now put out the line style selection menue to a popup window and wait   */
/* for the user's response.                                                */
/* ---------------------------------------------------------------------   */

If (parm > k) Then Do      /* parm given, known here to be numeric 00-06-26*/
  "MESSAGEBOX" myname": Illegal option detected: '"parm"'"  ,
    escape.1 || "NNot within range 1-"k", option must be one of:" ,
    escape.1 || "N"help  ,
    escape.1 || "Nor be numeric (line style choice)" ,
    escape.1 || "NPutting out window to choose line style."
  parm = ''                                /* force output of popup window */
End                                                            /* end else */

If (parm = '') Then Do

  'WINDOW' window_size '64' Length(alls)/11 myname || ': chose style'
  Do i = 1 To Length(alls) By 11
    temp = (i+10)/11
    'WINLINE' 'vert='     || bold || Substr(alls, i, 1)   || normal,
      'horiz='    || bold || Substr(alls, i+1, 1) || normal,
      'upr left=' || bold || Substr(alls, i+2, 1) || normal,
      '  ' emphasis || allsdesc.temp ,
      '\nSETRESULT' temp
  End                                  /* End do                           */
  result = 0
  'WINWAIT'
  'REFRESH'
  If (result = 0) Then Do
    Exit                                       /* exit if no line selected */
  End                                                            /* end if */

  parm = result
End                                                              /* end if */

/*-------------------------------------------------------------------------*/
/*  User has chosen line style, now see to drawing with that style         */
/*-------------------------------------------------------------------------*/

boxstyle= parm                         /* save line style                  */
Call set_stembox

/*-------------------------------------------------------------------------*/
/*  Now kick off the major loop in this macro. Read in one key press and   */
/*  react accordingly.                                                     */
/* ------------------------------------------------------------------------*/

origin = true                                  /* indicate first keystroke */

Call put_msg                                     /* show PF keys available */

Do always = 1            /* <- tantamount to a DO FOREVER, but contains ...*/
  /* ..a control variable for ITERATE etc.                                 */

  'EXTRACT /KEYPRESS/'

  Select

    /* box char chosen explicitly ?                                        */
    When (wordpos(keypress.1,box_keys) <> 0) Then Do
      Call box_char keypress.1
    End                                                        /* end when */

    When (wordpos(keypress.1,key_up) <> 0) Then Do
      parm = '!UP'
      Call process_move
      origin = false
    End                                                        /* end when */

    When (wordpos(keypress.1,key_right) <> 0) Then Do
      parm = '!RIGHT'
      Call process_move
      origin =  false
    End                                                        /* end when */

    When (wordpos(keypress.1,key_down) <> 0) Then Do
      parm = '!DOWN'
      Call process_move
      origin = false
    End                                                        /* end when */

    When (wordpos(keypress.1,key_left) <> 0) Then Do
      parm = '!LEFT'
      Call process_move
      origin = false
    End                                                        /* end when */

    When (wordpos(keypress.1,key_endis) <> 0) Then Do
      Call process_endis
      Call put_msg                   /* show the changed PF keys available */
    End                                                        /* end when */

    When (wordpos(keypress.1,key_newstyle) <> 0) Then Do
      Call process_newstyle
    End                                                        /* end when */

    When (wordpos(keypress.1,key_quit) <> 0) Then Do
      Call process_quit
    End                                                        /* end when */

    Otherwise Do
      'PRESSKEY' keypress.1                              /* pass to editor */
    End                                                   /* end otherwise */

  End                                                        /* end select */
  'REFRESH'
End                                                              /* end do */

/*-------------------------------------------------------------------------*/
/*  if PF12, then finish                                                   */
/* ------------------------------------------------------------------------*/

PROCESS_QUIT:

'PFLINE' x_pfline

'REFRESH'                              /* Ensure screen is redrawn         */
Exit                                                    /* exit this macro */

/*-------------------------------------------------------------------------*/
/*  subroutine to draw one character, i.e. move cursor 1 position          */
/*-------------------------------------------------------------------------*/

PROCESS_MOVE:

/*  if variable 'origin' is true,  we are at the origin of a line.         */

If (origin) Then Do

  Call set_stembox

  /* the next variable takes the values 'origin', 'yes' or 'no'            */
  checkreplace = 'ORIGIN'                     /* used in replacechar uprog */
  'EXTRACT /CURLINE/'
  Select
    When (parm='!UP') | (parm='!DOWN') Then Do
      char = Substr(alls, ((boxstyle-1)*11)+1, 1)
    End
    When (parm='!RIGHT') | (parm='!LEFT') Then Do
      char = Substr(alls, ((boxstyle-1)*11)+2, 1)
    End
  End                                  /* End select                       */

  If (parm = '!DOWN') & (down.boxstyle <> "") Then Do
    char = down.boxstyle
  End
  If (parm = '!LEFT') & (left.boxstyle <> "") Then Do
    char = Left.boxstyle
  End
  If (gaps.boxstyle <> "") Then Do
    If ( (cursor.1+cursor.2)//(gaps.boxstyle) = 0) Then Do
      char = gaps.boxstyle.parm
    End
  End

  char = replacechar( char )                   /* see to any intersections */

  /* line char has been chosen, insert it to current line                  */
  /* check boxstyle, just_cusor                                            */
  If (boxstyle <> just_cursor) Then Do
    curline.1 = Overlay(char, curline.1, cursor.2)
    'REPLACE' curline.1
  End                                                            /* end if */

  x_lastdir = parm                     /* save last direction              */

  Return
End

/*  Assume a normal movement, i.e. not first (origin) keystroke            */
/*-------------------------------------------------------------------------*/

direction = (wordpos(parm,'!UP !DOWN !LEFT !RIGHT'))
If (direction <> 0) Then Do

  lastdir =  x_lastdir                               /* get last direction */

  Call set_stembox

  checkreplace = 'YES'                        /* used in replacechar uprog */
  dirs2 = parm || lastdir                      /* current + last direction */
  If (parm = lastdir) Then Do               /* same direction as last time */
    Select
      When (parm='!UP') Then Do
        char = box._v
        If (cursor.1 = 1) Then Do
          'CURSOR 0' cursor.2
          'INPUT' Copies(' ', cursor.2)
          'CURSOR +1' cursor.2
        End                                                      /* end if */
        'CURSOR -1' cursor.2
      End
      When (parm='!DOWN') Then Do
        char = box._v
        'CURSOR +1'
        If (rc = 10) Then Do                          /* At bottom of file */
          'INPUT' Copies(' ', cursor.2)
          'CURSOR +1' cursor.2
        End                                       /* end Return code check */
      End
      When (parm='!RIGHT') Then Do
        char = box._h
        'CURSOR +0 +1'
      End
      When (parm='!LEFT') Then Do
        char = box._h
        'CURSOR +0 -1'
      End
    End                                /* End select                       */
  End
  Else Do                    /* changed directions, choose corner character*/
    Select
      When (dirs2 = '!UP!DOWN') | (dirs2 = '!DOWN!UP') Then Do
        char = box._v
      End
      When (dirs2 = '!LEFT!RIGHT') | (dirs2 = '!RIGHT!LEFT') Then Do
        char = box._h
      End
      When (dirs2 = '!UP!LEFT') | (dirs2 = '!RIGHT!DOWN') Then Do
        char = box._ll
        checkreplace = 'NO'                   /* used in replacechar uprog */
      End
      When (dirs2 = '!UP!RIGHT') | (dirs2 = '!LEFT!DOWN') Then Do
        char = box._lr
        checkreplace = 'NO'                   /* used in replacechar uprog */
      End
      When (dirs2 = '!LEFT!UP') | (dirs2 = '!DOWN!RIGHT') Then Do
        char = box._ur
        checkreplace = 'NO'                   /* used in replacechar uprog */
      End
      When (dirs2 = '!RIGHT!UP') | (dirs2 = '!DOWN!LEFT') Then Do
        char = box._ul
        checkreplace = 'NO'                   /* used in replacechar uprog */
      End
      Otherwise
    End                                /* End select                       */
  End

  'EXTRACT /CURLINE/'
  'EXTRACT /CURSOR/'

  If ((dirs2 = '!DOWN!DOWN') | (dirs2 = '!DOWN!UP')) ,
    & (down.boxstyle <> "") Then Do    /* char for downwards movement      */
    char = down.boxstyle
  End
  If ((dirs2 = '!LEFT!LEFT') | (dirs2 = '!LEFT!RIGHT')) ,
    & (left.boxstyle <> "") Then Do    /* char for moves to left           */
    char = Left.boxstyle
  End

  If (gaps.boxstyle <> "") Then Do     /* gaps in line wanted ?            */
    If ( (cursor.1+cursor.2)//(gaps.boxstyle) = 0) Then Do
      checkreplace = 'NO'                     /* used in replacechar uprog */
      char = gaps.boxstyle.parm
      If (char = "") Then char = " "
    End
  End

  char = replacechar( char )                /* see to any intersections    */

  /* character for drawing line has been chosen, insert to line            */

  If (boxstyle <> just_cursor) Then Do
    curline.1 = Overlay(char, curline.1, cursor.2)
    'REPLACE' curline.1
  End

  /* now save info for later calls                                         */

  x_lastdir = parm                                  /* save last direction */

  Return
End

Return

/* ----------------------------------------------------------------------- */
/* Subroutine to check if 2 lines cross                                    */
/* We set up a translate table for the possible intersections of various   */
/* box characters.  Then all we have to do is look up the target character */
/* and the existing character in the table to find out which character to  */
/* write on the screen.                                                    */
/* The variable checkreplace is used to decide what to do here:            */
/* 'ORIGIN'  the first keystroke for the line, check here but don't use    */
/*           the centre cross char etc.                                    */
/* 'NO'      don't check at all                                            */
/* 'YES'     check but we are now in 'normal' processing, i.e. not origin  */
/* ----------------------------------------------------------------------- */

REPLACECHAR:

Parse Arg newchar

linechar = Substr( curline.1, cursor.2, 1)            /* char in curr line */

If ((linechar = " ")  /*|(checkreplace = 'NO')*/) Then Do  /* rcs 00-09-26 */
  Return newchar       /* if blank in line to be overwritten, return input */
End                                                              /* end if */
/*                       'boxchar' now set in set_stembox                  */
/* boxchars = box._v box._h box._ul box._td box._ur box._tr,               */
/*                                                                         */
/*  box._tx box._tl box._ll box._tu box._lr     */        /* Box characters*/

/* If checkreplace = NO, then we are turning a corner, check if the next   */
/* character in the OLD direction is a blank, if it is, we can use the     */
/* current newchar, if not a blank, we use the usual algorithm for         */
/* intersections.  This is an improvement compared to the previous         */
/* coding here, but still not perfect, for example if a horizontal line    */
/* aready exists and we come from underneath and turn into the line; in    */
/* this case we would like a T but get an upper left corner.  In cases     */
/* like this, we can just move over the junction once or twice and         */
/* usually it will repair itself.                                          */

If (checkreplace = 'NO') Then Do                           /* rcs 00-09-26 */
  Select                                  /* select over the OLD direction */

    When (lastdir = '!UP') Then Do
      linenum = cursor.1 - 1
      colnum  = cursor.2
    End                                                        /* end when */

    When (lastdir = '!DOWN') Then Do
      linenum = cursor.1 + 1
      colnum  = cursor.2
    End                                                        /* end when */

    When (lastdir = '!RIGHT') Then Do
      linenum = cursor.1
      colnum  = cursor.2 + 1
    End                                                        /* end when */

    When (lastdir = '!LEFT') Then Do
      linenum = cursor.1
      colnum  = Max(cursor.2 -1, 1)
    End                                                        /* end when */

    Otherwise Do
      linenum = cursor.1
      colnum  = cursor.2
    End                                                   /* end otherwise */

  End                                                        /* end select */

  'EXTRACT /LINE' linenum'/'
  If (rc <> 0) Then Do
    lookaheadchar1 = ' '
  End                                             /* end Return code check */
  Else Do
    lookaheadchar1 = Substr(line.1,colnum,1)
  End                                                          /* end else */

  If (lookaheadchar1 = ' ') ,
    |   (wordpos(lookaheadchar1,boxchars) = 0) Then Do
    Return newchar
  End                                                            /* end if */
End                                                              /* end if */

otable.1 = box._v box._tx box._tr box._tx box._tl box._tr,
  box._tx box._tl box._tr box._tx box._tl                   /* Vertical bar*/
otable.2 = box._tx box._h box._td box._td box._td box._tx,
  box._tx box._tx box._tu box._tu box._tu                 /* Horizontal bar*/
otable.3 = box._tr box._td box._ul box._td box._td box._tr,
  box._tx box._tx box._tr box._tx box._tx              /* Upper left corner*/
otable.4 = box._tx box._td box._td box._td box._td box._td,
  box._tx box._tx box._tx box._tx box._tx         /* Downwards Tee (unused)*/
otable.5 = box._tl box._td box._td box._td box._ur box._tx,
  box._tx box._tl box._tx box._tx box._tl             /* Upper right corner*/
otable.6 = box._tr box._tx box._tr box._tx box._tx box._tr,
  box._tx box._tx box._tr box._tx box._tx             /* Right Tee (unused)*/
otable.7 = box._tx box._tx box._tx box._tx box._tx box._tx,
  box._tx box._tx box._tx box._tx box._tx          /* Centre cross (unused)*/
otable.8 = box._tl box._tl box._tx box._tx box._tl box._tx,
  box._tx box._tl box._tx box._tx box._tl              /* Left Tee (unused)*/
otable.9 = box._tr box._tu box._tr box._tx box._tx box._tr,
  box._tx box._tx box._ll box._tu box._tu              /* Lower left corner*/
otable.10= box._tx box._tu box._tx box._tx box._tx box._tx,
  box._tx box._tx box._tu box._tu box._tu           /* Upwards Tee (unused)*/
otable.11= box._tl box._tu box._tx box._tx box._tl box._tx,
  box._tx box._tl box._tu box._tu box._lr             /* Lower right corner*/

If (checkreplace = 'ORIGIN') Then Do
  If (parm = '!UP') Then Do
    otable.1 = box._v box._tu box._tl box._tx box._tl box._tr,
      box._tx box._tl box._tr box._tx box._tl               /* Vertical bar*/
  End
  If (parm = '!DOWN') Then Do
    otable.1 = box._v box._td box._tl box._tx box._tl box._tr,
      box._tx box._tl box._tr box._tx box._tl               /* Vertical bar*/
  End
  If (parm = '!RIGHT') Then Do
    otable.2 = box._tr box._h box._ul box._td box._td box._tr,
      box._tx box._tx box._tx box._tu box._tu               /* Vertical bar*/
  End
  If (parm = '!LEFT') Then Do
    otable.2 = box._tl box._h box._td box._td box._ur box._tx,
      box._tx box._tl box._tu box._tu box._lr             /* Horizontal bar*/
  End
End                                                            /* end else */

linecharpos = Wordpos(linechar,boxchars)

newcharpos  = Wordpos(newchar,boxchars)

If (newcharpos=0) Then Do           /* if char is not in boxchars, then... */
  Return newchar            /* ...it must be one of the gaps, up etc chars */
End                                                              /* end if */

If (linecharpos >0) Then Do
  newchar = Word(otable.newcharpos, linecharpos)
End                                                              /* end if */

Return newchar

/*-------------------------------------------------------------------------*/
/* A utility subprogram to set up a stem containing the box chars          */
/*-------------------------------------------------------------------------*/

SET_STEMBOX:

set     = (boxstyle-1)*11
box._v  = Substr(alls,set+1,1)         /* Vertical bar                     */
box._h  = Substr(alls,set+2,1)         /* Horizontal bar                   */
box._ul = Substr(alls,set+3,1)         /* Upper left corner                */
box._td = Substr(alls,set+4,1)         /* Top Tee junction(pointing down)  */
box._ur = Substr(alls,set+5,1)         /* Upper right corner               */
box._tr = Substr(alls,set+6,1)         /*Right Tee junction(pointing right)*/
box._tx = Substr(alls,set+7,1)         /* Centre cross character           */
box._tl = Substr(alls,set+8,1)         /* Left Tee junction(pointing left) */
box._ll = Substr(alls,set+9,1)         /* Lower left corner                */
box._tu = Substr(alls,set+10,1)        /* Bottom Tee junction (pointing up)*/
box._lr = Substr(alls,set+11,1)        /* Lower right corner               */

boxchars = box._v box._h box._ul box._td box._ur box._tr,
  box._tx box._tl box._ll box._tu box._lr                 /* Box characters*/

Return

/*-------------------------------------------------------------------------*/
/*  if PF11, then kick off choosing a new line style                       */
/* ------------------------------------------------------------------------*/

PROCESS_NEWSTYLE:

/* retrieve the PFKey settings that were in effect when this macro         */
/* was originally invoked                                                  */

'PFLINE' x_pfline

'REFRESH'                              /* Ensure screen is redrawn         */

''myname                               /* call this macro again (no parms!)*/
Exit

/*-------------------------------------------------------------------------*/
/*  if PF5, enable/disable (=endis) cursor keys for drawing                */
/* ------------------------------------------------------------------------*/

PROCESS_ENDIS:

If (cursor_enabled) Then Do
  Call set_disable
End                                                              /* end if */
Else Do
  Call set_enable
End                                                            /* end else */
Return

SET_ENABLE:
cursor_enabled = true
key_up     = key_up.en                               /* enable cursor keys */
key_right  = key_right.en
key_down   = key_down.en
key_left   = key_left.en
Return

SET_DISABLE:
cursor_enabled = false
key_up     = key_up.dis                             /* disable cursor keys */
key_right  = key_right.dis
key_down   = key_down.dis
key_left   = key_left.dis
Return

/* ----------------------------------------------------------------------- */
/* Utility subroutine to put out a prompt line showing the Pf keys         */
/* ----------------------------------------------------------------------- */

PUT_MSG:
msg_string = /*  myname */ ,
  'F1=' || '1B'x 'F2=' || '18'x 'F3=' || '19'x  'F4=' || '1a'x
If (cursor_enabled) Then Do          /* '1B'X etc. are up/down etc. arrows */
  msg_string = msg_string ,
    ' (' || '1B'x || ',' || '18'x ',' || '19'x  ',' || '1a'x || ,
    ')  F5=DisableCrs'
End                                                              /* end if */
Else Do
  msg_string = msg_string 'F5=EnableCrs'
End                                                            /* end else */

'PFLINE' Overlay( msg_string , Right('F11=NewStyle F12=Quit',80), 1)
'REFRESH'                                   /* force display of new pfline */

Return

/* ----------------------------------------------------------------------- */
/* Subprocedure to insert a box character to text being edited   s         */
/* ----------------------------------------------------------------------- */

BOX_CHAR:
Parse Arg key

temp = Wordpos(key, box_keys)
If (temp = 0) Then Do
  Return
End                                                              /* end if */

'EXTRACT /insert/'
If (insert.1) Then Do
  'INSMODE OFF'
  'KEYIN' Word(boxchars, temp)
  'INSMODE ON'
End                                                              /* end if */
Else Do
  'KEYIN' Word(boxchars, temp)
End                                                            /* end else */
'CURSOR +0 -1'

Return

/*boxchars = box._v box._h box._ul box._td box._ur box._tr,        */
/*  box._tx box._tl box._ll box._tu box._lr                                */

/* Debugging routines, remove later                                        */
/* -------------------------------                                         */

Novalue: Say " "
Say
Say '+++ Error in REXX exec' myname
Say '    Novalue condition from line' sigl
Say Sourceline(sigl)
Say '    variable:' Condition('D'); Trace ?r; Nop; Exit

Syntax:
Say
Say '+++ Error in REXX exec' myname
Say '    Syntax error no.' rc 'from line' sigl
Say '   'Errortext(rc)
Say Sourceline(sigl)
Say '    description:'Condition('D'); Trace ?r ; Nop

/* End of macro lines   ---------------------------------------------------*/
