/* MACX - Macro Expand - Display Assembler Macro in a Popup Window         */
/*                                                                         */
/* o  Place cursor on any row containing the name of an Assembler Macro    */
/*    which is defined elsewhere in the current file                       */
/* o  Issue MACX from an assigned key or from the command line             */
/* o  text of macro definition appears in a Popup Window.                  */
/*                                                                         */
/* Wesley Miller   WESLEY at RALVM29   pundit@vnet.ibm.com  (919) 254-9774 */
/*                                                                         */

'EXTRACT \CURLINE'       /* assume macro name is first word on current line*/
MacroName = word( curline.1 , 1 )

'EXTRACT \CURSOR'                      /* where is the cursor now?         */
SrcRow = cursor.1
SrcCol = cursor.2

'EXTRACT \FLSCREEN'   /* get the row number of the row at the top of screen*/
TopRow = flscreen.1

'TOP'                                  /* go to the top of the file        */
'/'macroname'/&/macro'         /* locate the macro definition for MacroName*/
LocateRC = rc                          /* remember the RC of the locate    */
if LocateRC <> 0                       /* if Not found                     */
   then do
      'MSG No macro definition for' MacroName 'found.'
      'CURSOR' TopRow - 1              /* put the screen near where it was */
      'PAGEDOWN'                     /* pagedown puts TopRow at TopOfScreen*/
      'CURSOR' SrcRow SrcCol           /* put the cursor back where it was */
      exit LocateRC                    /* and exit out with rc = locate rc */
    end                                /* end ... if rc <> 0   then ...    */

'EXTRACT \CURSOR'                      /* find out where it starts         */
MacTop = cursor.1

'/endm'                             /* find the end of the macro definition*/
'EXTRACT \CURSOR'                      /* remember where it ends           */
MacEnd = cursor.1

n = 0                                  /* display stem counter variable    */
MacLine.  = ''                         /* display stem                     */
MacLine.0 = 0                          /* with nothing in it               */
MaxMacLineLen = 1                 /* length of longest line in display stem*/

NumWidth = length( MacEnd )          /* how long is the longest row number?*/

do i = MacTop to MacEnd                /* for each line of macro definition*/
   'CURSOR' i                          /* ... cursor to it                 */
   'EXTRACT \CURLINE'                  /* ... extract it                   */
   n = n + 1                           /* ... add element to MacLine for it*/
   MacLine.n = right( i , NumWidth , '0' ) '|'  curline.1    /* write it in*/
   MaxMacLineLen = max( MaxMacLineLen , length( MacLine.n ))   /* How long?*/
 end                                   /* end ... do i = MacTop to MacEnd  */
MacLine.0 = n                          /* How many MacLines                */

'CURSOR' TopRow - 1                    /* put the screen near where it was */
'PAGEDOWN'                           /* pagedown puts TopRow at TopOfScreen*/
'CURSOR' SrcRow SrcCol                 /* put the cursor back where it was */

WinRows = min( 10 , MacLine.0 )        /* How many popup window rows?      */
WinWidth = min( 60 , MaxMacLineLen )   /* Popup rows <= 60 chars long      */

'WINDOW' WinRows WinWidth MacLine.0 'Expanded Macro:' MacroName
do i = 1 to MacLine.0                  /* fill it in                       */
   'WINLINE' MacLine.i '\nCURSOR +0 +0'/* CURSOR +0 +0 is No Op            */
 end                                   /* end ... do i = 1 to MacLine.0    */
