/******************************************************************/
/* Toby Thurston - 18 Nov 1997                                    */
/* Main driver for doing stats on numbers in a marked area        */
/******************************************************************/
'extract /mark/'; if mark.0 = 0 then do; 'msg Need a mark!'; exit; end
/******************************************************************/
/* read all the data                                              */
/******************************************************************/
data = ""
do i = mark.2 to mark.3
  'extract /line' i'/'                 /* read the line           */
  if mark.4 > 0                        /* if its a block mark     */
    then data = data substr(line.1,mark.4,mark.5-mark.4+1)
    else data = data line.1
end
data = onlynums(data)
if words(data) = 0 then do; 'msg No numbers found in marked area'; exit; end
/******************************************************************/
/* initialize all the counters                                    */
/******************************************************************/
n = 0; sum = 0; val. = 0; dp = 0; prod = 1; count. = 0; maxcount = 1
lo = 999999999; hi = -999999999
/******************************************************************/
/* lopo through the data                                          */
/******************************************************************/
do n = 1 to words(data)
  ? = word(data,n)
  if ? = "." | ? = "-"                /* treat . and - as missing */
    then do; val.n = 0; iterate; end
  val.n = ?                           /* ? must be numeric by now */
  count.? = count.? + 1                /* count for mode          */
  maxcount = max(maxcount,count.?)
  sum = sum + ?                        /* add up SUM              */
  prod = prod * ?                      /* make PRODUCT            */
  dp = max(dp,dp(?))                   /* adjust for max DPs      */
  lo = min(lo,?)                       /* track min               */
  hi = max(hi,?)                       /* and max values          */
end
n = n - 1            /* n is incremented once more at end of loop */
mean = sum/n                           /* Used for AVG and SDEV   */
/******************************************************************/
/* put in the desired descriptive quantity                        */
/******************************************************************/
'cursor data'                          /* move back to data       */
arg op
select
  when abbrev("SUM",op,1)     then signal sum
  when abbrev("AVG",op,1)     then signal avg
  when abbrev("MODE",op,2)    then signal mode
  when abbrev("COUNT",op,1)   then signal count
  when abbrev("RANGE",op,1)   then signal range
  when abbrev("MEDIAN",op,1)  then signal median
  when abbrev("SDEV",op,2)    then signal sdev
  when abbrev("PRODUCT",op,2) then signal product
  when op = "ALL" then do
   'input Range: '; 'cursor +0 +8'; call range
   'input Count: '; 'cursor +0 +8'; call count
   'input Sum:   '; 'cursor +0 +8'; call sum
   'input Prod:  '; 'cursor +0 +8'; call product
   'input Mean:  '; 'cursor +0 +8'; call avg
   'input Mode:  '; 'cursor +0 +8'; call mode
   'input Median:'; 'cursor +0 +8'; call median
   'input SDev:  '; 'cursor +0 +8'; call sdev
  end
otherwise
  'msg' op 'not implemented yet...'
end
exit
SUM:     'keyin' format(sum,,dp)             ; return
AVG:     'keyin' format(mean,,dp+1)          ; return
COUNT:   'keyin' n                           ; return
RANGE:   'keyin' lo"..."hi                   ; return
PRODUCT: 'keyin' format(prod,,dp)            ; return
MODE:
mode = ""
do i = 1 to n
  ? = val.i
  if count.? = maxcount then if wordpos(?,mode) = 0 then mode = mode format(?,,dp)
end;    'keyin' mode                      ; return
MEDIAN:
val.0 = -999999999                     /* first sort the values   */
do j = 2 to n
  i = j - 1
  if val.i > val.j then do
    key = val.j
    do until val.i <= key
      temp = i + 1 ; val.temp = val.i
      i = i - 1
    end
    temp = i + 1 ; val.temp = key
  end
end
if n // 2
  then do                              /* n was odd               */
    m = (n+1)/2                        /* find middle value       */
    'keyin' format(val.m,,dp)
  end
  else do                              /* n was even              */
    l = n/2                            /* find middle two values  */
    m = l + 1
    'keyin' format((val.l+val.m)/2,,dp+1)
  end
return
SDEV:
dev = 0
do i = 1 to n
  dev = dev + (val.i - mean) ** 2
end
sdev = sqrt(dev/(n-1))
'keyin' format(sdev,,dp+1)
return

dp: procedure                 /* count decimal places in a number */
 arg . "." ?
return length(?)

sqrt: procedure                        /* find square root        */
  arg x                                /* must be > 0             */
  if x = 0 then return 0
  sqrt = x/10
  do until sqrt = old
    old = sqrt
    sqrt = (sqrt + x/sqrt)*0.5
  end
return sqrt
                   /* This trick comes from "Programming in REXX" */
onlynums: procedure      /* Charles Daney, McGraw Hill 1992, p129 */
return space(translate(arg(1),,"/"delstr(xrange(),c2d("-")+1,13)))
