/******************************* NEWLINE.X **********************************/
/*                                                                          */
/* X2 macro to add a new line if INSERT mode is on, otherwise just cursor   */
/* to the next line. In either case place cursor under the first non-blank  */
/* character in previous line. Modify DEF file as follow:                   */
/*                                                                          */
/*  key enter = macro NEWLINE                                               */
/*                                                                          */
/* Written by Wing F. Yuen                               WFY316 at PKMFGVM1 */
/*                                                              21 Feb 1994 */
/*                                                                          */
/* Modified by Tom Brock to do the following:            TBROCK at BTVMANVM */
/*                                                              20 Jul 1995 */
/*                                                                          */
/*  1) Add a line to a comment box when enter is pressed while in a comment */
/*     box.  Turn INSERT mode off if on.                                    */
/*  2) If there is a quick comment without any code preceeding it, change   */
/*     the quick comment to a regular comment and align it to the right     */
/*     margin.                                                              */
/*  3) If INSERT mode is on, add a new line.                                */
/*  4) If INSERT mode is off, go to the next line unless we're              */
/*     on the last line, in which case add a new line.                      */
/*                                                                          */
/* Modifed by Wing Yuen to fix the following             WFY316 at PKMFGVM1 */
/*                                                              23 Aug 1995 */
/*                                                                          */
/*  1) removed check for Style2. This was already done by editing the       */
/*     values of "start_box", "blanks_before" etc.                          */
/*  2) added example of style 3                                             */
/*  3) accept quick comments entered as "//comment"                         */
/*                                                                          */
/*  Use in conjunction with cbox.x (creates the comment box).               */
/****************************************************************************/

/*+------------------------------------------------------------------------+
  |  Style 1:                                                              |
  |                                                                        |
  |    /*+---------------------------------+                               |
  |      |                                 |                               |
  |      +---------------------------------+*/                             |
  |                                                                        |
  |  Style 2:                                                              |
  |                                                                        |
  |     /************************************/                             |
  |     /*                                  */                             |
  |     /************************************/                             |
  |                                                                        |
  |  Style 3:                                                              |
  |                                                                        |
  |     /*Ŀ*/                             |
  |     /*                                */                             |
  |     /**/                             |
  |                                                                        |
  | Should be fairly easy to modify to fit your style of comment box if it |
  | is not one of the above.                                               |
  +------------------------------------------------------------------------+*/

/*+------------------------------------------------------------------------+
  | For style 1, set the following to '|  ' and '|  '.                     |
  | For style 2, set the following to '/* ' and ' */'.                     |
  | For style 3, set the following to '/*' and '*/'. (ASCII 0xB3)        |
  +------------------------------------------------------------------------+*/

start_box = '|  '                                                /* 3 chars */
end_box   = '|  '                                                /* 3 chars */

/*+------------------------------------------------------------------------+
  | The following will position the cursor where you want it on the new    |
  | line; the number of spaces to move right from the start_box.           |
  +------------------------------------------------------------------------+*/

move_over = 2

/*+------------------------------------------------------------------------+
  | Quick comment identifier.                                              |
  +------------------------------------------------------------------------+*/

quick_comment = '//'

c  = 'COMMAND'
cb = 'FALSE'                                            /* comment box flag */

c 'EXTRACT /INSERT/'
c 'EXTRACT /CURSOR/'
c 'EXTRACT /MARGINS/'
c 'EXTRACT /SIZE/'

save_line = CURSOR.1
save_col  = CURSOR.2

c 'EXTRACT /CURLINE/'

line = CURLINE.1

if line <> '' then do                              /* check for comment box */
  first_word = wordpos( start_box, line )

  if first_word = 1 then do                   /* yes we're in a comment box */
    cb = 'TRUE'
    start_col = wordindex( line, 1 )

    end_col = MARGINS.4

    /*+--------------------------------------------------------------------+
      | Calculate number of blanks the inserted line must contain.         |
      +--------------------------------------------------------------------+*/

    len = end_col - start_col - 5

    blanks = copies( ' ', len )

    if start_col > 1 then do
      indent = copies( ' ', start_col - 1 )
    end
    else do
      indent = ''
    end

    /*+--------------------------------------------------------------------+
      | Create the line to be inserted.                                    |
      +--------------------------------------------------------------------+*/

    ins_line = indent || start_box || blanks || end_box

    c 'INPUT' ins_line

    /*+--------------------------------------------------------------------+
      | Position the cursor on the new line.                               |
      +--------------------------------------------------------------------+*/

    col_num  = start_col + move_over
    line_num = save_line + 1

    c 'CURSOR' line_num col_num

    if INSERT.1 = 1 then do               /* if in insert mode, turn it off */
      c 'INSMODE OFF'
    end
  end
end

if cb = 'FALSE' then do                             /* not in a comment box */

  /*+----------------------------------------------------------------------+
    | Check for a quick comment.                                           |
    +----------------------------------------------------------------------+*/

  /* --- modified to detect "//comment" cases --- 895WFY */
  /* first_word = wordpos( quick_comment, line )         */
  /* if first_word = 1 then do                           */

  first_word = word( line, 1)
  If pos( quick_comment, first_word) = 1 then do   /* it is a quick comment */
    /*+--------------------------------------------------------------------+
      | Calculate the comment_suffix position.                             |
      +--------------------------------------------------------------------+*/

    end_col = MARGINS.4

    /*+--------------------------------------------------------------------+
      | Calculate the comment_prefix position.                             |
      +--------------------------------------------------------------------+*/

    line = strip( line )
    line = strip( line, L, '/' )

    /* ----- a blank before the comment looks better ----- 895WFY */
    If length( first_word) > 2 then line = ' ' || line

    len = length( line )

    start_col = end_col - len - 4                 /* change -2 to -4 895WFY */

    if start_col > 1 then do
      indent = copies( ' ', start_col - 1 )
    end
    else do
      indent = ''
    end

    /*+--------------------------------------------------------------------+
      | Format and insert the reformatted comment.                         |
      +--------------------------------------------------------------------+*/

    ins_line = indent || '/*' || line || ' */'

    last_line = SIZE.1

    c 'DELETE'

    if last_line <> save_line then do
      c 'UP'
      c 'INPUT' ins_line
      c 'DOWN'
    end
    else do
      c 'INPUT' ins_line
    end
  end
  else do

    /*+--------------------------------------------------------------------+
      | ENTER.                                                             |
      +--------------------------------------------------------------------+*/

    if INSERT.1 = 1 then do              /* in insert mode - add a new line */
      c 'INPUT'
    end
    else do                                           /* not in insert mode */

      /*+------------------------------------------------------------------+
        | If we're on the last line, insert a new line, otherwise just go  |
        | to the next line.                                                |
        +------------------------------------------------------------------+*/

      last_line = SIZE.1

      if last_line = save_line then do
        c 'INPUT'
      end
      else do
        c 'DOWN'
      end
    end
  end

  c 'CURSOR +0' COMPARE( ' ', CURLINE.1, ' ' )  /* return 1st mismatch col  */
end

Exit 0
