/***************************************************************************/
/*                                                                         */
/* PS.X                                                                    */
/*                                                                         */
/* Convert the current file into a PostScript format file, maintaining all */
/* colour highlights.  Requires version 1.94 (Beta B) or later of the X2   */
/* Editor.                                                                 */
/*                                                                         */
/* Once in PS format, you still have to get it to the printer.  If your    */
/* printer supports colour and PostScript all you have to do is to copy    */
/* the output from this program to the printer, and it'll handle the rest. */
/* If you're like me and you bought a printer that doesn't support PS :-(, */
/* you can still get your file to print.  I use the GhostScript            */
/* interpreter, which can be obtained from                                 */
/* http://www.cs.wisc.edu/~ghost/index.html                                */
/*                                                                         */
/* The generated PS code uses a Courier font.  A monospace font is         */
/* required to accurately reflect the editor view, even though it doesn't  */
/* look as nice as a proportional font.                                    */
/*                                                                         */
/* Written by Blair Thompson, April 2, 1998                                */
/* ---rgf, 2000-05-15: changed escape-mechanism ["\", "(", ")"]            */
/*                                                                         */
/***************************************************************************/

  font._height = 10
  font._width  = 6

  'EXTRACT /VERSION/'
  Parse Var Version.1 vernum .
  If vernum < 1.94
    Then Do
      'msg This macro requires at least X2 version 1.94'
      Exit
    End                                                           /* End do*/

  Call ps_header
  Call ps_body
Exit


/***************************************************************************/
/*                                                                         */
/* Set up the PostScript header information.  We define a few procs and    */
/* set the title, etc.                                                     */
/*                                                                         */
/***************************************************************************/

PS_HEADER:  Procedure Expose filename.1 psfile
  Parse Source os invoke sourcefn .
  'EXTRACT /FILENAME/'
  pext = Lastpos('.', filename.1)
  If pext > 0
    Then psfile = Substr(filename.1,1,pext-1)
    Else psfile = filename.1
   psfile = psfile || '.ps'

  'EDIT' psfile
  'TOP'
  'DELETE *'
  'MARGINS 1 1000 1 1000'

  'INPUT %!PS'
  'INPUT %%Title: PostScript conversion of' filename.1
  'INPUT %%Creator: X2 Editor, macro' sourcefn
  'INPUT %%CreationDate:'
  'CURSOR EOL STAY'
  'CURSOR +0 +1'
  'DATE LONG'

  'INPUT'
  'INPUT  /FS { %def'
  'INPUT    findfont exch scalefont setfont'
  'INPUT  } bind def'
  'INPUT  /MS {%def'
  'INPUT    moveto'
  'INPUT    show'
  'INPUT  } bind def'
  'INPUT  /SetColour {%def'
  'INPUT    aload pop setrgbcolor'
  'INPUT  } bind def'
  'INPUT'
  'INPUT  /Green        ' || '5b'x || '   0 128   0 ' || '5d'x || ' def'
  'INPUT  /Black        ' || '5b'x || '   0   0   0 ' || '5d'x || ' def'
  'INPUT  /Red          ' || '5b'x || ' 128   0   0 ' || '5d'x || ' def'
  'INPUT  /Blue         ' || '5b'x || '   0   0 128 ' || '5d'x || ' def'
  'INPUT  /Magenta      ' || '5b'x || ' 128   0 128 ' || '5d'x || ' def'
  'INPUT  /White        ' || '5b'x || ' 128 128 128 ' || '5d'x || ' def'
  'INPUT  /Cyan         ' || '5b'x || '   0 128 128 ' || '5d'x || ' def'
  'INPUT  /Yellow       ' || '5b'x || ' 128 128   0 ' || '5d'x || ' def'
  'INPUT %%EndProlog'
  'INPUT'
Return


/***************************************************************************/
/*                                                                         */
/* Process the file body, line by line, into the target PostScript file.   */
/* We keep track of the position on the page, and set a page eject when    */
/* appropriate.                                                            */
/*                                                                         */
/***************************************************************************/

PS_BODY:  Procedure Expose filename. psfile font.
  'INPUT ' font._height '/Courier FS'             /* Must be monospace font*/
  rowpos = 700                                               /* Top of page*/

  'EDIT' filename.1
  Call init_colours
  'TOP'
  Do Until rc <> 0
    Call ps_line
    rowpos = rowpos - font._height - 2
    If rowpos < 50
      Then Do                                         /* We need a new page*/
        'INPUT  showpage'
        'INPUT'
        rowpos = 700
      End                                                         /* End do*/
    'EDIT' filename.1
    'DOWN'
  End                                                             /* End do*/

  'EDIT' psfile
  'INPUT  showpage'
  'INPUT %%Trailer'
  'INPUT %%EOF'
Return


/***************************************************************************/
/*                                                                         */
/* Process the current file line.  We get a copy of the line and split it  */
/* into parts at the colour boundaries.  We can then output a PostScript   */
/* line for each colour segment.  Performance optimisations include        */
/* skipping blank line segments, and only changing colours if necessary.   */
/*                                                                         */
/***************************************************************************/

PS_LINE:  Procedure Expose psfile rowpos colour. font. lastcolour
  'EXTRACT /CURLINE/'
  'EXTRACT /LINECOLOUR/'
  'EDIT' psfile
  If curline.1 = ''
    Then Return                                    /* We can skip this line*/

  outcol = 72                                                /* Left margin*/
  Do i = 1 To linecolour.0
    Parse Var linecolour.i startcol endcol clr_index .
    line_seg = Substr(curline.1,startcol,endcol-startcol+1)
    seg_len = Length(line_seg)
    line_seg = Strip(line_seg,'T')
    If line_seg <> ''
      Then Do                                           /* We must print it*/
        If colour.clr_index <> lastcolour
          Then Do
            'INPUT ' colour.clr_index 'SetColour'
            lastcolour = colour.clr_index
          End                                                     /* End do*/
                /* escape chars with a special meaning for PS */
        if pos("\", line_seg) > 0 then line_seg = changestr("\", line_seg, "\\")
        if pos("(", line_seg) > 0 then line_seg = changestr("(", line_seg, "\(")
        if pos(")", line_seg) > 0 then line_seg = changestr(")", line_seg, "\)")

        'INPUT  (' || line_seg || ')' outcol rowpos 'MS'
      End                                                         /* End do*/
    outcol = outcol + (font._width * seg_len)
  End                                                             /* End do*/

Return


/***************************************************************************/
/*                                                                         */
/* Initialise our output colours, depending on the current editor colour   */
/* settings.  We're only interested in the foreground colour - we'll       */
/* assume that the background is white so we don't have to start printing  */
/* graphics.                                                               */
/*                                                                         */
/***************************************************************************/

INIT_COLOURS:  Procedure Expose colour.
  'EXTRACT /COLOURS/'
  colour. = 'Black'                                              /* Default*/
  Do clr_index = 1 To colours.0
    Parse Var colours.clr_index . fcolour ' ON ' .
    Select
      When fcolour = 'GREEN'
        Then colour.clr_index = 'Green'
      When fcolour = 'BLACK'
        Then colour.clr_index = 'Black'
      When fcolour = 'BLUE'
        Then colour.clr_index = 'Blue'
      When fcolour = 'BROWN'
        Then colour.clr_index = 'Yellow'
      When fcolour = 'CYAN'
        Then colour.clr_index = 'Cyan'
      When fcolour = 'DARK GREY'
        Then colour.clr_index = 'Black'
      When fcolour = 'GREEN'
        Then colour.clr_index = 'Green'
      When fcolour = 'LIGHT BLUE'
        Then colour.clr_index = 'Blue'
      When fcolour = 'LIGHT CYAN'
        Then colour.clr_index = 'Cyan'
      When fcolour = 'LIGHT GREEN'
        Then colour.clr_index = 'Green'
      When fcolour = 'LIGHT GREY'
        Then colour.clr_index = 'White'
      When fcolour = 'LIGHT MAGENTA'
        Then colour.clr_index = 'Magenta'
      When fcolour = 'LIGHT RED'
        Then colour.clr_index = 'Red'
      When fcolour = 'MAGENTA'
        Then colour.clr_index = 'Magenta'
      When fcolour = 'RED'
        Then colour.clr_index = 'Red'
      When fcolour = 'WHITE'
        Then colour.clr_index = 'White'
      When fcolour = 'YELLOW'
        Then colour.clr_index = 'Yellow'
      Otherwise Nop                                                /* Error*/
    End                                                       /* End select*/
  End
Return
