/*+--------------------------------------------------------------------------+*/
/*|  QE - Fancy Query of a extractable X2 variable                           |*/
/*|                                                                          |*/
/*|                author  Wesley Miller                                     |*/
/*|           Lotus Notes  Wesley Miller/Raleigh/IBM@IBMUS                   |*/
/*|              internet  wjmiller@us.ibm.com                               |*/
/*|             telephone  919-254-9774                                      |*/
/*+--------------------------------------------------------------------------+*/
/*|                              Change History                              |*/
/*|                                                                          |*/
/*| 98/05/27 - added CHAR/CURCHAR as special variables to extract the current|*/
/*|            character, i.e. the character at the cursor.  Added since X2  |*/
/*|            for AIX sometimes obscures the decoded character information  |*/
/*|            on the title line in order to show long filenames.            |*/
/*|                                                                          |*/
/*|            Also removed all box characters, added help, extract for      |*/
/*|            line n and impropved display a little.                        |*/
/*|                                                                          |*/
/*| 98/05/28 - fix to add EXIT after help text.  Thanks to Bill Halterman    |*/
/*| 98/09/11 - Two parameters for KEY setting (Blair Thompson)               |*/
/*| 99/12/08 - Two parameters for COLOUR setting (Blair Thompson)            |*/
/*|                                                                          |*/
/*+--------------------------------------------------------------------------+*/

   /*+-----------------------------------------------------------------------+*/
   /*|  GET VERSION INFORMATION                                              |*/
   /*|  o  this allows us to support newer features without breaking when    |*/
   /*|     using an older version of X2.                                     |*/
   /*+-----------------------------------------------------------------------+*/
   'EXTRACT \VERSION'                                   /* what version of X? */
   if   rc <> 0                                            /* extract failed? */
      then version.1 = 0.0                               /* use version = 0.0 */
   version.1 = word( version.1 , 1 )              /* delete "Beta" if present */
   If translate( version.1 ) = '1.77A'              /* special case for 1.77A */
      then version.1  = 1.77                                  /* drop the "A" */
   if  datatype( version.1 ) <> 'NUM'           /* version.1 is non-numeruic? */
      then version.1 = 0.0                               /* use version = 0.0 */

   /*+-----------------------------------------------------------------------+*/
   /*|  GET THE USERS ARGS                                                   |*/
   /*| o  if the first user arg is "LINE" check for a possible second arg    |*/
   /*| o  for a second arg, use it as the parm to EXTRACT \LINE n            |*/
   /*+-----------------------------------------------------------------------+*/
   Parse Arg args                                  /* whole text of user args */
   parse arg noun num .                          /* user args as we want them */

   /*+-----------------------------------------------------------------------+*/
   /*|  Extract the information                                              |*/
   /*|                                                                       |*/
   /*|  o  give the usual help stuff.                                        |*/
   /*|                                                                       |*/
   /*|  o  "line" is a special case.  Handle it separately.                  |*/
   /*|                                                                       |*/
   /*|  o  for X for AIX which doesn't always tell us the current character  |*/
   /*|     (filename is too long) create dummy extractables CURCHAR and CHAR |*/
   /*|     which display the current character.                              |*/
   /*|                                                                       |*/
   /*|  o  otherwise, just extract the variable.                             |*/
   /*|                                                                       |*/
   /*+-----------------------------------------------------------------------+*/
   select
      when   wordpos( noun , '? -? /? HELP -HELP /HELP' ) > 0 ,
             | noun = ''
         then do
         'WINDOW' 15 54 15  "Help for qe.x"
         'WINLINE                                                        \n nop'
         'WINLINE +----------------------------------------------------+ \n nop'
         'WINLINE | qe - query x2 extractables                         | \n nop'
         'WINLINE |                                                    | \n nop'
         'WINLINE | >>- qe ----- variable ------------------------<<   | \n nop'
         'WINLINE |                                                    | \n nop'
         'WINLINE | where: variable - is any "extract"-able x2         | \n nop'
         'WINLINE |                   variable or the special values   | \n nop'
         'WINLINE |                   "char" or "curchar" which return | \n nop'
         'WINLINE |                   the current character.           | \n nop'
         'WINLINE |                                                    | \n nop'
         'WINLINE |                   See the X2 help for a list of    | \n nop'
         'WINLINE |                   extractable variables.           | \n nop'
         'WINLINE |                                                    | \n nop'
         'WINLINE +----------------------------------------------------+ \n nop'
         exit
       end                                                     /* if HELP ... */

      when ( translate( noun ) = 'LINE' ) & ( num <> '' )          /* line xx */
         then 'extract \'noun num'\'
      When ( Translate( noun ) = 'KEY' ) & ( num <> '' )          /* key nn*/
         then 'extract \'noun num'\'
      When ( Translate( noun ) = 'COLOUR' ) & ( num <> '' )    /* colour nn*/
         then 'extract \'noun num'\'

      when   translate( noun ) = 'CURCHAR' ,                     /* for X4AIX */
           | translate( noun ) = 'CHAR'
         then do
            noun = 'CURCHAR'
            'extract \CURSOR'                         /* where is the cursor? */
            'extract \CURLINE'                        /* get the current line */
           curchar.0 = 3                        /* give char at cursor 3 ways */
           curchar.1 = substr( curline.1 , cursor.2 , 1 )        /* character */
           curchar.2 = c2x( curchar.1 ) 'hex'                          /* hex */
           curchar.3 = c2d( curchar.1 ) 'decimal'                  /* decimal */
           rc = 0                                       /* we were successful */
          end                                               /* if    then ... */

      otherwise
         'extract \'noun                              /* extract the variable */
    end                                                             /* select */

   /*+-----------------------------------------------------------------------+*/
   /*|  Count the returned lines and record the return code from the extract |*/
   /*+-----------------------------------------------------------------------+*/
   exrc = rc                                               /* rc from extract */
   returned_items = value( noun'.0' )            /* number of items extracted */

   /*+-----------------------------------------------------------------------+*/
   /*|  Prefix the lines with the name of the variable it shows.  Then make  |*/
   /*|  all the lines the same length.                                       |*/
   /*+-----------------------------------------------------------------------+*/
   maxlen = 0                              /* default length for longest line */
   if exrc = 0                                    /* if extract was valid ... */
      then do
         lineCnt = 0                                    /* count the WINLINES */
         do i = 0 to returned_items                 /* make a line of TEXT for*/
            lineCnt = lineCnt + 1                       /* each returned item */
            text.lineCnt =  noun'.'i ' = "'value( noun'.'i )'"'  /* form line */
            maxlen = max( maxlen , length( text.lineCnt )) /* take its length */
          end                                                           /* do */
         maxlen = max( maxlen , 40 )                   /* at least 40 columns */
         if version.1 < 1.81
            then maxlen = min( maxlen , 78 )              /* mpt more than 78 */
       end                                            /* if exrc = 0 then ... */
      else do                                                    /* exrc <> 0 */
         maxlen = 40                                            /* 40 columns */
         lineCnt = 1                                           /* one WINLINE */
         text.1 = 'Extract failed.'                             /* error text */
       end                                            /* if exrc = 0 ... else */

   /*+-----------------------------------------------------------------------+*/
   /*|  Add a box around the messages and show them.                         |*/
   /*+-----------------------------------------------------------------------+*/
   winwidth = maxlen + 4                             /* starting window width */
   if winwidth < 44
      then winwidth = 44
      else if winwidth > 75
              then winwidth = 75

   'WINDOW' lineCnt+2 winwidth lineCnt+2  'EXTRACT "'args'" RESULTS'/* make window */
   'WINLINE' '+-'copies("-",maxlen)'-+\nCURSOR +0 +0'          /* top WINLINE */
   do k = 1 to lineCnt                     /* show each of the built WINLINEs */
      text.k = left( text.k , maxlen )             /* padded to MAXLEN length */
      'WINLINE' '|' text.k '|\nCURSOR +0 +0'                  /* and boxed in */
    end                                                                 /* do */

   /*+-----------------------------------------------------------------------+*/
   /*|  Draw the last winline and add instruction text in the border.        |*/
   /*+-----------------------------------------------------------------------+*/
   lastLine = '+-'copies("-",maxlen)'-+\nCURSOR +0 +0'        /* last WINLINE */
   lastline = overlay( " Esc/Enter to close ", lastline, 22 )
   if maxlen + 4 > winwidth
      then lastline = overlay( " more > ", lastline, 64 )
   'WINLINE' lastline                                              /* show it */

