/*
   ---rgf,2000-06-13, Vienna
      match any character to the right (alt-F6) or to the left (alt-F5), optionally
      marking them (shift-F6 and shift-F5, respectively);
      two characters located consecutively are regarded as escapes and hence ignored;
      examples for single, double quotes given on the bottom
*/


'EXTRACT /CURSOR/'              /* get cursor positions                                         */
'EXTRACT /CURLINE/'             /* get current line                                             */
'EXTRACT /WRAP/'                /* get wrap settings                                            */
'EXTRACT /MSGMODE/'             /* get msgmode setting                                          */

'WRAP OFF'                      /* take wrap off by default                                     */
'MSGMODE OFF'                   /* set messagemode off                                          */

parse arg type "," dir "," mark

if dir = 'LEFT' then dir = '-'
                else dir = ''

if mark = 'MARK' then mark = 1
                 else mark = 0

if type = "CHAR" then
do
   char = SUBSTR(curline.1, cursor.2, 1)/* uses any char cursor is on at the moment             */
   call find_next_char char, dir, cursor.1 cursor.2, mark/* char, direction, row col, mark result*/
   If result = -1 then          /* no simple matching found                                     */
     'CURSOR' cursor.1 cursor.2 /* restore cursor, might have been moved in subroutine          */
end
else if type = "XMLTAG" then    /* use XML-tags                                                 */
do
   nop                          /* not implemented as of yet                                    */
end

'MSGMODE' msgmode.1             /* restore msgmode setting                                      */
'WRAP' wrap.1                   /* restore wrap setting                                         */

exit


/* escaping is assumed if "char" is duplicated without intervening other char */
find_next_char : procedure
   parse arg string, dir, start_row start_col, bMark

   If dir = '-' then col_incr = "-1"
                else col_incr = "+1"

   If pos('/', string) > 0 Then deli = '09'x/* how about a TAB-char as delimiter in case a slash itself is to be located*/
                           Else deli = '/'      /* default to slash as delimiter                */
   case = 'e'                   /* 'e' ... exact case, 'c' ... ignore case                      */

   'L' deli || string || deli dir case  /* issue locate command, so "REPFIND" starts to work    */

   If rc <> 0 then return -1    /* not found, or something went wrong, abort ...                */

   Do forever
      'REPFIND'                 /* repeat locate command                                        */
      If rc <> 0 then return -1 /* not found, or something went wrong, abort ...                */
      'EXTRACT /CURSOR/'        /* extract new cursor position                                  */

      bFound= ((cursor.2 + col_incr) < 1)/* on left dir, if on column 1, no need to check further to the left*/
      If \bFound Then
      do
         'EXTRACT /CURLINE/'    /* extract current line                                         */
         bFound = (substr(curline.1, cursor.2+col_incr, 1) <> string)   /* string not escaped?  */
      end

      If bFound then
      do
        If bMark then
        Do
          'MARK CLEAR'          /* clear mark                                                   */
          'CURSOR' start_row start_col
          'MARK BLOCK'          /* should be: 'MARK CHAR', once it gets implemented             */
          'CURSOR' cursor.1 cursor.2
          'MARK BLOCK'          /* should be: 'MARK CHAR', once it gets implemented             */
          return 1              /* indicate marking took place                                  */
        End
        return 0                /* indicate no marking took place                               */
      end
      else
      Do
         'CURSOR' cursor.1 col_incr     /* move cursor one column                               */
      End
   end


/* testdata:

   "aha""aha"
"aha"

   'max ''und moritz'
'max ''''''"und" """moritz"""'

   'aha '''' und moritz'
*/
