/******************************************************************/
/* sen.x -- cut & paste whole sentences                           */
/******************************************************************/
/* Usage:  put the cursor in middle of the sentence to be         */
/* processed then do                                              */
/*      sen c  == copy the sentence                               */
/*      sen d  == delete the sentence                             */
/*      sen x  == cut the sentence (copy & delete)                */
/*                                                                */
/* Once you have copied a sentence (c or x) you can do            */
/*      sen v  == insert the sentence after the sentence that     */
/*                your cursor is currently in.                    */
/*      sen p  == (same as sen v)                                 */
/*                                                                */
/* To see what you have copied look at the file in the ring       */
/* call "Sentence".  If you delete by accident, use "UNDO" (F9).  */
/*                                                                */
/* Restrictons:  You can only paste once, due to a feature of     */
/* "get".  Also I haven't tested what happens with hidden lines.  */
/*                                                                */
/*                                    Toby Thurston - 28 Jul 1997 */
/* Updates ********************************************************/
/* TET 19970728 c/reflow/reformat/** -- use correct cmd name      */
/******************************************************************/
'extract /cursor/'                     /* where are we now?       */
startpos = cursor_pos()                /* save startpos           */
'extract /curline/'                    /* what are we pointing at */
if curline.2 = 0                       /* not visible             */
  then do;'msg' prog() 'doesn''t work on hidden lines';signal finis;end
else if curline.1=""                   /* blank                   */
  then do; 'msg Put your cursor in the sentence you want to process';signal finis;end
else if row(startpos) = 0              /* TOF                     */
  then do; 'msg Put your cursor in the text';signal finis;end
/******************************************************************/
/* OK to proceed now....                                          */
/******************************************************************/
call top_para                       /* go to the top of this para */
sos = cursor_pos()                    /* save pos of Start of Sen */
'next_sentence'                        /* go to start of next sen */
eos = check_eos(cursor_pos() - 1)      /* save end of sen pos     */
do while eos < startpos                /* find the right sentence */
  sos = check_sos(eos + 1)
  'next_sentence'
  eos = check_eos(cursor_pos() - 1)
end
/******************************************************************/
/* now we know where the sentence starts and ends                 */
/******************************************************************/
mode = choose(arg(1)="","C",translate(arg(1))) /* default to copy */
select
  when mode = "C" then do              /* copy it                 */
    call copy_sentence
    'msg Sentence copied.  Use "'prog() 'v" to insert it else where.'
  end
  when mode = "D" then do              /* delete it               */
    call delete_sentence
    'msg Sentence deleted.'
  end
  when mode = "X" then do              /* cut it                  */
    call copy_sentence
    call delete_sentence
    'msg Sentence cut.  Use "'prog() 'v" to insert it else where.'
  end
  when pos(mode,"VP") > 0 then do      /* paste it                */
    'extract /ring/'     /* check something is available to paste */
    found = 0
    do i = 1 to ring.0
      if filespec("N",ring.i) = "Sentence" then do; found = 1; leave i; end
    end
    if found=0 then do;'msg Use "'prog() 'x" or "c" to cut or copy a sentence first';signal finis;end
    'cursor' cursor.1 cursor.2         /* move to start pos       */
    'next_sentence'                    /* find next sentence      */
    'split'                            /* split the line          */
    'get Sentence'                     /* insert the sentence     */
    'reformat'                         /* tidy up                 */
  end
otherwise
  'msg I don''t understand' mode".  Try X cut V paste C copy"
end
finis:
'cursor data'                  /* put the cursor back in the text */
'cursor' cursor.1 cursor.2             /* move to start pos       */
'refresh'                              /* dismiss the cmd dialog  */
exit
/******************************************************************/
/* End of main line                                               */
/******************************************************************/
check_sos: procedure     /* check that sos is not on a blank line */
  arg p
  if col(p) = 0 then p = p + 1
  do while row_is_blank(p)
    p = p + 1000
  end
return p

check_eos: procedure                   /* ditto for eos           */
  arg p
  if col(p) = 0 then p = p - 1
  do while row_is_blank(p)
    p = p - 1000
  end
return p

row_is_blank: procedure                /* is current row blank?   */
  arg p
  'extract /line' row(p)'/'
return line.1 = ""

top_para:                           /* go to the top of this para */
  'previous_para'                     /* back up to previous para */
  'next_para'                          /* down to top of this     */
  'cursor +0 1'                        /* cursor to COL1          */
  if cursor_pos() > startpos           /* we must have been in p1 */
    then 'cursor 1 1'
return

cursor_pos: procedure           /* save cursor as a single number */
  'extract /cursor/'
return cursor.1*1000 + cursor.2

row: return arg(1)%1000   /* return cursor row from single number */
col: return arg(1)//1000  /* return cursor col from single number */

copy_sentence: procedure expose sos eos
  'extract /line' row(sos)'/'
  if row(eos) = row(sos)           /* sentence is all on one line */
    then sentence = space(substr(line.1,col(sos),col(eos)-col(sos)))
    else do                            /* sentence spans lines    */
      sentence = substr(line.1,col(sos))
      do i = row(sos) + 1 to row(eos) - 1
        'extract /line' i'/'
        sentence = sentence line.1
      end
      'extract /line' row(eos)'/'
      sentence = space(sentence substr(line.1,1,col(eos)))
    end
  'x Sentence'                    /* save the text in a temp file */
  'top'
  'delete *'
  'input' sentence
  'alt 0 0'
  'previous_file'
return

delete_sentence: procedure expose sos eos
  'cursor' row(sos) 1                  /* go to first line of sen */
  'extract /line/'                     /* read the line           */
  pref = substr(line.1,1,col(sos)-1)   /* save any prefix         */
  if row(eos) > row(sos)               /* if more than one line   */
    then do
      'delete' row(eos)-row(sos)/* delete to just before last line */
      'extract /line/'                 /* read it                 */
    end
  'delete' ; if rc = 0 then 'up'  /* remove it & go up if not EOF */
  'input' pref""substr(line.1,col(eos)+1)  /* put in pref & suffix */
  'reformat'                           /* tidy up                 */
return

prog: procedure
  parse source . . program
  parse value filespec("N",program) with program "." .
return program
choose: return arg(arg()-arg(1))/* A TT trick (C) Toby Thurston 1996 */
