/******************************** SKELETON *********************************/
/*                         X2 Editor Macro SKELTON                         */
/*                provide a SKELTON for a new editor macro                 */
/*                                                                         */
/* This macro has been designed to speed up the process of coding new REXX */
/* programs and/or X editor macros by providing a complete skelton. This   */
/* skeleton includes routines for handling syntax and novalue conditions,  */
/* putting out header block comments such as these as a help text etc.     */
/*                                                                         */
/* Call:                                                                   */
/*                                                                         */
/*      1. SKELETON ?                                                      */
/*                                                                         */
/*      puts out these comments as a help text (/?  and -?  are also       */
/*      accepted for Unix etc.  compatibility).  ?  is the only input      */
/*      parameter (argument) to this macro.                                */
/*                                                                         */
/*      2. XWNT newfile.x /Cskeleton                                       */
/*                                                                         */
/*      if editing a file and this macro is called or invoked from the     */
/*      editor invocation line, this macro checks to see if the file is    */
/*      empty (number of lines is zero or 1) ) and then generates the      */
/*      skeleton.                                                          */
/*                                                                         */
/*      3. while editing a file, from the invocation line. Again this      */
/*      macro checks to see if the file is empty (0 or 1 line).            */
/*                                                                         */
/* Please note that the skeleton provided by this macro reflects very much */
/* the programming style of its author.  You will almost certainly want to */
/* modify this macro to your programming style.  This macro works by       */
/* copying the first part of itself (up to and including the first         */
/* MESSAGEBOX below (the Browse mode check) into the text about to be      */
/* edited.  You can therefore modify this macro itself to your programming */
/* style and you automatically modify the skeleton produced by this macro. */
/* Note that this macro adapts its own code to adjust to the current right */
/* margins setting, but if the right margins setting is less than 72       */
/* certain parts of the comments may be overwritten by the end of comment  */
/* characters.  The copied code does not include this block comment header */
/* you are reading now (several empty comment lines are provided in its    */
/* place though) and 2 debugging routines (for Novalue and Syntax).        */
/*                                                                         */
/* Please note that this macro generates a check to see if the text being  */
/* edited is in browse mode and if that is the case an error message is    */
/* issued.  If you are writing a macro that does not change the text being */
/* edited you may wish to remove this check.                               */
/*                                                                         */
/* The inline comments, i.e. those with text centred between asterisks     */
/* (see 'Get the name of this macro') may be transformed to another        */
/* comment style, please check the customisation section below.  You       */
/* should also replace 'RCS' as author, insert your own initials or name.  */
/*                                                                         */
/* Author:  Roger C. Spencer  (V2SPENCE at SDFVM2)                         */
/*                                                                         */
/* Customisation section: You may wish to change the following variables.  */
/*                                                                         */

author = 'RCS'                         /* inserted into block header ...   */
                                       /* 'change activity '               */

datestyle = 'H'                        /* date style for 'change           */
                                       /* activity'.  All arguments for    */
                                       /* REXX's date() bif are allowed    */
                                       /* and 'H' (Hyphen) for YY-MM-DD    */

cursor_position = 'CODE'               /* where to position the cursor..   */
                                       /* in the generated skeleton.       */
                                       /* Allowed values are:              */
                                       /* 'CODE' in the executable code    */
                                       /* 'COMMENT' in the block header..  */
                                       /*           comments               */

comment_style = '1'                    /* how to format the inline         */
                                       /* comments (such as 'Get the name  */
                                       /* of this macro' below).  '1'      */
                                       /* keeps comments as coded in this  */
                                       /* macro.  Other values are shown   */
                                       /* in the examples below.           */

comment_pad = '='                      /* used to generate inline comments */
                                       /* together with comment_style. See */
                                       /* examples below. May be set to    */
                                       /* blank but must be 1 char long.   */

/*========================== comment_style = '2' ==========================*/

/* comment_style = '3' =================================================== */

/* comment_style = '4', 2 lines                                            */
/* ============================                                            */

/* comment_style = '5', 2 lines                                            */
/* ======================================================================= */

/***************************************************************************/
/* Change activity                                                         */
/* 00-02-18 RCS (=Roger C.Spencer) new.                                    */
/* 00-03-08 RCS corrected picking up this macro's and text file's names    */
/***************************************************************************/

Signal Endhelpline                     /* end of help text                 */
ENDHELPLINE: lasthelpline = sigl -1

/*-------------------------------------------------------------------------*/
/* N O T E !  keep the above two lines as they are used to pick up         */
/* the number of the last line of help text to put out to a window.        */
/*-------------------------------------------------------------------------*/

Signal On NOVALUE; Signal On SYNTAX

parse source os . myname .             /* get this macro's name            */

false = 0; true = 1                    /* Boolean values                   */

"EXTRACT /FILEINFO/"

"EXTRACT /ESCAPE/"

/*********************** Get the name of this macro ************************/

/* Define in the variable 'filespec_os' the names of systems as            */
/* returned by 'EXTRACT /OS/' that support the filespec() function.        */
/* Note that the use of the filespec() function is a known system          */
/* dependency in this macro, however, it is protected by checking          */
/* the system we are running under. If filespec is not supported           */
/* we don't do anything, but we keep the IF branch for future use.         */

filespec_os = 'OS/2 WINDOWS NT WINDOWS 95'

'EXTRACT /OS/'
If (Wordpos(Translate(os.1), filespec_os) = 0) Then Do
  myname = myname                      /* <--- ffu, filespec not supported */

End                                    /* end if                           */
Else Do
  myname = Translate(Filespec('Name', myname))
  Parse Var myname myname '.' .        /* drop extension                   */
End                                    /* end else                         */

/************************** check if help wanted ***************************/

Arg parm                /* assume opts to be passed to editor at invocation*/

'EXTRACT /FLSCREEN/'                   /* get no. of lines on screen       */
If (flscreen.0 <> 2)
  Then window_size = 18                /* set up no. of lines...           */
  Else Do                              /* ...for pop-up windows            */
    window_size = (flscreen.2 - flscreen.1 + 1)- 4
  End

help = '? /? -? HELP'                  /* Help text wanted ?      .        */
If (Wordpos(parm, help) <>0) Then Do
  window_width = Max(Length(Sourceline(1)) ,        /* line 1 may be empty */
                   , Length(Sourceline(2)), Length(Sourceline(3)))
  'WINDOW' window_size window_width lasthelpline myname 'Help text'
  Do i = 1 To lasthelpline-1           /* make sure loop ends sometime     */
    'WINLINE' Sourceline(i) || '\nREFRESH'
  End
  Exit 0
End

/********************* check if we are in browse mode **********************/

If (fileinfo.5 = 'BROWSE') Then Do
  "MESSAGEBOX" myname": In Browse mode, exiting"
  Exit 0
End                                    /* end if                           */

/******************* remember number of last line above ********************/
/*********** this will be the last line to be copied to the text ***********/

Signal Last_line; LAST_LINE: lastoutline = sigl-4

/****************** check if it's a new file (i.e. empty) ******************/

If (fileinfo.2 >= 2) Then Do

  'MESSAGEBOX' fileinfo.2 'lines already in file'escape.1'nEnter' ,
    ' "A" to abort, anything else continues'

  If (Translate(result) = "A") Then Do
    Exit 0
  End                                  /* end if                           */

End                                    /* end if                           */

/***************************** check settings ******************************/

cursor_position = Translate(cursor_position)   /* make sure it's uppercase */
cursor_valid    ='CODE COMMENT'
If (Wordpos(cursor_position,cursor_valid) = 0) Then Do
  'MESSAGEBOX Illegal value for cursor_position:' cursor_position ,
    || escape.1'nnot one of' cursor_valid   ,
    || escape.1'nReset to "CODE"'
  cursor_position = 'CODE'
End                                    /* end if                           */

If (Length(comment_pad)<> 1) Then Do
  'MESSAGEBOX comment_pad not 1 character long >'comment_pad'<' ,
    || escape.1'nReset to "="'
  comment_pad = '='
End                                    /* end if                           */

comment_style = Translate(comment_style)       /* make sure it's uppercase */
comment_valid = '1 2 3 4 5'
If (Wordpos(comment_style,comment_valid) = 0) Then Do
  'MESSAGEBOX Illegal value for comment_style:' comment_style   ,
    || escape.1'nnot one of' comment_valid ,
    || escape.1'nReset to "1"'
  comment_style   = '1'
End                                    /* end if                           */

/************************ the real work starts here ************************/

cs = '/' || '*';  ce = '*' || '/'      /* comment start and end            */

lines_written = 0

Parse Var fileinfo.12 . . lmargin rmargin .

/* generate   2 header comment lines                                       */

macname = fileinfo.1                   /* get name of new macro file       */
If (Wordpos(Translate(os.1), filespec_os) = 0) Then Do
  macname = macname                    /* <--- ffu, filespec not supported */

End                                    /* end if                           */
Else Do
  macname = Translate(Filespec('Name', macname))
  Parse Var macname macname '.' .      /* drop extension                   */
End                                    /* end else                         */

l = Overlay( ce, cs, rmargin-1, , '*')
l = Overlay( ' 'macname' ', l, Trunc((Length(l)-Length(macname)+2)/2))
Call Inp l

l = Centre('X2 Editor Macro' macname, rmargin)
l = Overlay(cs,l,1)
l = Overlay(ce,l,rmargin-1)
Call Inp l

ecl = Overlay( ce, cs, rmargin-1)      /* empty comment line               */

Call Inp ecl

l = Overlay(macname':', ecl, 4)
Call Inp l

/* The folowing 2 lines define a position at the start of the block header */
/* that you may want to position the cursor in place of the current        */
/* position (to empty lines in the REXX code after all this setup).        */
If (cursor_position = 'COMMENT') Then Do
  cursor_to_here_c = Length(myname)+6  /* cursor positioned here...        */
  cursor_to_here_l = lines_written     /* ...later                         */
End                                    /* end if                           */

/* generate 10 comment lines and 2 'change activity' lines                 */

Do 10
  Call Inp ecl
End                                    /* end do                           */

Call Inp  Overlay('Change Activity', ecl, 4)
If (datestyle= 'H') Then Do
  Call Inp Overlay(Translate(Date('o'), '-', '/') author 'New', ecl,4)
End                                    /* end when                         */
Else Do
  Call Inp ,
    Overlay(Translate(Date(datestyle), '-', '/') author 'new', ecl, 4)
End                                    /* end else                         */

Call Inp Overlay( ce, cs, rmargin-1, , '*')

/* loop through this macro and push its lines into the edit text           */

Do i = lasthelpline To lastoutline

  l = Sourceline(i)

  If (Left(Strip(l,'L'),2) = cs) Then Do       /* comment line             */
    l = fmtcomm(Sourceline(i))         /* go format comment                */
  End                                  /* end if                           */

  Call Inp l

End                                    /* end do                           */

/* generate a header and some empty lines to play with                     */

l = Overlay( ce, cs, rmargin-1, , '*')
text = ' Real work starts here '
l = Overlay( text , l ,
  ,  Trunc((Length(l)-Length(text)+2)/2))
Call Inp fmtcomm(l)

If (cursor_position = 'CODE') Then Do  /* position cursor in code          */
  cursor_to_here_l = lines_written +1 + fileinfo.2
  cursor_to_here_c = 1
End                                    /* end if                           */

Do 15
  Call Inp ' '                         /* a series of empty lines          */
End                                    /* end do                           */

/* generate the exit statement                                             */

l = Overlay( ce, cs, rmargin-1, , '*')
text = ' Final Exit statement '
l = Overlay( text , l ,
  ,  Trunc((Length(l)-Length(text)+2)/2))
Call Inp fmtcomm(l)
Call Inp ' '                           /* blank line                       */
Call Inp 'Exit 0'

/* generate the debugging routines                                         */

Call Inp ' '                           /* empty line                       */
l = Overlay( ce, cs, rmargin-1, , '*')
text = ' Debugging Routines '
l = Overlay( text , l,
  ,  Trunc((Length(l)-Length(text)+2)/2))
Call Inp fmtcomm(l)                    /* possibly reformat it             */
Call Inp ' '                           /* empty line                       */

Call Inp  'NOVALUE:'
Call Inp  "'MESSAGEBOX +++ Error in REXX macro' myname '+++' ,"
Call Inp  "  escape.1'nNovalue condition from line' sigl ,"
Call Inp  "  escape.1'n'Sourceline(sigl) ,"
Call Inp  "  escape.1'nVariable:' Condition('D')"
Call Inp  'Trace ?r; nop'
Call Inp  'Exit'
Call Inp ' '                           /* empty line                       */
Call Inp  'SYNTAX:'
Call Inp  "'MESSAGEBOX +++ Error in REXX macro' myname '+++' ,"
Call Inp  "  escape.1'nSyntax error no.' rc 'from line' sigl ,"
Call Inp  "  escape.1'n'Errortext(rc) ,"
Call Inp  "  escape.1'n'Sourceline(sigl) ,"
Call Inp  "  escape.1'nDescription:'Condition('D')"
Call Inp  'Trace ?r; nop'
Call Inp  'Exit'

/* generate the final comment                                              */

Call Inp ' '                           /* blank line                       */
l = Overlay( ce, cs, rmargin-1, , '*')
text = ' End of Macro' macname' '
l = Overlay( text , l ,
  ,  Trunc((Length(l)-Length(text)+2)/2))
Call Inp fmtcomm(l)

/**************** run through all lines to align comments *****************/

'TOP'
'DOWN *'

/*************************** now exit this macro ***************************/

'CURSOR' cursor_to_here_l cursor_to_here_c

Exit 0

/************ subprocedure to (re)format an inline coment line *************/

FMTCOMM:

Parse Arg l

Select

  When (Left(l,4) <> '/' || '***') Then Do     /* do not change            */
    l = Delstr(l,Length(l)-1)          /* drop comment end chars           */
    Return  ,                          /* ...comment (not inline)          */
      Left(Overlay(ce, l ,rmargin-1,,Right(l,1)), rmargin)
  End                                  /* end when                         */

  When (comment_style = '1') Then Do
    l = Substr(l,3, Length(l) -4)
    l = Centre(l, rmargin, '*')
    l = Overlay(cs, l , 1)
    Return ,
      Left(Overlay(ce, l ,rmargin-1), rmargin)
  End                                  /* end when                         */

  When (comment_style = '2') Then Do
    l = Translate(Substr(l,3, Length(l) -4), comment_pad, '*')
    l = Centre(l, rmargin, comment_pad)
    l = Overlay(cs, l , 1)
    Return ,
      Left(Overlay(ce, l ,rmargin-1), rmargin)
  End                                  /* end when                         */

  When (comment_style = '3') Then Do
    l =  Strip(l, 'Both', '/')         /* strip comment chars              */
    l =  Strip(l, 'Both', '*')         /* strip comment chars              */
    Return  ,
      Left(Overlay(' 'ce, cs l ,rmargin-2, , comment_pad), rmargin)
  End                                  /* end when                         */

  When (comment_style = '4') Then Do   /* 2 lines of output                */
    l =  Strip(l, 'Both', '/')         /* strip comment chars              */
    l =  Strip(l, 'Both', '*')         /* strip comment chars              */
    l =  Strip(l, 'Both', ' ')         /* strip blanks                     */
    Call Inp  ,                        /* put out first line               */
      Overlay(ce, cs l ,rmargin-1)
    Return ,                           /* return second line               */
      Overlay(ce, cs Copies(comment_pad,Length(l)), rmargin-1)
  End                                  /* end when                         */

  When (comment_style = '5') Then Do   /* 2 lines of output                */
    l =  Strip(l, 'Both', '/')         /* strip comment chars              */
    l =  Strip(l, 'Both', '*')         /* strip comment chars              */
    l =  Strip(l, 'Both', ' ')         /* strip blanks                     */
    Call Inp  ,                        /* put out first line               */
      Overlay(ce, cs l ,rmargin-1)
    Return ,                           /* return second line               */
      Overlay(' 'ce, cs Copies(comment_pad,rmargin-3), rmargin-2)
  End                                  /* end when                         */

  Otherwise Do                         /* ??, already checked              */
    'MESSAGEBOX uprog fcmtcomm: illegal comment_style:' comment_style
    Return l
  End

End

/***************** subprocedure to insert one line to text *****************/

INP:

Parse Arg inp_me
'INPUT' inp_me
If (rc) Then Do
  'MESSAGEBOX Return code' rc 'from INPUT command, aborting'
  Exit 8
End                                    /* end if                           */
lines_written = lines_written + 1
Return

/******************** Debugging routines, remove later *********************/

NOVALUE:
'MESSAGEBOX +++ Error in REXX macro' myname '+++' ,
  escape.1'n    Novalue condition from line' sigl ,
  escape.1'n'Sourceline(sigl) ,
  escape.1'n    variable:' Condition('D')
Trace ?r
Nop
Exit

SYNTAX:
'MESSAGEBOX +++ Error in REXX macro' myname '+++' ,
  escape.1'n    Syntax error no.' rc 'from line' sigl ,
  escape.1'n'    Errortext(rc) ,
  escape.1'n'Sourceline(sigl) ,
  escape.1'n    description:'Condition('D')
Trace ?r
Nop

/************************** End of macro SKELETON **************************/
