/***************************************************************************/
/*                                                                         */
/* Calculate the total number of lines in the edit environment.  We reduce */
/* the line count of each file by the number of blank lines found.  This   */
/* macro is useful to get a quick estimate of the total lines of code in   */
/* an application.                                                         */
/*                                                                         */
/* Written by Blair Thompson, January 20, 1996                             */
/*                                                                         */
/***************************************************************************/

  'EXTRACT /NAME/'
  curfile  = NAME.1
  totsize  = 0
  totfiles = 0

  Do Until NAME.1 = curfile
    Call check_file                           /* Add this file's parameters*/
    totfiles = totfiles + 1
    'NEXT'
    'EXTRACT /NAME/'
  End                                                             /* End do*/

  'MSG' totsize 'non-blank lines found in' totfiles 'file(s)'
Exit


/***************************************************************************/
/*                                                                         */
/* Calculate the number of non-blank lines in the current file in the      */
/* ring.  We do this by using the COUNT command to get the total number of */
/* blanks, then subtract this from the total file size.  We're             */
/* non-destructive to the file, but we do lose any marked area or hidden   */
/* lines.                                                                  */
/*                                                                         */
/***************************************************************************/

CHECK_FILE:  Procedure Expose totsize
  'EXTRACT /SIZE/'
  'ALL'                             /* Make sure we can see the entire file*/
  'BOTTOM'
  'CURSOR +0 1'                                      /* Position the cursor*/
  'MARK CLEAR'                                /* Make sure there is no mark*/
  'MARK BLOCK'                                    /* Start a new block mark*/
  'TOP'
  'MARK BLOCK'                                 /* Mark column 1 of the file*/
  'COUNT ?? LM'                                    /* How many blank lines?*/
  totblanks = rc
  If totblanks < 0
    Then totblanks = 0                                  /* We had an error!*/
  totsize = totsize + SIZE.1 - totblanks
  'MARK CLEAR'
Return
