/***************************************************************************/
/*                                                                         */
/* XEAINFO.CMD                                                             */
/*                                                                         */
/* Edit the X2 editor EA information file.                                 */
/*                                                                         */
/* Written by Blair Thompson, November 5, 1999                             */
/*                                                                         */
/***************************************************************************/

  Parse Source os invoke sourcefn .
  Parse Arg opt .

  Select
    When Address() <> 'X'
      Then Call init
    When opt = '$SHOW'
      Then Call macro_setup
    When opt = '$SAVE'
      Then Call macro_save
    Otherwise Nop
  End                                                         /* End select*/
Exit


/***************************************************************************/
/*                                                                         */
/* Look for the EA file by appending its name to the XPATH.  Give up if we */
/* can't find it.  If we've got it, create a few more useful filenames.    */
/*                                                                         */
/***************************************************************************/

GET_EAFILE:  Procedure Expose eafile tmpfile bakfile
  xpath = Value('X2PATH',,'ENVIRONMENT')
  If xpath = ''
    Then xpath = Value('XPATH',,'ENVIRONMENT')
  eafile = xpath'\XEAINFO.DTA'

  If Stream(eafile,'C','QUERY EXISTS') = ''
    Then Do
      Say 'File' eafile 'not found!'
      Exit
    End                                                           /* End do*/

  pdot = Lastpos('.',eafile)
  If pdot > 0
    Then Do
      tmpfile = Substr(eafile,1,pdot) || 'tmp'
      bakfile = Substr(eafile,1,pdot) || 'bak'
    End                                                           /* End do*/
    Else Do
      tmpfile = eafile || '.tmp'
      bakfile = eafile || '.bak'
    End
Return                                        /* End of get_eafile function*/


/***************************************************************************/
/*                                                                         */
/* Initial invocation, i.e. from the command line.  We read & parse the EA */
/* file, and create a temporary file for the user to edit.  All we're      */
/* expecting the user to do is delete the entries they no longer want!     */
/*                                                                         */
/* We use the editor return code to determine whether they want to save    */
/* the changes they made.  If they want to save the changes we've already  */
/* got most of the data in memory, so we don't have to re-read the EA      */
/* file.                                                                   */
/*                                                                         */
/***************************************************************************/

INIT:  Procedure Expose sourcefn
  Call get_eafile

  size_offset = 25                              /* Offset to structure size*/
  pad = 'fd'x                                              /* Pad character*/

  If Stream(tmpfile,'C','QUERY EXISTS') <> ''
    Then Do                           /* We already have the temporary file*/
      Say tmpfile 'already exists on disk.  Ok to replace it?'
      Parse Upper Pull resp .
      If resp = '' | Abbrev('YES',resp) = 0
        Then Exit
      '@DEL' tmpfile '>NUL'
    End                                                           /* End do*/

  numEntries = charin(eafile,,4)                             /* Header info*/
  numEntries = c2d(reverse(numEntries))
  data = charin(eafile,,chars(eafile))                      /* Get the rest*/
  Call stream eafile,'C','CLOSE'                        /* Close input file*/

  fn. = ''                                                  /* The filename*/
  filedata. = ''                                         /* Associated data*/
  Do entryNum = 1 While data <> ""      /* Now process all the data entries*/
    pNull = Pos('00'x,data)
    If pNull = 0
      Then Do                               /* It should be null delimited!*/
        Say 'Error, unable to find end of filename'
        Exit
      End                                                         /* End do*/

    fn.entryNum = Substr(data,1,pNull-1)
    If Stream(fn.entryNum,'C','QUERY EXISTS') = ''
      Then prefix = '===>'
      Else prefix = '    '
    data = Substr(data,pNull+1)
    Do While Substr(data,1,1) = pad             /* Remove alignment padding*/
      data = Substr(data,2)
    End                                                           /* End do*/

    structsize = c2d(reverse(substr(data,size_offset,2)))
    filedata.entryNum = Substr(data,1,structsize)
    data = Substr(data,structsize+1)

    Call Lineout tmpfile, Left(prefix fn.entryNum,256) Right(entryNum,5,'0')
  End                                                             /* End do*/

  Call stream tmpfile,'C','CLOSE'                      /* Close output file*/
  'x' tmpfile '"-c'sourcefn '$SHOW"'
  If rc <> 322
    Then Say 'No EA data changed'
    Else Call change_data
Return                                              /* End of Init function*/


/***************************************************************************/
/*                                                                         */
/* Ok, so the user wants to save their changes.  All we'll be reading is   */
/* the tag numbers of the entries that are left in the file - we assume    */
/* that everything else is to be trashed.  We only write the entries that  */
/* haven't been deleted.                                                   */
/*                                                                         */
/***************************************************************************/


CHANGE_DATA:  Procedure Expose eafile tmpfile bakfile fn. filedata. pad
  keepit. = 0
  numEntries = 1
  Do While Lines(tmpfile)                       /* Let's see what's changed*/
    aline = Linein(tmpfile)
    Parse Var aline . 256 entryNum .
    entryNum = entryNum + 0                        /* Remove leading zeroes*/
    keepit.entryNum = 1
    numEntries = numEntries + 1
  End                                                             /* End do*/
  Call stream tmpfile,'C','CLOSE'                       /* Close input file*/

  '@del' tmpfile '>NUL'
  numEntries = Right(d2c(numEntries),4,'00'x)
  Call charout tmpfile,reverse(numEntries)
  Do entryNum = 1 While fn.entryNum <> ''
    If keepit.entryNum = 0
      Then Iterate
    Call charout tmpfile,fn.entryNum
    Call charout tmpfile,'00'x
    padbytes = 4 - ((length(fn.entryNum) + 1) // 4)          /* DWORD align*/
    If padbytes < 4
      Then Call charout tmpfile,Left('',padbytes,pad)
    Call charout tmpfile,filedata.entryNum
  End                                                             /* End do*/

  Call stream tmpfile,'C','CLOSE'                      /* Close output file*/

  '@COPY' eafile bakfile '>NUL'
  Say eafile 'has been backed up to' bakfile
  '@COPY' tmpfile eafile '>NUL'
  Say eafile 'has been re-written with your changes'
  '@DEL' tmpfile '>NUL'
Return                                       /* End of Change_data function*/


/***************************************************************************/
/*                                                                         */
/* We've started an editor session for the user to look at their EA data   */
/* in a manageable format.  Set a key and so inform the unsuspecting user. */
/*                                                                         */
/***************************************************************************/

MACRO_SETUP:  Procedure Expose sourcefn
  'KEY f4 = macro' sourcefn '$SAVE'
  'cursor 1 1'

  'extract /escape/'
  nl = escape.1'N'
  msg = 'X2 EA Information' || nl || nl
  msg = msg' This file contains all files from the saved EA Info Data. 'nl
  msg = msg' Lines highlighted with ===> were not found on disk. 'nl
  msg = msg' Delete any lines no longer required, then press F4  'nl
  msg = msg' to save the changes.'
  'MESSAGEBOX' msg
Return                                       /* End of Macro_setup function*/


/***************************************************************************/
/*                                                                         */
/* So they want to save their changes, do they?  Set the RC for the        */
/* calling program and cross your fingers.                                 */
/*                                                                         */
/***************************************************************************/

MACRO_SAVE: Procedure
  'exitrc 322'
  'ea off'
  'file'
Return                                        /* End of Macro_save function*/
